﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using Fpsware.Library3.Composites;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using Misc = Fpsware.Library3.Helpers.Misc;

namespace Fpsware.Botbases.RAFBot2
{
    internal static class Grinder
    {
        #region Local Declarations

        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private const int MinFreeBagSlots = 2;
        private static bool IsBagsFull = false;
        public delegate T Selection<out T>(object context);
        
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        #endregion

        private static bool IsPoiType(this PoiType poiType) { return BotPoi.Current != null && BotPoi.Current.Type == poiType; }

        public static Composite CreateGrinderBehavior()
        {
            return new Decorator(ret => Setting.Instance.UseGrinder.ResultOK(),
                new PrioritySelector(

                    // Maintain our action trigger timers. If grinder was enabled then reset the timers
                    new Decorator(ret => !RAFBot2.IsActionTriggersTimeReset, new Action(context => { RAFBot2.IsActionTriggersTimeReset = true; GrinderActionTrigger.ActionTriggersList().ResetActionTriggerTimers(); })),

                    //DoTradeSkill(ret=> "Knothide Armor Kit"),

                    // Action triggers
                    new ThrottlePasses(1,2,
                        new Decorator(ret=> GrinderActionTrigger.IsTriggerConditionMet && !Me.Combat, 
                            new PrioritySelector(
                                new Decorator(ret => GrinderActionTrigger.RequiredAction == "TABvAGcAbwB1AHQA".Base64Decode(), Logout()),
                                new Decorator(ret => GrinderActionTrigger.RequiredAction == "UgBlAHQAdQByAG4AIABIAG8AbQBlAA==".Base64Decode(), ReturnHome()),
                                new Decorator(ret => GrinderActionTrigger.RequiredAction == "VQBzAGUAIABIAGUAYQByAHQAaABzAHQAbwBuAGUA".Base64Decode(), UseHearthstone())
                        ))
                    ),

                    #region Heal / Eat / Drink

                    // Heal
                    new Decorator(ret => !Me.HasAura("Food") && !Me.HasAura("Drink"), RoutineManager.Current.HealBehavior),

                    // Rest
                    new Decorator(ret => !Me.Combat, RoutineManager.Current.RestBehavior),

                    // Eating or Drinking then stop here
                    new Decorator(ret => Me.HasAura("Food") || Me.HasAura("Drink"), new ActionAlwaysSucceed()),

                    #endregion

                    // Encounter Logic behavior
                    new Decorator(ret => Setting.Instance.UseEncounterLogic.OK() && !Misc.IsUserMoving && EncounterLogicActions.IsEncounterConditionMet,
                        new PrioritySelector(

                            EncounterLogicActions.CreateEncounterLogicActions(),

                            // If we're not moving at least face the target
                            new Decorator(ret => !Me.IsMoving && Me.GotTarget && !Me.IsSafelyFacing(CT), new Action(context => CT.Face())),

                            // While the encounter condition is in effect don't go any further
                            new ActionAlwaysSucceed()
                            )
                        ),

                    // Combat
                    new Decorator(ret => Me.IsActuallyInCombat && !IsPoiType(PoiType.Kill),  new Action(ret => { BotPoi.Current = new BotPoi(PoiType.Kill); })),

                    // Full bags reported, check if still full
                    new ThrottlePasses(1,5, new Decorator(ret => IsBagsFull, new Action(context => { if (Me.FreeNormalBagSlots > MinFreeBagSlots) IsBagsFull = false; })) ),

                    #region Stack inventory and auto delete items

                    // Every 10 seconds stack items in your bags
                    new ThrottlePasses(1, 10, new Decorator(ret => !Me.Combat, new Action(context => { while (Lua.GetReturnVal<int>(Helpers.Helpers.StackLua, 0) != 1) { } }))),

                    // Every few seconds destroy our trash
                    new ThrottlePasses(1, 1,
                        new Decorator(ret => !Me.Combat && !Misc.IsUserMoving && !string.IsNullOrEmpty(Setting.Instance.AutoDeleteItems),
                            new Action(context =>
                            {
                                try
                                {
                                    string[] autoDeleteInventory = Setting.Instance.AutoDeleteItems.Split('|');
                                    WoWItem deleteItem = Me.CarriedItems.FirstOrDefault(i => autoDeleteInventory.Contains(i.Name));

                                    if (deleteItem != null)
                                    {
                                        Log.Info(string.Format("Destroying {0}", deleteItem.Name));
                                        deleteItem.PickUp();
                                        Lua.DoString("DeleteCursorItem()");
                                    }

                                }
                                catch (Exception e)
                                {
                                    Log.FailLog("=== shit! error during auto delete");
                                }

                            }))),

                    #endregion

                    #region Harvesting and Looting

                    // Something to loot? Set the POI for it.
                    new Decorator( ret => Setting.Instance.LootingMobs.OK() && !Me.Combat && Me.FreeNormalBagSlots >= MinFreeBagSlots && !IsPoiType(PoiType.Harvest) && !IsPoiType(PoiType.Skin) && !IsPoiType(PoiType.Loot) && !IsPoiType(PoiType.Kill) && Loot.LootableWoWUnit != null,
                        new Action(ret => { BotPoi.Current = new BotPoi(Loot.LootableWoWUnit, PoiType.Loot); })),

                    // Something to Harvest? Set the POI for it.
                    new Decorator( ret => Setting.Instance.LootingHarvest.OK() && !Me.Combat && Me.FreeNormalBagSlots >= MinFreeBagSlots && !IsPoiType(PoiType.Harvest) && !IsPoiType(PoiType.Skin) && !IsPoiType(PoiType.Loot) && !IsPoiType(PoiType.Kill) && Harvestable.LootableWoWGameObject != null,
                        new Action(
                            ret => { BotPoi.Current = new BotPoi(Harvestable.LootableWoWGameObject, PoiType.Harvest); })),

                    // Something to skin? Set the POI for it.
                    new Decorator( ret => (Setting.Instance.LootingSkinMobs.OK()) && !Me.Combat && Me.FreeNormalBagSlots >= MinFreeBagSlots && !IsPoiType(PoiType.Harvest) && !IsPoiType(PoiType.Skin) && !IsPoiType(PoiType.Loot) && !IsPoiType(PoiType.Kill) && Loot.SkinnableWoWUnit != null,
                        new Action(ret => { BotPoi.Current = new BotPoi(Loot.SkinnableWoWUnit, PoiType.Skin); })),

                    #endregion

                    // Ensure BotPot = NONE if nothing else is happening
                    new Decorator(ret => BotPoi.Current == null,  new Action(ret => { BotPoi.Current = new BotPoi(PoiType.None); })),

                    // Switch: Decide what we should be doing if nothing else is happening
                    new Decorator(ret => BotPoi.Current != null,
                        new Switch<PoiType>(ret => BotPoi.Current.Type,

                            // Nothing decided yet so lets make it happen
                            new SwitchArgument<PoiType>(PoiType.None,
                                new PrioritySelector(

                                    // Skin
                                    new Decorator(ret => Setting.Instance.LootingSkinMobs.OK() && !Me.Combat && Me.FreeNormalBagSlots <= MinFreeBagSlots && Loot.SkinnableWoWUnit != null,
                                        new Action(ret => BotPoi.Current = new BotPoi(PoiType.Skin))),

                                    // Harvest
                                    new Decorator(ret => Setting.Instance.LootingHarvest.OK() && !Me.Combat && Me.FreeNormalBagSlots <= MinFreeBagSlots && Harvestable.LootableWoWGameObject != null,
                                        new Action(ret => BotPoi.Current = new BotPoi(PoiType.Harvest))),

                                    // Loot?
                                    new Decorator(ret => Setting.Instance.LootingMobs.OK() && !Me.Combat && Me.FreeNormalBagSlots <= MinFreeBagSlots && Loot.LootableWoWUnit != null,
                                        new Action(ret => BotPoi.Current = new BotPoi(PoiType.Loot))),

                                    // Combat / Pull
                                    new Decorator(ret => Me.Combat || !Me.Combat && Me.GotValidTarget(),
                                        new Action(ret => BotPoi.Current = new BotPoi(PoiType.Kill))),
                                        
                                    // Find something to pull or move to a different hotspot
                                    new Decorator(ret => !Me.Combat && (!Me.GotTarget || !CT.IsValidTarget()),
                                        new Action(ret => BotPoi.Current = new BotPoi(PoiType.Hotspot)))

                            )),


                            // We have a target to kill
                            new SwitchArgument<PoiType>(PoiType.Kill,
                                new PrioritySelector(

                                #region PreCombat Behavior
                                new Decorator(ret => !Me.Combat && Me.GotValidTarget(), RoutineManager.Current.PreCombatBuffBehavior),

                                #endregion

                                // Is there a more suitable mob closer than our current target?
                                // We have a target, but make sure there is nothing suitable closer than what we've selected
                                new Decorator(ret => Me.GotValidTarget() && !Me.Combat && !CT.Combat && !Me.IsCasting && ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Any(u => !u.IsFriendly && !u.IsPlayer && !u.IsTagged && u.Entry == CT.Entry && u.Distance < CT.Distance && !Blacklist.Contains(u, BlacklistFlags.All)),
                                    new Action(context => ObjectManager.GetObjectsOfType<WoWUnit>(true, false).FirstOrDefault(u => !u.IsTagged && GrinderHelpers.GrinderMobsList().Any(m => m.ID == u.Entry) && u.Distance < CT.Distance && !Blacklist.Contains(u, BlacklistFlags.All)).Target())),
            
                                    #region Pull Behavior
                
                                    // if we have a valid target but are not yet in combat
                                    new Decorator(ret => !Me.Combat && Me.GotValidTarget() && (Setting.Instance.LootingMobs.OK() && Loot.LootableWoWUnit == null || Setting.Instance.LootingSkinMobs.OK() && Loot.SkinnableWoWUnit == null),
                                        new PrioritySelector(

                                            new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Pulling target"; })),

                                            // Are we flying? Dismount
                                            new Decorator(ret => Me.IsFlying && CT.Distance < 40, new Movement.LandAndDismount("UAB1AGwAbABpAG4AZwAsACAAYgB1AHQAIAB3AGUAJwByAGUAIABmAGwAeQBpAG4AZwAuACAAVwBlACAAcwBoAG8AdQBsAGQAIABiAGUAIABvAG4AIAB0AGgAZQAgAGcAcgBvAHUAbgBkAA==".Base64Decode())),
                                            new Decorator(ret => Me.IsFlying && CT.Distance > 40, new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Flightor, ret => Movement.MountStrategyType.None, ret => false)),

                                            new Decorator(ret => !Misc.IsUserMoving, Alive.CreateCharacterMovement()),
                
                                            // Do CC pull buffs and actual pull behavior
                                            RoutineManager.Current.PullBuffBehavior,
                                            RoutineManager.Current.PullBehavior

                                    )),
            
                                    #endregion
                                    
                                    // Make sure we have a valid target from our POI
                                    //new Decorator(ret => (!Me.GotTarget || !CT.IsValidTarget()) && BotPoi.Current != null && BotPoi.Current.AsObject.ToUnit() != null && BotPoi.Current.AsObject.ToUnit().IsValid && BotPoi.Current.AsObject.ToUnit().IsValidTarget(),
                                    new Decorator(ret => (!Me.GotTarget || Me.GotTarget && !CT.IsValidTarget()) && BotPoi.Current != null && BotPoi.Current.AsObject != null && BotPoi.Current.AsObject.ToUnit() != null && BotPoi.Current.AsObject.ToUnit().IsValid && BotPoi.Current.AsObject.ToUnit().IsValidTarget(),
                                        new Action(context => BotPoi.Current.AsObject.ToUnit().Target())),

                                    // Assist out pet
                                    new Decorator(ret => (!Me.GotTarget || Me.GotTarget && !CT.IsValidTarget()) && Me.GotAlivePet && Me.Pet.GotTarget && Me.Pet.CurrentTarget.IsValidTarget(), new Action(ret => BotPoi.Current = new BotPoi(Me.Pet.CurrentTarget, PoiType.Kill))),

                                    // Are we flying? Dismount
                                    new Decorator(ret => Me.IsFlying, new Movement.LandAndDismount("SQBuACAAYwBvAG0AYgBhAHQALAAgAGIAdQB0ACAAdwBlACcAcgBlACAAZgBsAHkAaQBuAGcALgAgAFcAZQAgAHMAaABvAHUAbABkACAAYgBlACAAbwBuACAAdABoAGUAIABnAHIAbwB1AG4AZAA=".Base64Decode())),

                                    // Do combat behavior
                                    new Decorator(ret => Me.Combat, CreateGrinderCombatBehavior()),

                                    // Nothing left to kill then clear our POI
                                    new Decorator(ret => (BotPoi.Current != null && BotPoi.Current.AsObject != null && BotPoi.Current.AsObject.ToUnit() != null && BotPoi.Current.AsObject.IsValid && !BotPoi.Current.AsObject.ToUnit().IsValidTarget()) || (BotPoi.Current == null), new Action(ret => BotPoi.Current = new BotPoi(PoiType.None))),
                                    new Decorator(ret => !Me.Combat && (!Me.GotTarget || !CT.IsValidTarget()), new Action(ret => BotPoi.Current = new BotPoi(PoiType.None)))
                            )),
                            

                            // Loot something
                            new SwitchArgument<PoiType>(PoiType.Loot,
                                new PrioritySelector(

                                    // Nothing left to loot? Clear the POI
                                    new Decorator(ret => Loot.LootableWoWUnit == null, new Action(ret => BotPoi.Current = new BotPoi(PoiType.None))),

                                    // Full bags? Display a warning message.
                                    new Decorator(ret => Me.FreeNormalBagSlots <= MinFreeBagSlots,
                                        new ThrottlePasses(1, 5,
                                            new Action(c =>
                                            {
                                                Log.FailLog("TwB1AHIAIABiAGEAZwBzACAAYQByAGUAIABmAHUAbABsAC4AIABXAGkAbABsACAAbgBvACAAbABvAG4AZwBlAHIAIABsAG8AbwB0ACAAbQBvAGIAcwA=".Base64Decode());
                                                BotPoi.Current = new BotPoi(PoiType.None);
                                                IsBagsFull = true;
                                            }))),

                                    

                                    // Lets get to looting
                                    new Decorator(ret => Me.FreeNormalBagSlots >= MinFreeBagSlots,
                                        new PrioritySelector(
                                            new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Looting ..."; })),
                                            new Loot.QuickLootingBehaviorPS(ret => Loot.LootableWoWUnit, ret => Setting.Instance.LootingSkinMobs.OK()),
                                            new ActionAlwaysSucceed()
                                            ))
                            )),



                            // Skin something
                            new SwitchArgument<PoiType>(PoiType.Skin,
                                new PrioritySelector(

                                    // Nothing left to skin? Clear the POI
                                    new Decorator(ret => Loot.SkinnableWoWUnit == null, new Action(ret => BotPoi.Current = new BotPoi(PoiType.None))),

                                    // Full bags? Display a warning message.
                                    new Decorator(ret => Inventory.SkinningBagSlotsFree <= MinFreeBagSlots,
                                        new ThrottlePasses(1, 5,
                                            new Action(c =>
                                            {
                                                Log.FailLog("TwB1AHIAIABiAGEAZwBzACAAYQByAGUAIABmAHUAbABsAC4AIABXAGkAbABsACAAbgBvACAAbABvAG4AZwBlAHIAIABzAGsAaQBuACAAbQBvAGIAcwA=".Base64Decode());
                                                BotPoi.Current = new BotPoi(PoiType.None);
                                                IsBagsFull = true;
                                            }))),


                                    // Lets get to skinning
                                    new Decorator(ret => Inventory.SkinningBagSlotsFree >= MinFreeBagSlots && Loot.SkinnableWoWUnit.CanSkin,
                                        new PrioritySelector(
                                            new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Skinning ..."; })),
                                            new Decorator(ret => Me.IsFlying, new Movement.LandAndDismount("UwBrAGkAbgBuAGkAbgBnACwAIABiAHUAdAAgAHcAZQAnAHIAZQAgAGYAbAB5AGkAbgBnAC4AIABXAGUAIABzAGgAbwB1AGwAZAAgAGIAZQAgAG8AbgAgAHQAaABlACAAZwByAG8AdQBuAGQA".Base64Decode())),
                                            new Loot.QuickLootingBehaviorPS(ret => Loot.SkinnableWoWUnit, ret => Setting.Instance.LootingSkinMobs.OK()),
                                            new ActionAlwaysSucceed()
                                            ))
                            )),

                            // Harvest something
                            new SwitchArgument<PoiType>(PoiType.Harvest,
                                new PrioritySelector(

                                    // Nothing left to harvest? Clear the POI
                                    new Decorator(ret => Harvestable.LootableWoWGameObject == null, new Action(ret => BotPoi.Current = new BotPoi(PoiType.None))),

                                    // Full bags? Display a warning message.
                                    new Decorator(ret => (Harvestable.LootableWoWGameObject.IsHerb && Inventory.HerbBagSlotsFree <= MinFreeBagSlots) || (Harvestable.LootableWoWGameObject.IsMineral && Inventory.MiningBagSlotsFree <= MinFreeBagSlots),
                                        new ThrottlePasses(1, 5,
                                            new Action(c =>
                                            {
                                                Log.FailLog("TwB1AHIAIABiAGEAZwBzACAAYQByAGUAIABmAHUAbABsAC4AIABXAGkAbABsACAAbgBvACAAbABvAG4AZwBlAHIAIABoAGEAcgB2AGUAcwB0AGkAbgBnACAAbgBvAGQAZQBzAA==".Base64Decode());
                                                BotPoi.Current = new BotPoi(PoiType.None);
                                                IsBagsFull = true;
                                            }))),


                                    // Lets get to harvesting
                                    new Decorator(ret => ((Harvestable.LootableWoWGameObject.IsHerb && Inventory.HerbBagSlotsFree > MinFreeBagSlots) || (Harvestable.LootableWoWGameObject.IsMineral && Inventory.MiningBagSlotsFree > MinFreeBagSlots)) && Harvestable.LootableWoWGameObject.CanUse(),
                                        new PrioritySelector(
                                            new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Harvesting ..."; })),
                                            new Decorator(ret => Me.IsFlying && Harvestable.LootableWoWGameObject.Distance < 10, new Movement.LandAndDismount("SABhAHIAdgBlAHMAdABpAG4AZwAsACAAYgB1AHQAIAB3AGUAJwByAGUAIABmAGwAeQBpAG4AZwAuACAAVwBlACAAcwBoAG8AdQBsAGQAIABiAGUAIABvAG4AIAB0AGgAZQAgAGcAcgBvAHUAbgBkAA==".Base64Decode())),
                                            new Harvestable.QuickHarvestBehaviorPS(ret => Harvestable.LootableWoWGameObject),
                                            new ActionAlwaysSucceed()
                                            ))
                            )),


                            // Find a pull target or move to a hotspot
                            new SwitchArgument<PoiType>(PoiType.Hotspot,
                                new PrioritySelector(

                                    // Don't do anything if the user is moving the character
                                    new Decorator(ret => Misc.IsUserMoving, new ActionAlwaysSucceed()),

                                    #region Search for valid mobs to pull

                                    // Look for targets matching entry IDs provided within our current area, but no further away than 'GrinderMaxMobDistance'
                                    new Decorator(ret => !Me.GotTarget || !CT.IsValidTarget() || CT.TaggedByOther && !CT.TappedByAllThreatLists,
                                        new Action(ctx =>
                                        {
                                            // Find the closest unit within the max distance to your hotspots
                                            WoWUnit closestUnit = ObjectManager.GetObjectsOfType<WoWUnit>(true, false)
                                                .Where(u =>
                                                        u.IsValidTarget() && GrinderHelpers.GrinderMobsList().Any(m=> m.ID == u.Entry) && (!u.IsTagged) &&
                                                        Navigator.CanNavigateFully(Me.Location, u.Location) &&
                                                        // Make sure its close enough to one of our hotsposts
                                                        (GrinderHelpers.IsNearHotspot(u, Setting.Instance.GrinderMaxMobDistance)) &&
                                                        !Blacklist.Contains(u, BlacklistFlags.All) && !u.IsPlayer)
                                                .OrderBy(u => u.DistanceSqr)
                                                .FirstOrDefault();

                                            // Got a valid unit
                                            if (closestUnit != null)
                                            {
                                                BotPoi.Current = new BotPoi(closestUnit, PoiType.Kill);
                                                return RunStatus.Success;
                                            }

                                            //BotPoi.Current = new BotPoi(PoiType.None);
                                            return RunStatus.Failure;
                                        }
                                    )),
                                    #endregion

                                    #region Move to the next hotspot
                                    new Decorator(ret => !Me.GotTarget || !CT.IsValidTarget(), //&& GrinderHelpers.MasterHotspotsList().Count > 0,//!string.IsNullOrEmpty(Setting.Instance.GrinderHotspots),
                                        new PrioritySelector(

                                            //new TreeSharpExt.ActionFail(context => Log.Info("==== in hotspot...")),

                                            // Remove the hotspot from the active list if we're in range
                                            new Decorator(ret => Me.Location.Distance2D(GrinderHelpers.CurrentHotspot) < 40, 
                                                new Action(context =>
                                                {
                                                    //Log.FailLog("===== Me.Location.Distance(CurrentHotspot) < 40");

                                                    GrinderHelpers.WorkingHotspotList.Remove(GrinderHelpers.CurrentHotspot);
                                                    GrinderHelpers.CurrentHotspot = WoWPoint.Empty;

                                                    //Log.FailLog("===== WorkingHotspotList: " + GrinderHelpers.WorkingHotspotList.Count);
                                                    //Log.FailLog("===== MasterHotspotsList: " + GrinderHelpers.MasterHotspotsList().Count);

                                                })),

                                            // Fly to the next hotspot if its a long way
                                            new Decorator(ret=> Me.Location.Distance(GrinderHelpers.CurrentHotspot) > 1000, 
                                                new PrioritySelector(
                                                    new TreeSharpExt.ActionFail(context => { TreeRoot.StatusText = "RgBsAHkAaQBuAGcAIAB0AG8AIABuAGUAeAB0ACAAaABvAHQAcwBwAG8AdAA=".Base64Decode(); }),
                                                    new Movement.MoveTo(ret => GrinderHelpers.CurrentHotspot, ret => Movement.MovementProviderType.Flightor, ret => Movement.MountStrategyType.MountFlying, ret => false)
                                                )),

                                            // Flying? Then land
                                            new Decorator(ret => Me.IsFlying && !Setting.Instance.GrinderPreferFlyingMount.OK() && Me.Location.Distance(GrinderHelpers.CurrentHotspot) < 200, new Movement.LandAndDismount("Close to hotspot, should be on the ground")),

                                            // Basic navigation to the next hotspot
                                            new TreeSharpExt.ActionFail(context => { TreeRoot.StatusText = "TQBvAHYAaQBuAGcAIAB0AG8AIABuAGUAeAB0ACAAaABvAHQAcwBwAG8AdAA=".Base64Decode(); }),

                                            // Ground mount or flying mount?
                                            new Decorator(ret => !Setting.Instance.GrinderPreferFlyingMount.OK(), new Movement.MoveTo(ret => GrinderHelpers.CurrentHotspot, ret => Movement.MovementProviderType.Navigator, ret => Movement.MountStrategyType.None, ret => false)),
                                            new Decorator(ret => Setting.Instance.GrinderPreferFlyingMount.OK(), new Movement.MoveTo(ret => GrinderHelpers.CurrentHotspot, ret => Movement.MovementProviderType.Flightor, ret => Movement.MountStrategyType.MountFlying, ret => false))
                                            
                                            )
                                        )
                                        
                                    #endregion

                            ))


                        )),



                    // Add this so it does not fall through to our regular non-grider behaviors
                    new ActionAlwaysSucceed()

                    ));
        }


        public static Composite CreateGrinderCombatBehavior()
        {
            return new Decorator(ret => Me.Combat,
                new PrioritySelector(

                    // Check for aggroed mobs
                    new Decorator(ret => Me.GotTarget && !CT.Combat && CT.DistanceSqr > 15 * 15 && ObjectManager.GetObjectsOfTypeFast<WoWUnit>().FirstOrDefault(u => u.Combat && u.IsHostile && u.GotTarget && u.CurrentTargetGuid == Me.Guid && CT.Guid != u.Guid) != null,
                        new ThrottlePasses(1, 1,
                            new Action(ctx =>
                            {
                                WoWUnit aggroUnit = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().FirstOrDefault(u => u.Combat && u.IsHostile && u.GotTarget && u.CurrentTargetGuid == Me.Guid && CT.Guid != u.Guid);
                                if (aggroUnit == null) return;

                                Log.Info(string.Format("We got aggro from {0}.", aggroUnit.Name));
                                aggroUnit.Target();
                            }

                    ))),

                    // Movement
                    new Decorator(ret => !Misc.IsUserMoving, Alive.CreateCharacterMovement()),
                    
                    // Combat Buff
                    new Decorator(ret => Setting.Instance.BehaviorCombatBuff.ResultOK(), RoutineManager.Current.CombatBuffBehavior),

                    // Actual Combat Behavior
                    new PrioritySelector(RoutineManager.Current.CombatBehavior)

                    ));



        }

        public static Composite Logout()
        {
            return new Decorator(ret => !Me.Combat,
                new Sequence(
                    new Action(context => { Log.Info("VABoAGEALQB0AGgAYQAtAHQAaABhAC0AdABoAGEAIAAuAC4ALgAuAFQAaABhAHQAcwAgAGEAbABsACAAZgBvAGwAawBzACEA".Base64Decode()); Lua.DoString("Logout()"); }),
                    new WaitContinue(30, ret=> Me.Combat, new ActionAlwaysSucceed()),
                    new Action(context => TreeRoot.Stop("Event trigger told us to logout."))

                    ));
        }

        public static Composite ReturnHome()
        {
            return new Decorator(ret => !Me.Combat,
                new PrioritySelector(

                    // Are we close enough to our home location? If so stop HB
                    new Decorator(ret=> Me.Location.Distance(RAFBot2.GrinderHomeLocation) < 5, 
                        new PrioritySelector(
                            new TreeSharpExt.LogMessage("QQByAHIAaQB2AGUAZAAgAGEAdAAgAG8AdQByACAAaABvAG0AZQAgAGwAbwBjAGEAdABpAG8AbgAuACAAUwB0AG8AcABwAGkAbgBnACAASABCAA==".Base64Decode()),
                            new Action(context => TreeRoot.Stop("Event trigger told us to return home. We're now at home"))
                            
                            )),
                    new ThrottlePasses(1, 30, new Action(context => Log.Info(string.Format("RQB2AGUAbgB0ACAAdAByAGkAZwBnAGUAcgAgAGgAYQBzACAAZgBpAHIAZQBkAC4AIABOAG8AdwAgAG0AbwB2AGkAbgBnACAAdABvACAAbwB1AHIAIABoAG8AbQBlACAAbABvAGMAYQB0AGkAbwBuAC4A".Base64Decode(), GrinderActionTrigger.RequiredAction)))),
                    new Movement.MoveTo(ret => RAFBot2.GrinderHomeLocation, ret => Movement.MovementProviderType.Flightor, ret => Movement.MountStrategyType.MountFlying, ret => false)

                    ));
        }

        public static Composite UseHearthstone()
        {
            return new Decorator(ret => !Me.Combat,
                new Sequence(
                    new DecoratorContinue(ret => Me.Mounted || Me.IsFlying, new Movement.MountStrategyPS(ret => Movement.MountStrategyType.DismountOrCancelShapeshift)),
                    new DecoratorContinue(ret => Me.IsMoving, new Movement.MoveStop()),
                    new Decorator(ret => !Me.IsMoving && !Me.IsCasting, new Action(context => Lua.DoString(string.Format("UseItemByName(\"{0}\")", 6948)))),
                    new Action(context => Log.Info("RQB2AGUAbgB0ACAAdAByAGkAZwBnAGUAcgAsACAAdQBzAGkAbgBnACAASABlAGEAcgB0AGgAcwB0AG8AbgBlAA==".Base64Decode())),
                    new WaitContinue(3, ret=> Me.IsCasting, new ActionAlwaysSucceed()),
                    new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),
                    new DecoratorContinue(ret => !Me.IsCasting, new Action(context => TreeRoot.Stop("RQB2AGUAbgB0ACAAdAByAGkAZwBnAGUAcgAgAHQAbwBsAGQAIAB1AHMAIAB1AHMAZQAgAGgAZQBhAHIAdABoAHMAdABvAG4AZQAuACAAVwBlACAAaABlAGEAcgB0AGgAZQBkACwAIAB3AGUAJwByAGUAIABkAG8AbgBlAC4A".Base64Decode()))),
                    new ActionAlwaysSucceed()

                    ));
        }

        public static Composite Castspell(Harvestable.Selection<string> spell)
        {
            return new Decorator(ret => !Me.Combat && !Me.IsCasting && SpellManager.HasSpell(spell(ret)),
                new Sequence(
                    new DecoratorContinue(ret=> Me.Mounted || Me.IsFlying, new Movement.MountStrategyPS(ret=> Movement.MountStrategyType.DismountOrCancelShapeshift)),
                    new Decorator(ret => !Me.Mounted && !Me.IsFlying, new Action(context => SpellManager.Cast(spell(context)))),
                    new Action(context => Log.Info("Event trigger, casting spell " + spell(context))),
                    new WaitContinue(1, ret=> Me.IsCasting, new ActionAlwaysSucceed()),
                    new WaitContinue(5, ret=> !Me.IsCasting, new ActionAlwaysSucceed())


                    ));
        }
        

        /// <summary>
        /// NOT WORKING! DO NOT USE
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public static Composite DoTradeSkill(Selection<string> name)
        {
            return new Decorator(ret => !Me.Combat && !Me.IsCasting && name != null && name(ret) != null, //&& SpellManager.HasSpell(name(ret)),
                new Sequence(
                    new DecoratorContinue(ret => Me.Mounted || Me.IsFlying, new Movement.MountStrategyPS(ret => Movement.MountStrategyType.DismountOrCancelShapeshift)),
                    new Action(context => SpellManager.Cast("Leatherworking")),
                    new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),
                    new Action(context => DoLeatherThing("Knothide Armor Kit")),
                    //new DecoratorContinue(ret => !Me.Mounted && !Me.IsFlying,
                        
                        /*new Action(ret =>
                        {
                            Log.Info("=== using: " + name(ret));
                            string myLua = string.Format(@"local s for i=1,GetNumTradeSkills() do s=GetTradeSkillInfo(i) if (s=='{0}') then DoTradeSkill(i,1) end end", name(ret));
                            //Lua.DoString(string.Format("RunMacroText(\"{0}\")", myLua), 0);
                            Lua.DoString(myLua);
                            //Lua.DoString(string.Format("RunMacroText(\"{0}\")", "/s hello"), 0);

                            //Lua.DoString("RunMacroText(\"" + RealLuaEscape(myLua) + "\")");
                        })),
                         */
                    new Action(ret => Log.Info("Event trigger, DoTradeSkill " + name(ret))),
                    new WaitContinue(1, ret => Me.IsCasting, new ActionAlwaysSucceed()),
                    new WaitContinue(5, ret => !Me.IsCasting, new ActionAlwaysSucceed())
                    //new Action(context => SpellManager.Cast("Leatherworking"))


                    ));
        }

        static void DoLeatherThing(string name)
        {
            Lua.DoString("RunMacroText(\"/script local s for i=1,GetNumTradeSkills() do s=GetTradeSkillInfo(i) if (s=='Knothide Armor Kit') then DoTradeSkill(i,1) end end\");");
            //string myLua = string.Format(@"local s for i=1,GetNumTradeSkills() do s=GetTradeSkillInfo(i) if (s=='{0}') then DoTradeSkill(i,1) end end", name);
            //Lua.DoString(string.Format("RunMacroText(\"{0}\");", myLua));
            //Lua.DoString(myLua);
        }


        /*
        private static string RealLuaEscape(string luastring)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(luastring);
            return bytes.Aggregate(String.Empty, (current, b) => current + ("\\" + b));
        }

        private static Composite RunMacroText(string macro, CanRunDecoratorDelegate cond)
        {
            return new Action(a => Lua.DoString("RunMacroText(\"" + RealLuaEscape(macro) + "\")"));
        }
         */

        //string myLUA = @"local s for i=1,GetNumTradeSkills() do s=GetTradeSkillInfo(i) if (s=='Smelt Ghost Iron') then DoTradeSkill(i,GetItemCount('Ghost Iron Ore')/2) end end"; 
        //Lua.DoString(string.Format("{0}", myLUA),0);
    }
}
