﻿using CommonBehaviors.Actions;
using Fpsware.Botbases.RAFBot2.Questing;
using Fpsware.Library3.Composites;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Helpers;
using Styx;
using Styx.CommonBot.Frames;
using Styx.CommonBot.POI;
using Styx.WoWInternals.WoWObjects;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Action = Styx.TreeSharp.Action;
using Misc = Fpsware.Library3.Helpers.Misc;

namespace Fpsware.Botbases.RAFBot2.Behaviors
{
    public static class QuestBehaviors
    {
        #region Local declarations

        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        private static WoWUnit _objectiveKillUnit;
        internal const int DistanceForQuestItem = 30;
        internal static WoWGameObject ObjectToGrab;

        #endregion

        public static Composite CreateQuestBehavior()
        {
            return new PrioritySelector(

                // Auto quest turnin 
                new Decorator(ret => Setting.Instance.AutoQuestTurnIn.OK() && !Misc.IsUserMoving && QuestingHelpers.AutoQuestTurninUnit != null,
                    new PrioritySelector(

                        new ThrottlePasses(1, 5, new TreeSharpExt.LogMessage("QQB1AHQAbwAgAFEAdQBlAHMAdAAgAFQAdQByAG4AIABpAG4AIABFAE4AQQBCAEwARQBEACwAIABhAG4AZAAgAHcAZQAgAGgAYQB2AGUAIABhACAAcQB1AGUAcwB0ACAAZwBpAHYAZQByACAAbgBlAGEAcgAgAGIAeQAuAA==".Base64Decode())),
                        new Decorator(ret => (!Me.GotTarget || CT.Guid != QuestingHelpers.AutoQuestTurninUnit.Guid), new Action(context => QuestingHelpers.AutoQuestTurninUnit.Target())),
                        new Decorator(ret => CT.WithinInteractRange && !WoWMovement.ActiveMover.IsMoving && !QuestFrame.Instance.IsVisible, new Action(c => CT.Interact())),
                        new Movement.StopMovingPS(ret => Movement.StopMovingWhen.InMeleeRangeOfUnit, ret => CT, ret => 1, ret => CT.Location),
                        new Decorator(ret => !CT.WithinInteractRange, new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Navigator))
                    )),

                // Auto quest pickup
                new Decorator(ret => Setting.Instance.AutoQuestPickUp.OK() && !Misc.IsUserMoving && QuestingHelpers.AutoQuestPickupUnit != null,
                    new PrioritySelector(
                        new ThrottlePasses(1, 5, new TreeSharpExt.LogMessage("QQB1AHQAbwAgAFEAdQBlAHMAdAAgAFAAaQBjAGsAIAB1AHAAIABFAE4AQQBCAEwARQBEACwAIABhAG4AZAAgAHcAZQAgAGgAYQB2AGUAIABhACAAcQB1AGUAcwB0ACAAZwBpAHYAZQByACAAbgBlAGEAcgAgAGIAeQAuAA==".Base64Decode())),
                        new Decorator(ret => (!Me.GotTarget || CT.Guid != QuestingHelpers.AutoQuestPickupUnit.Guid), new Action(context => QuestingHelpers.AutoQuestPickupUnit.Target())),
                        new Decorator(ret => CT.WithinInteractRange && !WoWMovement.ActiveMover.IsMoving && !QuestFrame.Instance.IsVisible, new Action(c => CT.Interact())),
                        new Movement.StopMovingPS(ret => Movement.StopMovingWhen.InMeleeRangeOfUnit, ret => CT, ret => 1, ret => CT.Location),
                        new Decorator(ret => !CT.WithinInteractRange, new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Navigator))
                    )),


                // Regular quest interaction, use the leader's target
                new Decorator(ret => !Misc.IsUserMoving && QuestingHelpers.IsTargetInteractable, 
                    new PrioritySelector(
                        new Decorator(ret => RAFBot2.IsLeaderValid && RAFGroup.Leader.GotTarget && (!Me.GotTarget || CT.Guid != RAFGroup.Leader.CurrentTargetGuid), new Action(context => RAFGroup.Leader.CurrentTarget.Target())),
                        new Decorator(ret => Me.GotTarget && CT.WithinInteractRange && !WoWMovement.ActiveMover.IsMoving && QuestFrame.Instance != null && !QuestFrame.Instance.IsVisible, new Action(c => CT.Interact())),
                        new Decorator(ret => Me.GotTarget, new Movement.StopMovingPS(ret => Movement.StopMovingWhen.InMeleeRangeOfUnit, ret => CT, ret => 1, ret => CT.Location) ),
                        new Decorator(ret => Me.GotTarget && !CT.WithinInteractRange, new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Navigator))
                )),

                // Process the quest log for advanced fetures. i.e query wowhead.com for additional information
                new Decorator(ret=> Setting.Instance.QuestUseAdvancedFeatures.OK() && (Setting.Instance.QuestObjectiveKills.OK() || Setting.Instance.QuestObjectCollection.OK()),
                    new PrioritySelector(
                        new Decorator(ret => RAFBot2.QuestJustAccepted, new Action(context => { RAFBot2.QuestJustAccepted = false; QuestingHelpers.ProcessQuestLog(); })),
                        new ThrottlePasses(1, 10, new Action(context => QuestingHelpers.ProcessQuestLog()))
                    )
                ),

                // Pickup game objects that we have quests for
                new Decorator(ret => Setting.Instance.QuestUseAdvancedFeatures.OK() && Setting.Instance.QuestObjectCollection.OK() && !Misc.IsUserMoving && QuestingHelpers.InteractableQuestObject != null, 
                    new PrioritySelector(

                        new ThrottlePasses(1, 5, new TreeSharpExt.LogMessage("VwBlACAAYQBwAHAAZQBhAHIAIAB0AG8AIABoAGEAdgBlACAAYQBuACAAaQBuAHQAZQByAGEAYwB0ACAAcQB1AGUAcwB0ACAAbwBiAGoAZQBjAHQAIABuAGUAYQByACAAYgB5AA==".Base64Decode())),
                        new Decorator(ret => Me.IsMoving && QuestingHelpers.InteractableQuestObject.WithinInteractRange, new Movement.MoveStop()),
                        new TreeSharpExt.ActionFail(context => QuestingHelpers.AdvancedQuestingInAction = true),
                        new Decorator(ret => !QuestingHelpers.InteractableQuestObject.WithinInteractRange, new Movement.MoveTo(ret => QuestingHelpers.InteractableQuestObject.Location, ret => Movement.MovementProviderType.Navigator)),

                        // In range of the object, then loot it
                        new Decorator(ret => QuestingHelpers.InteractableQuestObject.WithinInteractRange && !WoWMovement.ActiveMover.IsMoving && (!LootFrame.Instance.IsVisible && !Me.IsCasting), 
                            new Sequence(
                                new Action(context => Lua.DoString("SetCVar('AutoLootDefault','1')")),
                                new Action(c => QuestingHelpers.InteractableQuestObject.Interact()),
                                new WaitContinue(2, ret=> Me.IsCasting || Me.Looting, new ActionAlwaysSucceed()),
                                new WaitContinue(5, ret => !Me.IsCasting && !Me.Looting, new ActionAlwaysSucceed()),
                                new Action(context => Lua.DoString("SetCVar('AutoLootDefault','0')")),
                                new WaitContinue(1, ret=> false, new ActionAlwaysSucceed())
                            ))
                )),

                // Find and target mobs associated with our quest objectives
                new Decorator(ret => Setting.Instance.QuestUseAdvancedFeatures.OK() && Setting.Instance.QuestObjectiveKills.OK() && !Me.Combat && (!Me.GotTarget || !Me.GotValidTarget()) && (!Setting.Instance.AutoQuestPickUp.OK() || QuestingHelpers.AutoQuestPickupUnit == null) && (!Setting.Instance.AutoQuestTurnIn.OK() || QuestingHelpers.AutoQuestTurninUnit == null),
                    new PrioritySelector(
                        new ThrottlePasses(1, 1, new Decorator(ret=> _objectiveKillUnit == null || !_objectiveKillUnit.IsValid || !_objectiveKillUnit.IsValidTarget(),
                            new Action(context =>
                            {
                                _objectiveKillUnit = QuestingHelpers.ObjectiveKillMobWoWUnit(Setting.Instance.QuestObjectiveKillsMaxDistance);
                                if (_objectiveKillUnit != null && _objectiveKillUnit.IsValid && _objectiveKillUnit.IsValidTarget())
                                {
                                    QuestingHelpers.AdvancedQuestingInAction = true;
                                    _objectiveKillUnit.Target();
                                    BotPoi.Current = new BotPoi(_objectiveKillUnit, PoiType.Kill);
                                }
                            }
                        )))
                ))
                
                

                );
        }


        
    }
}
