﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Fpsware.Library3.Extensions
{
    public static class Misc
    {
        /// <summary>
        /// Returns TRUE if the passed string does not '.Contains': "None", "Never" or "DISABLED"
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool IsAllowed(this string str)
        {
            if (str.ToUpper().Contains("NEVER")) return false;
            if (str.ToUpper().Contains("NONE")) return false;
            if (str.ToUpper().Contains("DISABLED")) return false;

            return true;
        }

        public static string GetRegXProperty(this string source, string regEx, string defaultValueOnFailure = "")
        {
            string result = Regex.Match(source, regEx, RegexOptions.IgnoreCase).Value.Trim();
            return !string.IsNullOrEmpty(result) ? result : defaultValueOnFailure;
        }


        public static bool TryParse<T>(this Enum theEnum, string valueToParse, out T returnValue)
        {
            returnValue = default(T);
            int intEnumValue;
            if (Int32.TryParse(valueToParse, out intEnumValue))
            {
                if (Enum.IsDefined(typeof(T), intEnumValue))
                {
                    returnValue = (T)(object)intEnumValue;
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        ///   A string extension method that turns a Camel-case string into a spaced string. (Example: SomeCamelString -> Some Camel String)
        /// </summary>
        /// <remarks>
        ///   Created 2/7/2011.
        /// </remarks>
        /// <param name = "str">The string to act on.</param>
        /// <returns>.</returns>
        public static string CamelToSpaced(this string str)
        {
            var sb = new StringBuilder();
            foreach (char c in str)
            {
                if (char.IsUpper(c))
                {
                    sb.Append(' ');
                }
                sb.Append(c);
            }
            return sb.ToString();
        }

        private static string Right(string s, int c)
        {
            return s.Substring(c > s.Length ? 0 : s.Length - c);
        }
        public static string UnitID(ulong guid)
        {
            return Right(string.Format("{0:X4}", guid), 4);
        }

        public static string AlignLeft(this string s, int width)
        {
            int len = s.Length;
            if (len >= width)
                return s.Substring(0, width);

            return s + ("                                                                                                                          ".Substring(0, width - len));
        }

        public static string AlignRight(this string s, int width)
        {
            int len = s.Length;
            if (len >= width)
                return s.Substring(0, width);

            return ("                                                                                                                          ".Substring(0, width - len)) + s;
        }
    }
}
