﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Library3.Extensions
{
    public static class WoWObjectExt
    {
        public static WoWPoint AnticipatedLocation(this WoWObject wowObject, TimeSpan atTime)
        {
            var wowUnit = wowObject as WoWUnit;

            if (wowUnit == null)
            {
                return wowObject.Location;
            }

            var anticipatedLocation = wowUnit.Location.RayCast( wowUnit.RenderFacing, (float) (wowUnit.MovementInfo.CurrentSpeed*atTime.TotalSeconds));

            return (anticipatedLocation);
        }


        // 30May2013-03:56UTC chinajade
        public static void BlacklistForCombat(this WoWObject wowObject, TimeSpan duration)
        {
            Blacklist.Add(wowObject.Guid, BlacklistFlags.Pull | BlacklistFlags.Combat, duration);
        }


        // 11Apr2013-03:56UTC chinajade
        public static void BlacklistForInteracting(this WoWObject wowObject, TimeSpan duration)
        {
            Blacklist.Add(wowObject.Guid, BlacklistFlags.Interact, duration);
        }


        public static void BlacklistForPulling(this WoWObject wowObject, TimeSpan duration)
        {
            Blacklist.Add(wowObject.Guid, BlacklistFlags.Pull, duration);
        }


        // 11Apr2013-04:41UTC chinajade
        public static bool IsBlacklistedForCombat(this WoWObject wowObject)
        {
            return Blacklist.Contains(wowObject.Guid, BlacklistFlags.Combat);
        }


        // 11Apr2013-04:41UTC chinajade
        public static bool IsBlacklistedForInteraction(this WoWObject wowObject)
        {
            return Blacklist.Contains(wowObject.Guid, BlacklistFlags.Interact);
        }


        // 4Jun2013-04:41UTC chinajade
        public static bool IsBlacklistedForPulling(this WoWObject wowObject)
        {
            return Blacklist.Contains(wowObject.Guid, BlacklistFlags.Pull);
        }


        // 2Sep2013 chinajade
        public static int SafeGuid(this WoWObject wowObject)
        {
            return (int) (wowObject.Guid & 0x0ffffff);
        }


        public static double SurfacePathDistance(this WoWObject objectTo)
        {
            return StyxWoW.Me.SurfacePathDistance(objectTo);
        }


        public static double SurfacePathDistance(this WoWObject objectFrom, WoWObject objectTo)
        {
            return objectFrom.Location.SurfacePathDistance(objectTo.Location);
        }

        public static double SurfacePathCost(this WoWObject objectTo)
        {
            return StyxWoW.Me.SurfacePathCost(objectTo);
        }


        public static double SurfacePathCost(this WoWObject objectFrom, WoWObject objectTo)
        {
            return objectFrom.Location.SurfacePathCost(objectTo.Location);
        }
    }
}
