﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Fpsware.Library3.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Library3.Extensions
{
    public static class WoWUnitExt
    {

        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        public static bool GotValidTarget(this WoWUnit unit)
        {
            return unit.GotTarget && unit.CurrentTarget.IsValidTarget();
        }

        public static bool IsEatingOrDrinking(this WoWUnit unit)
        {
            if (unit.HasAura("Food")) return true;
            if (unit.HasAura("Drink")) return true;
            
            //return unit.HasAura("Food") || !unit.HasAura("Drink");
            return false;
        }

        public static bool IsEating(this WoWUnit unit)
        {
            return unit.HasAura("Food");
        }

        public static bool IsDrinking(this WoWUnit unit)
        {
            return unit.HasAura("Drink");
        }

        public static bool IsCaster(this LocalPlayer u)
        {
            return !u.IsMelee();
        }

        public static bool IsMelee(this LocalPlayer u)
        {
            switch (u.Class)
            {
                case WoWClass.DeathKnight:
                case WoWClass.Rogue:
                case WoWClass.Warrior:
                    return true;

                case WoWClass.Paladin:
                    return Me.Specialization != WoWSpec.PaladinHoly;

                case WoWClass.Monk:
                    return Me.Specialization != WoWSpec.MonkMistweaver;

                case WoWClass.Druid:
                    if (Me.Specialization == WoWSpec.DruidFeral || Me.Specialization == WoWSpec.DruidGuardian)
                        return true;
                    break;

                case WoWClass.Shaman:
                    return Me.Specialization == WoWSpec.ShamanEnhancement;
            }

            return false;
        }


        /// <summary>
        /// Similar to WoWUnit.Mounted, but also tests for auras that provide a 'mounted' characteristic.
        /// <para>Notes:<list type="bullet">
        /// <item><description><para> * The following auras count as 'mounted' auras:
        /// Druid(Flight Form), Druid(Swift Fight Form, Druid(Travel Form), Shaman(Ghost Wolf),
        /// Worgen(Running Wild).</para></description></item>
        /// </list></para>
        /// </summary>
        /// <param name="wowUnit"></param>
        /// <returns></returns>
        public static bool IsMounted(this WoWUnit wowUnit)
        {
            return (wowUnit.Mounted || wowUnit.GetAllAuras().Any(a => MountedAuras.Contains(a.SpellId)));
        }
        private readonly static int[] MountedAuras =
            {
                33943,      // Druid: Flight Form
                40120,      // Druid: Swift Flight Form
                  783,      // Druid: Travel Form
                93326,      // Herbalist: Sandstone Drake
                 2645,      // Shaman: Ghost Wolf
                87840,      // Worgen: Running Wild
            };


        // 16Apr2013-10:34UTC chinajade
        public static bool IsShapeshifted(this WoWUnit wowUnit)
        {
            return wowUnit.GetAllAuras().Any(a => ShapeshiftAuras.Contains(a.SpellId));
        }
        private readonly static int[] ShapeshiftAuras =
            {
                 1066,      // Druid: Aquatic Form
                 5487,      // Druid: Bear Form
                  768,      // Druid: Cat Form
                33943,      // Druid: Flight Form
                40120,      // Druid: Swift Flight Form
                  783,      // Druid: Travel Form       
                93326,      // Herbalist: Sandstone Drake
                 2645,      // Shaman: Ghost Wolf
                87840,      // Worgen: Running Wild
            };


        // 11Apr2013-07:48UTC chinajade
        public static bool IsUntagged(this WoWUnit wowUnit)
        {
            return (wowUnit != null) && (wowUnit.TappedByAllThreatLists || wowUnit.TaggedByMe || !wowUnit.TaggedByOther);
        }


        #region Auras

        public static bool HasMyAura(this WoWUnit unit, string aura)
        {
            return HasMyAura(unit, aura, 0);
        }

        public static bool HasMyAura(this WoWUnit unit, string aura, int stacks)
        {
            return HasAura(unit, aura, stacks, StyxWoW.Me);
        }

        private static bool HasAura(this WoWUnit unit, string aura, int stacks, WoWUnit creator)
        {
            return unit.GetAllAuras().Any(a => a.Name == aura && a.TimeLeft.TotalSeconds > 2 && a.StackCount >= stacks && (creator == null || a.CreatorGuid == creator.Guid));
        }

        public static bool HasActiveAura(this WoWUnit unit, string aura)
        {
            return unit.ActiveAuras.ContainsKey(aura);
        }

        public static bool HasAuraWithEffect(this WoWUnit unit, WoWApplyAuraType applyType)
        {
            return
                unit.Auras.Values.Any(a => a.Spell != null && a.Spell.SpellEffects.Any(se => applyType == se.AuraType));
        }

        public static bool HasAuraWithEffect(this WoWUnit unit, params WoWApplyAuraType[] applyType)
        {
            var hashes = new HashSet<WoWApplyAuraType>(applyType);
            return
                unit.Auras.Values.Any(
                    a => a.Spell != null && a.Spell.SpellEffects.Any(se => hashes.Contains(se.AuraType)));
        }

        public static TimeSpan GetAuraTimeLeft(this WoWUnit onUnit, string auraName, bool fromMyAura = true)
        {
            WoWAura wantedAura =
                onUnit.GetAllAuras()
                    .FirstOrDefault(
                        a =>
                            a != null && a.Name == auraName && a.TimeLeft > TimeSpan.Zero &&
                            (!fromMyAura || a.CreatorGuid == StyxWoW.Me.Guid));

            return wantedAura != null ? wantedAura.TimeLeft : TimeSpan.Zero;
        }

        public static TimeSpan GetAuraTimeLeft(this WoWUnit onUnit, int auraID, bool fromMyAura = true)
        {
            WoWAura wantedAura = onUnit.GetAllAuras() .FirstOrDefault( a => a.SpellId == auraID && a.TimeLeft > TimeSpan.Zero && (!fromMyAura || a.CreatorGuid == StyxWoW.Me.Guid));

            return wantedAura != null ? wantedAura.TimeLeft : TimeSpan.Zero;
        }

        public static uint GetAuraStacks(this WoWUnit onUnit, string auraName, bool fromMyAura = true)
        {
            WoWAura wantedAura = onUnit.GetAllAuras() .FirstOrDefault( a => a.Name == auraName && a.TimeLeft > TimeSpan.Zero && (!fromMyAura || a.CreatorGuid == StyxWoW.Me.Guid));

            if (wantedAura == null) return 0;

            return wantedAura.StackCount == 0 ? 1 : wantedAura.StackCount;
        }

        public static bool HasAnyAura(this WoWUnit unit, params string[] auraNames)
        {
            WoWAuraCollection auras = unit.GetAllAuras();
            HashSet<string> hashes = new HashSet<string>(auraNames);
            return auras.Any(a => hashes.Contains(a.Name));
        }

        #endregion

        #region Is Controlled in any way (eg Rooted, Silenced, Slowed, Dazed, Enraged
        public static bool IsRooted(this WoWUnit unit)
        {
            return unit.Auras.Any(x => x.Value.Spell.Mechanic == WoWSpellMechanic.Rooted);
        }

        public static bool IsSilenced(this WoWUnit unit)
        {
            return unit.Auras.Any(x => x.Value.Spell.Mechanic == WoWSpellMechanic.Silenced);
        }

        public static bool IsSlowed(this WoWUnit unit)
        {
            return unit.Auras.Any(x => x.Value.Spell.Mechanic == WoWSpellMechanic.Slowed);
        }

        public static bool IsDazed(this WoWUnit unit)
        {
            return unit.Auras.Any(x => x.Value.Spell.Mechanic == WoWSpellMechanic.Dazed);
        }

        public static bool IsEnraged(this WoWUnit unit)
        {
            return unit.Auras.Any(x => x.Value.Spell.Mechanic == WoWSpellMechanic.Enraged);
        }
        #endregion


        public static bool IsTargetingMe(this WoWUnit u)
        {
            return u.GotTarget && u.CurrentTargetGuid == Me.Guid;
        }

        public static bool IsInOurParty(this WoWUnit wowUnit)
        {
            return ((StyxWoW.Me.PartyMembers.FirstOrDefault(partyMember => (partyMember.Guid == wowUnit.Guid))) != null);
        }


        public static bool IsMeTank(this LocalPlayer player)
        {
            {
                if (player.HasAura("Bear Form")) return true;
                if (Units.Friendly.Tanks.Contains(player)) return true;
                if (player.Role == WoWPartyMember.GroupRole.Tank) return true;

                bool all = Units.Friendly.Tanks.All(p => !p.IsAlive);
                return (player.Role & WoWPartyMember.GroupRole.Tank) != 0 || all && player.HasAura("Bear Form");
            }
        }

        public static bool IsMeHealer(this LocalPlayer player)
        {
            {
                if (player.Role == WoWPartyMember.GroupRole.Healer) return true;

                return (player.Role & WoWPartyMember.GroupRole.Healer) != 0;
            }
        }

        public static bool IsCrowdControlled(this WoWUnit unit)
        {
            if (unit != null)
            {
                return unit.GetAllAuras().Any(a =>
                                                    a.IsHarmful && (
                                                    a.Spell.Mechanic == WoWSpellMechanic.Banished ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Disoriented ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Charmed ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Horrified ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Incapacitated ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Polymorphed ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Sapped ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Shackled ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Asleep ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Frozen ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Invulnerable ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Invulnerable2 ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Turned ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Fleeing ||
                                                    a.Spell.Mechanic == WoWSpellMechanic.Silenced ||

                                                    a.Spell.Name == "Hex"));
            }
            return false;
        }



        public static bool ShowPlayerNames { get; set; }
        public static string SafeName(this WoWObject obj)
        {
            if (obj.IsMe)
            {
                return "Me";
            }

            string name;
            if (obj is WoWPlayer)
            {
                if (!obj.ToPlayer().IsFriendly)
                {
                    name = "Enemy.";
                }
                else
                {
                    if (RaFHelper.Leader == obj)
                        name = "lead.";
                    else if (Units.Friendly.Tanks.Any(t => t.Guid == obj.Guid))
                        name = "tank.";
                    else if (Units.Friendly.Healers.Any(t => t.Guid == obj.Guid))
                        name = "healer.";
                    else
                        name = "dps.";
                }
                name += ShowPlayerNames ? ((WoWPlayer)obj).Name : ((WoWPlayer)obj).Class.ToString();
            }
            else if (obj is WoWUnit && obj.ToUnit().IsPet)
            {
                WoWUnit root = obj.ToUnit().OwnedByRoot;
                name = root == null ? "(unknown)" : root.SafeName() + ":Pet";
            }
            else
            {
                name = obj.Name;
            }

            return name + "." + Misc.UnitID(obj.Guid);
        }


        internal static bool IsBoss(this WoWUnit unit)
        {
            return unit != null && (unit.IsBoss || BossList.BossIds.Contains(unit.Entry));
        }


    }
}
