﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using JetBrains.Annotations;
using Styx;

namespace Fpsware.Botbases.RAFBot2
{
    public static class GrinderActionTrigger
    {
        static Stopwatch _elapsedLongtermStopwatch = new Stopwatch();
        static Stopwatch _elapsedShorttermStopwatch = new Stopwatch();
        public static bool IsTriggerConditionMet
        {
            get
            {
                if (_actionTriggers.Count <= 0) return false;

                // Enumerate all action triggers. Usually only 1
                foreach (ActionTrigger actionTrigger in ActionTriggersList())
                {
                    List<bool> triggerConditionResults = new List<bool>();

                    // Enumerate all trigger conditions (continue while all results are true)
                    foreach (TriggerCondition triggerCondition in actionTrigger.Triggers.TakeWhile(triggerCondition => triggerConditionResults.Count <= 0 || triggerConditionResults.All(b => b)))
                    {
                        switch (triggerCondition.Event)
                        {
                            //case "QuestComplete": break;

                            case "ItemCount":
                                triggerConditionResults.Add(StyxWoW.Me.CarriedItems.Count(i => i.Name == triggerCondition.Name) >= triggerCondition.Quantity);
                                break;

                            case "ElapsedMinutes":
                                TimeSpan span = DateTime.Now - triggerCondition.ElapsedMinutesStart;
                                
                                if (span.TotalMinutes >= triggerCondition.Quantity)
                                {
                                    _elapsedShorttermStopwatch.Reset();
                                    _elapsedShorttermStopwatch.Stop();
                                    triggerConditionResults.Add(true);
                                }
                                else
                                {
                                    /*
                                    // Start the long term timer
                                    if (span.TotalMinutes >= 30 && !_elapsedLongtermStopwatch.IsRunning)
                                    {
                                        Log.Info("==== starting the long term timer");
                                        _elapsedLongtermStopwatch.Reset(); 
                                        _elapsedLongtermStopwatch.Start();
                                    }

                                    // Less than 2 minutes to go? Show messages every 15 seconds
                                    if (span.TotalMinutes <= 2)
                                    {
                                        // Start the timer
                                        if (!_elapsedShorttermStopwatch.IsRunning)
                                        {
                                            _elapsedShorttermStopwatch.Reset();
                                            _elapsedShorttermStopwatch.Start();
                                        }

                                        Log.Info("==== WTF.... " + span.TotalMinutes);
                                        if (_elapsedShorttermStopwatch.ElapsedMilliseconds >= (15*1000))
                                        {
                                            _elapsedShorttermStopwatch.Reset();
                                            _elapsedShorttermStopwatch.Start();
                                            Log.InitLog(string.Format("Action Trigger has {0:##} seconds remaining", (triggerCondition.Quantity - span.TotalMinutes) * 60));
                                        }
                                    }

                                    // Long term timer message every 5 minutes
                                    else
                                    {
                                        if (_elapsedLongtermStopwatch.ElapsedMilliseconds >= (5 * (60 * 1000)))
                                        {
                                            _elapsedLongtermStopwatch.Reset();
                                            _elapsedLongtermStopwatch.Start();
                                            Log.InitLog(string.Format("Action Trigger has {0:##} minutes remaining", (triggerCondition.Quantity - span.TotalMinutes)));
                                        }
                                    }
                                     */
                                }
                                
                                break;
                        }
                    }

                    // If all results are TRUE the trigger is met
                    if (triggerConditionResults.Count > 0 && triggerConditionResults.All(b => b))
                    {
                        RequiredAction = actionTrigger.Action;
                        return true;
                    }

                }

                return false;
            }
        }
        public static void ResetActionTriggerTimers(this List<ActionTrigger> actionTriggersList)
        {
            bool result = false;
            foreach (ActionTrigger actionTrigger in ActionTriggersList())
            {
                foreach (TriggerCondition triggerCondition in actionTrigger.Triggers.Where(triggerCondition => triggerCondition.Event == "ElapsedMinutes"))
                {
                    triggerCondition.ElapsedMinutesStart = DateTime.Now;
                    result = true;
                }
            }

            if (result && Setting.Instance.UseGrinder.OK())
            {
                Log.Info("All Action Triggers 'ElapsedMinutes' has been reset.");
            }
        }
        public static string RequiredAction { get; private set; }

        [UsedImplicitly]
        private static List<ActionTrigger> _actionTriggers = new List<ActionTrigger>();

        private static string _parsedActionTriggerString;
        public static List<ActionTrigger> ActionTriggersList()
        {
            // Nothing has changed? Return the cached list
            if (string.IsNullOrEmpty(Setting.Instance.GrinderActionTriggers) || _parsedActionTriggerString == Setting.Instance.GrinderActionTriggers)
            {
                return _actionTriggers;
            }

            // Make the list
            _actionTriggers.Clear();
            string[] split = Setting.Instance.GrinderActionTriggers.Split('|');
            foreach (ActionTrigger trigger in split.Select(s => new ActionTrigger(s)))
            {
                _actionTriggers.Add(trigger);
            }

            _parsedActionTriggerString = Setting.Instance.GrinderActionTriggers;
            return _actionTriggers;
        }


        public class ActionTrigger
        {
            public Collection<TriggerCondition> Triggers { get; set; }
            public string Action { get; set; }

            public ActionTrigger()
            {
                Action = "";
                Triggers = new Collection<TriggerCondition>();
            }

            public ActionTrigger(string rawString)
            {
                Triggers = new Collection<TriggerCondition>();
                ActionTriggerRawString = rawString;
            }


            private string _rawString = null;
            public string ActionTriggerRawString
            {
                get
                {
                    // Trigger([ItemCount:Fel Iron Ore#40] _AND_ [ItemCount: Ghost Iron Ore#200] _AND_ [ItemCount: Some Weed#5]) Action(Logout)
                    StringBuilder sb = new StringBuilder();
                    sb.Append("Trigger(");

                    // Enumerate all trigger conditions and convert it to readable strings
                    foreach (TriggerCondition trigger in Triggers)
                    {
                        if (sb.ToString().EndsWith("]")) sb.AppendFormat(" {0} ", "_AND_");

                        switch (trigger.Event)
                        {
                            case "ItemCount":
                                sb.AppendFormat("[ItemCount:{0}#{1}]", trigger.Name, trigger.Quantity);
                                break;

                            case "ElapsedMinutes":
                                sb.AppendFormat("[{0}:{1}]", trigger.Event, trigger.Quantity);
                                break;   
                        }
                    }

                    sb.AppendFormat(") Action({0})", Action.Trim());
                    return sb.ToString();
                }

                set
                {
                    // Trigger([ItemCount:Fel Iron Ore#40] _AND_ [ItemCount: Ghost Iron Ore#200] _AND_ [ItemCount: Some Weed#5]) Action(Logout)
                    _rawString = value;
                    Triggers.Clear();

                    // [ItemCount:Fel Iron Ore#40] _AND_ [ItemCount: Iron Ore#200] _AND_ [ItemCount: Some Weed#5]
                    string innerActionsString = Regex.Match(_rawString, @"(?<=Trigger\()[^)]*(?=\))", RegexOptions.IgnoreCase).Value;

                    // Logout
                    Action = Regex.Match(_rawString, @"(?<=Action\()[^)]*(?=\))", RegexOptions.IgnoreCase).Value.Trim();

                    // [ItemCount:Fel Iron Ore#40]
                    // [ItemCount: Iron Ore#200]
                    // [ItemCount: Some Weed#5]
                    string[] splited = innerActionsString.Split(new[] { "_AND_" }, StringSplitOptions.None);

                    // [ItemCount:Fel Iron Ore#40]
                    foreach (string s in splited)
                    {
                        // ItemCount
                        string name = "";
                        string _event = Regex.Match(s, @"(?<=\[)[^)]*?(?=\:)", RegexOptions.IgnoreCase).Value;
                        string quantityTemp = "";

                        switch (_event.Trim())
                        {
                            case "ItemCount":
                                name = Regex.Match(s, @"(?<=\:)[^)]*?(?=\#)", RegexOptions.IgnoreCase).Value;
                                quantityTemp = Regex.Match(s, @"(?<=\#)\d*(?=\])").Value;
                                break;

                            case "ElapsedMinutes":
                                quantityTemp = Regex.Match(s, @"(?<=\:)\d*(?=\])").Value;
                                break;
                        }

                        int _quantity = 0;

                        try { _quantity = Convert.ToInt16(quantityTemp); } catch (Exception e) {  }

                        Triggers.Add(new TriggerCondition(_event, name, _quantity, DateTime.Now));

                    }

                }
            }
        }

        public class TriggerCondition
        {
            public string Event { get; set; }
            public string Name { get; set; }
            public int Quantity { get; set; }
            public DateTime ElapsedMinutesStart { get; set; }
            public string ActionCriteria { get; set; }

            public TriggerCondition()
            {
                Event = "";
                Name = "";
                Quantity = 0;
                ElapsedMinutesStart = DateTime.Now;
                ActionCriteria = "";

                if (Event == "ElapsedMinutes" && ElapsedMinutesStart != null)
                {
                    //Log.FailLog("==== the timer has started ....");
                }
            }

            public TriggerCondition(string eventName, string nameString, int quantity, DateTime elapsedMinutesStart)
            {
                Event = eventName;
                Name = nameString;
                Quantity = quantity;
                ElapsedMinutesStart = elapsedMinutesStart;
                ActionCriteria = "";

                if (Event == "ElapsedMinutes" && ElapsedMinutesStart != null)
                {
                    //Log.FailLog("==== the timer has started ....");
                }
            }

        }

    }
}
