﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bots.DungeonBuddy.Helpers;
using CommonBehaviors.Actions;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.World;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using Logger = Bots.DungeonBuddy.Helpers.Logger;

namespace Fpsware.Library3.Helpers
{
    public static class Avoidance
    {

        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }





        /// <summary>
        /// Find and move to a safe spot away from your current target
        /// </summary>
        /// <param name="minDistanceFromAnyMobs">The minimum distance you must be from any hostile mob</param>
        /// <param name="minDistanceFromCurrentTarget">The minimum distance away from your current target to find a safe spot</param>
        /// <param name="maxDistanceFromCurrentTarget">The maximum distance away from your current target to find a safe spot</param>
        /// <param name="useMageBlink">MAGE ONLY: Use Blink to move towards safe spot</param>
        /// <returns></returns>
        public static Composite MoveToSafeSpot(int minDistanceFromAnyMobs = 30, int minDistanceFromCurrentTarget = 25, int maxDistanceFromCurrentTarget = 40, bool useMageBlink = false)
        {
            WoWPoint safeSpotLocation = WoWPoint.Empty;

            //return new Decorator(ret=> safeSpotLocation != WoWPoint.Empty,
            return new PrioritySelector(

                // Step 1. Find a safe spot
                new Action(context =>
                {
                    SafeArea sa = new SafeArea
                    {
                        MinSafeDistance = minDistanceFromAnyMobs,
                        MinScanDistance = minDistanceFromCurrentTarget,
                        MaxScanDistance = maxDistanceFromCurrentTarget,
                        IncrementScanDistance = 5,
                        RaysToCheck = 36,
                        LineOfSightMob = Me.CurrentTarget,
                        MobToRunFrom = Me.CurrentTarget,
                        CheckLineOfSightToSafeLocation = true,
                        CheckSpellLineOfSightToMob = false
                    };

                    safeSpotLocation = sa.FindLocation();
                    return RunStatus.Failure;

                }),

                // Step 2. Get there
                // A. Mage Blink
                // B. Everyone else, navigation

                new Decorator(ret => !useMageBlink, new Action(context => Navigator.MoveTo(safeSpotLocation))),

                // Mage? Blink to location
                new Decorator(ret => useMageBlink && Me.Class == WoWClass.Mage && SpellManager.CanCast("Blink") && Me.GotTarget && CT.HealthPercent > 45 && GameWorld.IsInLineOfSight(Me.Location, safeSpotLocation),
                    new Sequence(
                        new Action(context => Me.SetFacing(safeSpotLocation)),
                //new WaitContinue(1, ret=> false, new ActionAlwaysSucceed()),
                        new WaitContinue(TimeSpan.FromMilliseconds(400), ret => false, new ActionAlwaysSucceed()),
                        new Action(ctx => SpellManager.Cast("Blink")),
                        new WaitContinue(TimeSpan.FromMilliseconds(400), ret => false, new ActionAlwaysSucceed()),
                        new Action(context => CT.Face())
                //new WaitContinue(1, ret=> false, new ActionAlwaysSucceed())
                    )),

                // Fall through, just try to navigate 
                new Action(context => Navigator.MoveTo(safeSpotLocation))



            );

        }

        public class SafeArea
    {
        public const float OneDegreeAsRadian = 0.0174532925f;

        /// <summary><para>Minimum distance away from character to find safe place</para></summary>
        public float MinScanDistance { get; set; }
        /// <summary><para>Maximum distance away from character to find safe place</para></summary>
        public float MaxScanDistance { get; set; }
        /// <summary><para>Increment added each repetition to MinScanDistance until MaxScanDistance reached.</para></summary>
        public float IncrementScanDistance { get; set; }
        /// <summary><para>Number of evenly spaced checks around perimter.  36 would yield 36 checks around perimeter spaced 10 degrees apart.</para></summary>
        public int RaysToCheck { get; set; }
        /// <summary>Minimum distance from safe spot to nearest enemy</summary>
        public int MinSafeDistance { get; set; }
        /// <summary>Range to keep to LineOfSightMob if CheckRangeToLineOfSightMob</summary>
        public float RangeToLineOfSightMob { get; set; }
        /// <summary>Unit we are trying to get away from</summary>
        public WoWUnit MobToRunFrom { get; set; }
        /// <summary>Unit we need to be able to attack from destination</summary>
        public WoWUnit LineOfSightMob { get; set; }
        /// <summary>Require line of sight from origin to safe location</summary>
        public bool CheckLineOfSightToSafeLocation { get; set; }
        /// <summary>Require spell line of sight from safe location to mob</summary>
        public bool CheckSpellLineOfSightToMob { get; set; }
        /// <summary>Require safe location to be within 40 yds of current target</summary>
        public bool CheckRangeToLineOfSightMob { get; set; }
        /// <summary>Select best navigable point available</summary>
        public bool ChooseSafestAvailable { get; set; }

        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        public SafeArea()
        {
            MinScanDistance = 10;               // minimum distance to scan for safe spot
            MaxScanDistance = 30;               // maximum distance to scan for safe spot
            IncrementScanDistance = 10;             // distance increment for each iteration of safe spot check 
            RaysToCheck = 36;                   // number of checks within 360 degrees (36 = every 10 degrees)
            MinSafeDistance = 15;                       // radius of the minimal safe area at safe spot
            MobToRunFrom = NearestEnemyMobAttackingMe;  // trying to get away from this guy
            LineOfSightMob = Me.CurrentTarget;          // safe spot must have line of sight to this mob

            CheckLineOfSightToSafeLocation = true;
            CheckSpellLineOfSightToMob = LineOfSightMob != null;

            RangeToLineOfSightMob = 0;
            CheckRangeToLineOfSightMob = false;
            
            ChooseSafestAvailable = true;
        }

        public static bool IsEnemy(WoWUnit u)
        {
            if (u == null || !u.CanSelect || !u.Attackable || !u.IsAlive || u.IsNonCombatPet || u.IsCritter)
                return false;

            if (!u.IsPlayer)
                return u.IsHostile || u.Aggro || u.PetAggro;

            WoWPlayer p = u.ToPlayer();

            return p.IsHorde != Me.IsHorde;
        }

        public static IEnumerable<WoWUnit> AllEnemyMobs
        {
            get
            {
                //return ObjectManager.ObjectList.Where(o => o is WoWUnit && Unit.ValidUnit(o.ToUnit()) ).Select(o => o.ToUnit()).ToList();
                return
                    ObjectManager.ObjectList.Where(o => o is WoWUnit && o.ToUnit().IsValidTarget())
                                 .Select(o => o.ToUnit())
                                 .ToList();

            }
        }

        public List<WoWPoint> AllEnemyMobLocationsToCheck
        {
            get
            {
                return (from u in AllEnemyMobs
                        where u != MobToRunFrom && u != LineOfSightMob && u.Distance2DSqr < (65 * 65)
                        select u.Location).ToList();
            }
        }

        public static IEnumerable<WoWUnit> AllEnemyMobsAttackingMe
        {
            get
            {
                return AllEnemyMobs.Where(u =>
                                          u.Combat
                                          && u.CurrentTargetGuid == Me.Guid
                                          && !u.IsPet
                    );
                //&& StyxWoW.Me.Level < (u.Level + (u.Elite ? 12 : 5)));
            }
        }

        public static WoWUnit NearestEnemyMobAttackingMe
        {
            get
            {
                return AllEnemyMobsAttackingMe.OrderBy(uu => uu.Distance2DSqr).FirstOrDefault();
            }
        }

        public static WoWPoint NearestMobLoc(WoWPoint p, IEnumerable<WoWPoint> mobLocs)
        {
            if (!mobLocs.Any())
                return WoWPoint.Empty;

            return mobLocs.OrderBy(u => u.Distance2DSqr(p)).First();
        }

        /// <summary>
        /// locates safe point away from enemies
        /// </summary>
        /// <returns></returns>
        public WoWPoint FindLocation() { return FindLocation(Me.GetTraceLinePos()); }

        public WoWPoint FindLocation(WoWPoint ptOrigin)
        {
            DateTime startFind = DateTime.Now;
            int countPointsChecked = 0;
            int countFailToPointNav = 0;
            int countFailRange = 0;
            int countFailSafe = 0;
            int countFailToPointLoS = 0;
            int countFailToMobLoS = 0;
            double furthestNearMobDistSqr = 0f;
            WoWPoint ptFurthest = WoWPoint.Empty;
            bool reallyCheckRangeToLineOfSightMob = CheckRangeToLineOfSightMob && Me.GotTarget;
            WoWPoint ptAdjOrigin = ptOrigin;
            // ptAdjOrigin.Z += 1f;   // comment out origin adjustment since using GetTraceLinePos()

            float arcIncrement = ((float)Math.PI * 2) / RaysToCheck;
            
            List<WoWPoint> mobLocations = AllEnemyMobLocationsToCheck;
            double minSafeDistSqr = MinSafeDistance * MinSafeDistance;

            float baseDestinationFacing = MobToRunFrom == null ?
                                            Me.RenderFacing + (float)Math.PI
                                            : WoWMathHelper.CalculateNeededFacing(MobToRunFrom.Location, Me.Location);

            // Logger.WriteDebug( Color.Cyan, "SafeArea: search near {0:F0}d @ {1:F1} yds for mob free area", RadiansToDegrees(baseDestinationFacing), MinSafeDistance);

            for (int arcIndex = 0; arcIndex < RaysToCheck; arcIndex++)
            {
                // rather than tracing around the circle, toggle between clockwise and counter clockwise for each test
                // .. so we favor a position furthest away from mob
                float checkFacing = baseDestinationFacing;
                if ((arcIndex & 1) == 0)
                    checkFacing += arcIncrement * (arcIndex >> 1);
                else
                    checkFacing -= arcIncrement * ((arcIndex >> 1) + 1);

                for (float distFromOrigin = MinScanDistance; distFromOrigin <= MaxScanDistance; distFromOrigin += IncrementScanDistance)
                {
                    countPointsChecked++;

                    WoWPoint ptDestination = ptOrigin.RayCast(checkFacing, distFromOrigin);
                    if (!Navigator.CanNavigateFully(Me.Location, ptDestination))
                    {
                        // Logger.WriteDebug( Color.Cyan, "Safe Location failed navigation check for degrees={0:F1} dist={1:F1}", RadiansToDegrees(checkFacing), distFromOrigin);
                        countFailToPointNav++;
                        continue;
                    }

                    WoWPoint ptNearest = NearestMobLoc(ptDestination, mobLocations);
                    if (ptNearest == WoWPoint.Empty)
                    {
                        if (furthestNearMobDistSqr < minSafeDistSqr)
                        {
                            furthestNearMobDistSqr = minSafeDistSqr;
                            ptFurthest = ptDestination;     // set best available if others fail
                        }
                    }
                    else
                    {
                        double mobDistSqr = ptDestination.Distance2DSqr(ptNearest);
                        if (furthestNearMobDistSqr < mobDistSqr)
                        {
                            furthestNearMobDistSqr = mobDistSqr;
                            ptFurthest = ptDestination;     // set best available if others fail
                        }
                        if (mobDistSqr <= minSafeDistSqr)
                        {
                            countFailSafe++;
                            continue;
                        }
                    }

                    if (reallyCheckRangeToLineOfSightMob && RangeToLineOfSightMob < ptDestination.Distance(LineOfSightMob.Location) - LineOfSightMob.MeleeRange())
                    {
                        countFailRange++;
                        continue;
                    }

                    if (CheckLineOfSightToSafeLocation)
                    {
                        WoWPoint ptAdjDest = ptDestination;
                        ptAdjDest.Z += 1f;
                        if (!GameWorld.IsInLineOfSight(ptAdjOrigin, ptAdjDest))
                        {
                            // Logger.WriteDebug( Color.Cyan, "Mob-free location failed line of sight check for degrees={0:F1} dist={1:F1}", degreesFrom, distFromOrigin);
                            countFailToPointLoS++;
                            continue;
                        }
                    }

                    if (CheckSpellLineOfSightToMob && LineOfSightMob != null)
                    {
                        if (!GameWorld.IsInLineOfSpellSight(ptDestination, LineOfSightMob.GetTraceLinePos()))
                        {
                            if (!GameWorld.IsInLineOfSight(ptDestination, LineOfSightMob.GetTraceLinePos()))
                            {
                                // Logger.WriteDebug( Color.Cyan, "Mob-free location failed line of sight check for degrees={0:F1} dist={1:F1}", degreesFrom, distFromOrigin);
                                countFailToMobLoS++;
                                continue;
                            }
                        }
                    }

                    Log.DebugLog("SafeArea: Found mob-free location ({0:F1} yd radius) at degrees={1:F1} dist={2:F1} on point check# {3} at {4}, {5}, {6}", MinSafeDistance, WoWMathHelper.RadiansToDegrees(checkFacing), distFromOrigin, countPointsChecked, ptDestination.X, ptDestination.Y, ptDestination.Z);
                    Log.DebugLog("SafeArea: processing took {0:F0} ms", (DateTime.Now - startFind).TotalMilliseconds);
                    return ptDestination;
                }
            }

            Log.DebugLog("SafeArea: No mob-free location ({0:F1} yd radius) found within {1:F1} yds ({2} checked, {3} nav, {4} not safe, {5} range, {6} pt los, {7} mob los)", MinSafeDistance, MaxScanDistance, countPointsChecked, countFailToPointNav, countFailSafe, countFailRange, countFailToPointLoS, countFailToMobLoS);
            if (ChooseSafestAvailable && ptFurthest != WoWPoint.Empty)
            {
                Log.DebugLog("SafeArea: choosing best available spot in {0:F1} yd radius where closest mob is {1:F1} yds", MinSafeDistance, Math.Sqrt(furthestNearMobDistSqr));
                Log.DebugLog("SafeArea: processing took {0:F0} ms", (DateTime.Now - startFind).TotalMilliseconds);
                return ChooseSafestAvailable ? ptFurthest : WoWPoint.Empty;
            }

            Log.DebugLog("SafeArea: processing took {0:F0} ms", (DateTime.Now - startFind).TotalMilliseconds);
            return WoWPoint.Empty;
        }


    }




        public static WoWPoint CalculatePointBehindMe()
        {
            return StyxWoW.Me.Location.RayCast(StyxWoW.Me.Rotation + WoWMathHelper.DegreesToRadians(180), 25f);
        }

        public static WoWPoint CalculatePointBehindMe(float distance)
        {
            return StyxWoW.Me.Location.RayCast(StyxWoW.Me.Rotation + WoWMathHelper.DegreesToRadians(180), distance);
        }

        public static WoWPoint CalculatePointBehindMe(float distance, float degreesToRadians)
        {
            return StyxWoW.Me.Location.RayCast(StyxWoW.Me.Rotation + WoWMathHelper.DegreesToRadians(degreesToRadians), distance);
        }






        private static List<WoWPoint> _safespots = null;
        private static double AvoidMobMinRange { get; set; }
        // <returns>returns WoWPoint away from the TARGETUNIT</returns>
        public static WoWPoint PreferredSafespot(WoWUnit targetunit, bool isGroundEffectProblem = false)
        {
            IEnumerable<WoWPoint> preferredSafespotOrder = from spot in _safespots
                // The +1 term guarantees distances are never less than 1.  This prevents
                // degenerate cases in preference evaluation for distances less than one, or zero.
                let myDistanceToSpot = Me.Location.Distance(spot) + 1
                let mobDistanceToSpot = targetunit.Location.Distance(spot) + 1
                orderby // preference ordering equation:
                    
                    // prefer spots that are close to me, but not mob
                    (myDistanceToSpot / mobDistanceToSpot)
                    
                        // If ground effect problem, avoid any nearby spots
                    + ((isGroundEffectProblem && (myDistanceToSpot < AvoidMobMinRange)) ? 100 : 0)
                    
                        // prefer spots away from mob
                    + ((targetunit.Location.Distance(spot) < AvoidMobMinRange) ? 1000 : 0)
                    
                        // prefer spots the toon doesn't have to run through the mob
                    + (WoWMathHelper.IsInPath(targetunit, Me.Location, spot) ? 10000 : 0)
                select spot;

            return preferredSafespotOrder.FirstOrDefault();
        }

        /// <returns>Returns a WoWPoint at DISTANCE behind UNIT</returns>
        public static WoWPoint SafespotBehindMob(WoWUnit unit, double distance)
        {
            return unit.Location.RayCast(unit.Rotation + (float)Math.PI, (float)distance);
        }

        /// <summary>
        /// Returns the nearest mob in TARGETS that has an aura of one of the AURAIDS
        /// </summary>
        /// <param name="targets"></param>
        /// <param name="auraIds"></param>
        /// <returns>may return null</returns>
        public static WoWUnit NearestMobWithAura(IEnumerable<WoWUnit> targets, IEnumerable<int> auraIds)
        {
            IEnumerable<WoWUnit> targetQuery =
                from target in targets
                where target.Auras.Values.Any(a => auraIds.Contains(a.SpellId))
                orderby target.Distance
                select target;

            return (targetQuery.FirstOrDefault());
        }


        /// <summary>
        /// Returns the nearest mob in TARGETS casting one of the SPELLIDs
        /// </summary>
        /// <param name="targets"></param>
        /// <param name="spellIds"></param>
        /// <returns>may return null</returns>
        public static WoWUnit NearestMobCastingSpell(IEnumerable<WoWUnit> targets, int[] spellIds)
        {
            IEnumerable<WoWUnit> targetQuery =
                from target in targets
                where UnitSpellFromCastingIds(target, spellIds) != null
                orderby target.Distance
                select target;

            return (targetQuery.FirstOrDefault());
        }

        /// <returns>Returns the WoWSpell if UNIT is casting one of the SPELLIDS; otherwise, returns null</returns>
        private static WoWSpell UnitSpellFromCastingIds(WoWUnit unit, IEnumerable<int> spellIds)
        {
            return (unit.IsCasting ? (spellIds.Contains(unit.CastingSpellId) ? unit.CastingSpell : null)
                : unit.IsChanneling ? (spellIds.Contains(unit.ChanneledCastingSpellId) ? unit.ChanneledSpell : null)
                : null);
        }
    }
}
