﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System;
using System.Linq;
using System.Text.RegularExpressions;
using Fpsware.Library3.Extensions;
using Styx;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Library3.Helpers
{
    public static class CLC
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return StyxWoW.Me.CurrentTarget; } }
        private static WoWUnit Pet { get { return StyxWoW.Me.Pet; } }

        public static bool ResultOK(this String settingName)
        {
            return ResultOK(settingName, 100);
        }

        public static bool OK(this String settingName)
        {
            return ResultOK(settingName, 100);
        }

        public static bool ResultOK(this String settingName, int maxDistanceFromUnit = 60, WoWUnit fromUnit = null)
        {
            string s = settingName.ToUpper();

            // Using a switch on the most common strings for performance
            switch (s)
            {
                case "ALWAYS":
                case "ENABLED":
                case "IMMEDIATELY":
                case "AUTOMATIC":
                case "AUTOMATICALLY":
                case "AUTOMATICALY":
                case "ALWAYS SHOW":
                    return true;

                case "NEVER":
                case "DISABLED":
                case "IGNORE":
                case "MANUAL":
                case "IGNORED":
                case "DON'T SHOW":
                    return false;
            }

            if ((s.Contains("BOSS ONLY") || s.Contains("ON BOSS")) && CT != null && WoWUnitExt.IsBoss(CT)) return true;

            if (s.Contains("ON ADDS")) return AddsOk("2+ adds", false, maxDistanceFromUnit, fromUnit);
            if (s.Contains("+ ADDS") || s.Contains("+ HOSTILE")) return AddsOk(settingName, false, maxDistanceFromUnit, fromUnit);

            if (s.Contains("WITHIN MELEE") && CT != null && (CT.IsWithinMeleeRange || CT.WithinInteractRange)) return true;
            if (s.Contains("IS FLEEING") && CT != null && CT.Fleeing) return true;
            if (s.Contains("IS CASTING") && CT != null && CT.IsCasting && CT.CastingSpell != null && CT.CastingSpell.CastTime > 1) return true;

            if ((s.Contains("IS PLAYER") || s.Contains("ON PLAYER")) && CT != null && CT.IsPlayer) return true;
            if (s.Contains("+ PLAYER")) return AddsOk(settingName, true, maxDistanceFromUnit);
            if (s.Contains("OUT OF COMBAT") && !Me.Combat) return true;
            //if (s.Contains("DON'T SHOW")) return false;

            return false;


        }


        /// <summary>
        /// Returns TRUE if the number of aggro units (or players) >= the parsed quantity. Eg '3+ Adds' requires 3 or more mobs to be in combat with us, pet, party or raid members
        /// </summary>
        /// <param name="settingName">Setting string to be checked</param>
        /// <param name="playersOnly">Check for PLAYERS only</param>
        /// <param name="maxDistanceFromUnit">Only include units equal or less than this distance away from the 'fromUnit'</param>
        /// <param name="fromUnit">The unit to use as a reference location for checking maxDistanceAway, if null the player is used</param>
        /// <returns></returns>
        //public static bool AddsOk(string settingName, bool playersOnly = false, int maxDistanceAway = 60, WoWUnit fromUnit = null)
        private static bool AddsOk(this string settingName, bool playersOnly = false, int maxDistanceFromUnit = 60, WoWUnit fromUnit = null)
        {
            WoWUnit unit = fromUnit ?? Me;
            int countOfAgroUnits = 0;
            int minCount = ResultNumber(settingName);

            if (!playersOnly)
            {
                countOfAgroUnits = Units.Hostile.UnitsInCombatTargetingAny.Count(u => u.Location.DistanceSqr(unit.Location) <= (maxDistanceFromUnit * maxDistanceFromUnit));
                if (countOfAgroUnits >= minCount) return true;
            }

            countOfAgroUnits = playersOnly
                ? Units.Hostile.UnitsInCombatTargetingAny.Count( u => u.IsPlayer && u.Location.Distance(unit.Location) < maxDistanceFromUnit)
                : Units.Hostile.UnitsInCombatTargetingAny.Count( u => u.Location.Distance(unit.Location) < maxDistanceFromUnit);

            return minCount > 0 && countOfAgroUnits >= minCount;
        }

        /// <summary>
        /// Returns the Integer found in the string. Eg '5+ Adds' would return 5 as an int32
        /// </summary>
        /// <param name="settingName"></param>
        /// <returns></returns>
        public static int ResultNumber(this String settingName)
        {
            int result;

            string numFromString = Regex.Match(settingName, @"\d+").Value;
            return Int32.TryParse(numFromString, out result) ? result : 99;
        }
    }
}
