﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Styx;
using Styx.Helpers;
using Styx.Patchables;
using Styx.Pathing;
using Styx.WoWInternals;
using Styx.WoWInternals.World;

namespace Fpsware.Library3.Helpers
{
    public static class MapHelpers
    {

        public static bool IsFacing2D(WoWPoint me, float myFacingRadians, WoWPoint target, float arcRadians)
        {
            me.Z = target.Z = 0;
            arcRadians = WoWMathHelper.NormalizeRadian(arcRadians);
            float num = WoWMathHelper.CalculateNeededFacing(me, target);
            float num2 = WoWMathHelper.NormalizeRadian(num - myFacingRadians);
            if (num2 > 3.1415926535897931)
            {
                num2 = (float)(6.2831853071795862 - num2);
            }
            bool result = (num2 <= arcRadians / 2f);
            return result;
        }

        public static WoWPoint StringToWoWPoint(string location)
        {
            WoWPoint loc = WoWPoint.Zero;
            Regex pattern = new Regex(@"-?\d+\.?(\d+)?", RegexOptions.CultureInvariant);
            MatchCollection matches = pattern.Matches(location);
            if (matches.Count < 3) return loc;

            loc.X = Convert.ToSingle(matches[0].ToString());
            loc.Y = Convert.ToSingle(matches[1].ToString());
            loc.Z = Convert.ToSingle(matches[2].ToString());
            return loc;
        }

        public static double MyDistanceFromGround
        {
            get
            {
                WoWPoint hitLocation;
                WoWPoint upper = StyxWoW.Me.Location.Add(0.0f, 0.0f, 2000.0f);
                WoWPoint lower = StyxWoW.Me.Location.Add(0.0f, 0.0f, -2000.0f);

                bool result = (GameWorld.TraceLine(upper, lower, (GameWorld.CGWorldFrameHitFlags.HitTestGround), out hitLocation));

                return result ? StyxWoW.Me.Location.Z - hitLocation.Z : 0;
            }
        }


        //Test if a WoWPoint is located within a Polygon (can be used with continent borders OR ClusterPoints)
        public static bool PointInPolygon(WoWPoint p, WoWPoint[] poly)
        {
            bool inside = false;
            if (poly.Count() < 3) { return inside; }

            WoWPoint oldPoint = new WoWPoint(poly[poly.Count() - 1].X, poly[poly.Count() - 1].Y, poly[poly.Count() - 1].Z);

            for (int i = 0; i < poly.Count(); i++)
            {
                WoWPoint newPoint = new WoWPoint(poly[i].X, poly[i].Y, poly[i].Z);

                WoWPoint p1;
                WoWPoint p2;
                if (newPoint.X > oldPoint.X)
                {
                    p1 = oldPoint;
                    p2 = newPoint;
                }
                else
                {
                    p1 = newPoint;
                    p2 = oldPoint;
                }

                if ((newPoint.X < p.X) == (p.X <= oldPoint.X)
                && ((double)p.Y - (double)p1.Y) * (double)(p2.X - p1.X)
                 < ((double)p2.Y - (double)p1.Y) * (double)(p.X - p1.X))
                {
                    inside = !inside;
                }

                oldPoint = newPoint;
            }

            return inside;
        }

        /// <summary>
        /// Transforms a point (x,y) on a Map to WoWPoint for example: /script local posX, posY = GetPlayerMapPosition("player");print(posX);print(posY);
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="mapId"></param>
        /// <returns>WoWPoint</returns>
        public static WoWPoint MapToWorldCoords(float x, float y, uint mapId)
        {
            var worldPoint = new WoWPoint();
            WoWDb.DbTable worldMapArea = StyxWoW.Db[ClientDb.WorldMapArea];
            WoWDb.Row worldMapAreaFields = worldMapArea.GetRow(mapId);
            var ay = worldMapAreaFields.GetField<float>(4);
            var by = worldMapAreaFields.GetField<float>(5);
            var ax = worldMapAreaFields.GetField<float>(6);
            var bx = worldMapAreaFields.GetField<float>(7);
            worldPoint.X = ax + (y * (bx - ax));
            worldPoint.Y = ay + (x * (by - ay));
            try
            {
                worldPoint.Z = Navigator.FindHeights(worldPoint.X, worldPoint.Y).Min();
            }
            catch
            {
                return TryGetHeight(worldPoint);
            }

            return worldPoint;
        }


        /// <summary>
        /// Return us a new Z coordinate
        /// </summary>
        /// <param name="point">The point of which we should get the new height</param>
        /// <returns></returns>
        public static WoWPoint TryGetHeight(WoWPoint point)
        {
            const float PIx2 = (float)Math.PI * 2f;
            const int step = 20;

            for (int d = 5; d <= 50; d += 5)
            {
                for (int i = 0; i < 20; i++)
                {
                    WoWPoint newPoint = point.RayCast((i * PIx2) / step, d);
                    try
                    {
                        newPoint.Z = Navigator.FindHeights(newPoint.X, newPoint.Y).Min();
                        return newPoint;
                    }
                    catch { }
                }
            }
            return point;
        }


        /// <summary>
        /// Calculates a new point in WoW where we can move to
        /// </summary>
        /// <param name="mapID">mapid where the point should be in</param>
        /// <param name="currentLocation">our current location because we need a starting point, can also be used to generate a list of points</param>
        /// <param name="distanceFromCurrentLocation">min distance from current location</param>
        /// <param name="headingFromCurrentLocation">heading from current location (in which direction should we move)</param>
        /// <returns>Returns a point</returns>
        public static WoWPoint CalculateNewPoint(uint mapID, WoWPoint currentLocation, int distanceFromCurrentLocation = 0, int headingFromCurrentLocation = 0)
        {
            const int defaultdistance = 200;
            Random random = new Random();
            int randomHeading = random.Next(0, headingFromCurrentLocation == 0 ? 360 : headingFromCurrentLocation);
            int randomDistance = random.Next(0, distanceFromCurrentLocation == 0 ? distanceFromCurrentLocation : defaultdistance);
            WoWPoint t = currentLocation.RayCast(randomHeading, randomDistance);
            return t;
        }

        public static uint FindMapId(string localMapName)
        {
            WoWDb.DbTable t = StyxWoW.Db[ClientDb.WorldMapArea];
            int max = t.MaxIndex;
            for (int i = t.MinIndex; i <= max; )
            {
                WoWDb.Row r = t.GetRow((uint)i);
                string mapName = StyxWoW.Memory.ReadString(r.GetField<IntPtr>(3), Encoding.ASCII);
                if (mapName == localMapName)
                {
                    return (uint)i;
                }
                //Log.InitLog(mapName);

                if (i < max) i = r.GetField<int>(14); else break;
            }
            return 0;
        }

        public static uint GetContinentId(uint mapID)
        {
            WoWDb.DbTable t = StyxWoW.Db[ClientDb.WorldMapArea];
            WoWDb.Row r = t.GetRow(mapID);
            return r.GetField<uint>(1);
        }

        public static float DistanceFromGround(WoWPoint location)
        {
            return location.Z > FindGroundLocation(location).Z
                       ? location.Z - FindGroundLocation(location).Z
                       : FindGroundLocation(location).Z - location.Z;
        }

        public static bool IsObjectUnderground(this WoWPoint location)
        {
            WoWPoint hitLocation;
            WoWPoint locationUpper = location.Add(0.0f, 0.0f, 2000.0f);
            WoWPoint locationLower = location.Add(0.0f, 0.0f, -2000.0f);

            bool hitResult = (GameWorld.TraceLine(locationUpper, locationLower, (GameWorld.CGWorldFrameHitFlags.HitTestGround), out hitLocation));

            if (hitResult)
            {
                if ((int)hitLocation.Z - 1 > (int)location.Z) return true;
            }

            return false;
        }

        public static bool IsObjectInWater(this WoWPoint objLocation)
        {
            float distance = 1.5f;
            if (StyxWoW.Me.Class == WoWClass.Druid) distance = 1.25f;
            if (!StyxWoW.Me.Mounted && StyxWoW.Me.IsSwimming) distance = 0f;

            return
                GameWorld.TraceLine(new WoWPoint(objLocation.X, objLocation.Y, (objLocation.Z + distance)),
                                    new WoWPoint(objLocation.X, objLocation.Y, (objLocation.Z + 10f)),
                                    GameWorld.CGWorldFrameHitFlags.HitTestLiquid) ||

                GameWorld.TraceLine(new WoWPoint(objLocation.X, objLocation.Y, (objLocation.Z + distance)),
                                    new WoWPoint(objLocation.X, objLocation.Y, (objLocation.Z + 10f)),
                                    GameWorld.CGWorldFrameHitFlags.HitTestLiquid2);
        }

        public static IEnumerable<WoWPoint> GetAllHeights(WoWPoint point)
        {
            List<WoWPoint> heights = new List<WoWPoint>();
            using (StyxWoW.Memory.AcquireFrame())
            {
                heights.AddRange(Navigator.FindHeights(point.X, point.Y).Select(height => new WoWPoint(point.X, point.Y, height)));
            }
            return heights;
        }

        public static WoWPoint FindGroundLocation(WoWPoint location)
        {
            return StyxWoW.Me.IsFlying
                ? GetAllHeights(location).OrderBy(x => x.Distance(location)).FirstOrDefault()
                : GetAllHeights(location)
                    .OrderBy(x => x.Distance(location))
                    .Where(x => Navigator.CanNavigateFully(location, x))
                    .FirstOrDefault(x => x.Z <= (location.Z + 1.5));
        }
    }
}
