﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using CommonBehaviors.Actions;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.World;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using ActionFail = Fpsware.Library3.Extensions.TreeSharpExt.ActionFail;

namespace Fpsware.Library3.Helpers
{
    public static class Movement
    {
        public delegate T Selection<out T>(object context);
        private static LocalPlayer Me { get { return (StyxWoW.Me); } }

        public enum MovementProviderType
        {
            Flightor,
            Navigator,
            ClickToMove
        }

        public enum MountStrategyType
        {
            CancelShapeshift,
            Dismount,
            DismountOrCancelShapeshift,
            MountGround,
            MountFlying,
            Mount,
            None
                
        }

        public enum StopMovingWhen
        {
            Now,
            AtLocation,
            InRangeOfUnit,
            InRangeOfLocation,
            InMeleeRangeOfUnit,
            InLoSOfUnit
            
        }


        /// <summary>
        /// Find and move to a safe spot away from your current target
        /// </summary>
        /// <param name="unit"></param>
        /// <param name="minDistanceFromAnyMobs">The minimum distance you must be from any hostile mob</param>
        /// <param name="minDistanceFromCurrentTarget">The minimum distance away from your current target to find a safe spot</param>
        /// <param name="maxDistanceFromCurrentTarget">The maximum distance away from your current target to find a safe spot</param>
        /// <param name="useMageBlink">MAGE ONLY: Use Blink to move towards safe spot</param>
        /// <returns></returns>
        public static Composite MoveToSafeSpot(Selection<WoWUnit> unit, Selection<int> minDistanceFromAnyMobs, Selection<int> minDistanceFromCurrentTarget, Selection<int> maxDistanceFromCurrentTarget, bool useMageBlink = false)
        {
            WoWPoint safeSpotLocation = WoWPoint.Empty;

            //return new Decorator(ret=> safeSpotLocation != WoWPoint.Empty,
            return new PrioritySelector(

                // Step 1. Find a safe spot
                new Action(context =>
                {
                    Avoidance.SafeArea sa = new Avoidance.SafeArea
                    {
                        MinSafeDistance = minDistanceFromAnyMobs(context),
                        MinScanDistance = minDistanceFromCurrentTarget(context),
                        MaxScanDistance = maxDistanceFromCurrentTarget(context),
                        IncrementScanDistance = 5,
                        RaysToCheck = 36,
                        LineOfSightMob = unit(context),
                        MobToRunFrom = unit(context),
                        CheckLineOfSightToSafeLocation = true,
                        CheckSpellLineOfSightToMob = false
                    };

                    safeSpotLocation = sa.FindLocation();
                    return RunStatus.Failure;

                }),

                // Step 2. Get there
                // A. Mage Blink
                // B. Everyone else, navigation

                new Decorator(ret => !useMageBlink, new Sequence(
                    new Action(context =>
                    {
                        Log.Info("==== we should be moving to a safe spot, away from " + unit(context));
                        Log.Info("=== safe spot is " + Me.Location.Distance(safeSpotLocation).ToString());
                        Navigator.MoveTo(safeSpotLocation);
                    })
                    )),

                // Mage? Blink to location
                new Decorator(ret => useMageBlink && Me.Class == WoWClass.Mage && SpellManager.CanCast("Blink") && Me.GotTarget && unit(ret).HealthPercent > 45 && GameWorld.IsInLineOfSight(Me.Location, safeSpotLocation),
                    new Sequence(
                        new Action(context => Me.SetFacing(safeSpotLocation)),
                        new WaitContinue(TimeSpan.FromMilliseconds(400), ret => false, new ActionAlwaysSucceed()),
                        new Action(context => SpellManager.Cast("Blink")),
                        new WaitContinue(TimeSpan.FromMilliseconds(400), ret => false, new ActionAlwaysSucceed()),
                        new Action(ret => unit(ret).Face())
                    )),

                // Fall through, just try to navigate 
                new Action(context => Navigator.MoveTo(safeSpotLocation))



            );

        }







        internal static float MeleeRange(this WoWUnit unit)
        {
            if (unit == null) return 0f;
            if (unit.IsPlayer) return 3.5f;

            return Math.Max(5f, StyxWoW.Me.CombatReach + 1.3333334f + unit.CombatReach);
        }

        public static WoWPoint CalculatePointBehindTarget(WoWUnit unit)
        {
            float facing = unit.Rotation;
            facing += WoWMathHelper.DegreesToRadians(180); 
            facing = WoWMathHelper.NormalizeRadian(facing);

            return unit.Location.RayCast(facing, unit.MeleeRange() - 1f);
        }

        public static WoWPoint CalculatePointBehindTarget(WoWUnit unit, float distance)
        {
            float facing = unit.Rotation;
            facing += WoWMathHelper.DegreesToRadians(180);
            facing = WoWMathHelper.NormalizeRadian(facing);

            return unit.Location.RayCast(facing, distance);
        }


        /*
        public static WoWPoint CalculatePointBehindTarget(WoWUnit unit)
        {
            return unit.Location.RayCast(unit.Rotation + WoWMathHelper.DegreesToRadians(180), unit.MeleeRange() - 0.5f);
        }
         */

        /// <summary>
        /// Calculates a WoWPoint from the target given distance and rotation. DegreesToRadians (0 = in front, 90 = right side, 180 = behind, 270 = left side)
        /// </summary>
        /// <param name="unit"></param>
        /// <param name="distanceFromTarget"></param>
        /// <param name="degreesToRadians"></param>
        /// <returns></returns>
        public static WoWPoint CalculatePointFromTarget(WoWUnit unit, float distanceFromTarget, float degreesToRadians)
        {
            return unit.Location.RayCast(unit.Rotation + WoWMathHelper.DegreesToRadians(degreesToRadians), distanceFromTarget);
        }


        /// <summary>
        /// Simply just stop moving (also waits 250ms to ensure it happens)
        /// </summary>
        public class MoveStop : PrioritySelector
        {
            public MoveStop()
            {
                Children = CreateChildren();
            }

            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
                    new Decorator(context => WoWMovement.ActiveMover.IsMoving,
                        new Sequence(
                            new Action(context => Navigator.PlayerMover.MoveStop()),
                            new Wait(TimeSpan.FromMilliseconds(250), context => WoWMovement.ActiveMover.IsMoving, new ActionAlwaysSucceed())
                            ))
                };
            }
        }
        
        public class StopMovingPS : PrioritySelector
        {
            public Selection<StopMovingWhen> StopMovingWhenStrategyDelegate { get; set; }
            public Selection<WoWUnit> WoWUnitDelegate { get; set; }
            public Selection<WoWPoint> WoWLocationDelegate { get; set; }
            public Selection<int> RangeFromUnitDelegate { get; set; }
            

            private StopMovingWhen CachedStopMovingWhenStrategy { get; set; }
            private WoWUnit CachedWoWUnit { get; set; }
            private int CachedRange { get; set; }
            private WoWPoint CachedWoWLocation { get; set; }

            /// <summary>
            /// Stop moving immediately
            /// </summary>
            public StopMovingPS()
            {
                StopMovingWhenStrategyDelegate = (context => StopMovingWhen.Now);
                WoWUnitDelegate = (context => null);
                WoWLocationDelegate = (context => Me.Location);
                RangeFromUnitDelegate = (context => 10);
                
                Children = CreateChildren();
            }

            public StopMovingPS(Selection<StopMovingWhen> whenToStopMoving, Selection<WoWUnit> unit, Selection<int> range , Selection<WoWPoint> location = null )
            {
                StopMovingWhenStrategyDelegate = whenToStopMoving;
                WoWUnitDelegate = unit;
                WoWLocationDelegate = location ?? (context => WoWPoint.Empty);
                RangeFromUnitDelegate = range;

                Children = CreateChildren();
            }

            /// <summary>
            /// Stop moving when in range of the unit
            /// </summary>
            /// <param name="unit"></param>
            /// <param name="range"></param>
            public StopMovingPS(Selection<WoWUnit> unit, Selection<int> range)
            {
                StopMovingWhenStrategyDelegate = (context => StopMovingWhen.InRangeOfUnit);
                WoWUnitDelegate = unit;
                RangeFromUnitDelegate = range;
                WoWLocationDelegate = (context => WoWPoint.Empty);

                Children = CreateChildren();
            }

            public StopMovingPS(Selection<WoWUnit> unit)
            {
                StopMovingWhenStrategyDelegate = (context => StopMovingWhen.InMeleeRangeOfUnit);
                WoWUnitDelegate = unit;
                RangeFromUnitDelegate = (context => 2);
                WoWLocationDelegate = (context => WoWPoint.Empty);

                Children = CreateChildren();
            }

            /// <summary>
            /// Stop moving when in range of the location
            /// </summary>
            /// <param name="location"></param>
            /// <param name="range"></param>
            public StopMovingPS(Selection<WoWPoint> location, Selection<int> range)
            {
                StopMovingWhenStrategyDelegate = (context => StopMovingWhen.InRangeOfLocation);
                WoWLocationDelegate = location;
                RangeFromUnitDelegate = range;
                WoWUnitDelegate = (context => Me);

                Children = CreateChildren();
            }

            public StopMovingPS(Selection<WoWPoint> location)
            {
                StopMovingWhenStrategyDelegate = (context => StopMovingWhen.AtLocation);
                WoWLocationDelegate = location;
                RangeFromUnitDelegate = (context => 2);
                WoWUnitDelegate = (context => Me);

                Children = CreateChildren();
            }

            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
                    new PrioritySelector(
                        new ActionFail(context =>
                        {
                            CachedStopMovingWhenStrategy = StopMovingWhenStrategyDelegate(context);
                            CachedWoWUnit = WoWUnitDelegate(context);
                            CachedWoWLocation = WoWLocationDelegate(context);
                            CachedRange = RangeFromUnitDelegate(context);
                        }
                    )),

                    // Make sure we're moving otherwise nothing to do
                    new Decorator(ret=> WoWMovement.ActiveMover.IsMoving,
                        new PrioritySelector(

                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.Now, new MoveStop()),
                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.InLoSOfUnit && CachedWoWUnit != null && CachedWoWUnit.InLineOfSpellSight, new MoveStop()),
                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.InMeleeRangeOfUnit && CachedWoWUnit != null && CachedWoWUnit.Distance <= CachedWoWUnit.MeleeDistance(), new MoveStop()),
                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.InRangeOfLocation && Me.Location.Distance(CachedWoWLocation) <= CachedRange, new MoveStop()),
                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.AtLocation && Me.Location.Distance(CachedWoWLocation) <=1, new MoveStop()),
                            new Decorator(ret=> CachedStopMovingWhenStrategy == StopMovingWhen.InRangeOfUnit && CachedWoWUnit != null && CachedWoWUnit.Distance <= CachedRange, new MoveStop())
                            
                        ))
                };
            }



        }

        public class MoveTo : PrioritySelector
        {
            public Selection<MountStrategyType> MountStrategyDelegate { get; set; }
            public Selection<MovementProviderType> MovementProviderDelegate { get; set; }
            public Selection<WoWPoint> DestinationDelegate { get; set; }
            public Selection<bool> FanoutDelegate { get; set; }


            private MountStrategyType CachedMountStrategy { get; set; }
            private MovementProviderType CachedMovementProvider{ get; set; }
            private WoWPoint CachedDestination { get; set; }

            private bool CachedFanout { get; set; }

            public MoveTo(Selection<WoWPoint> destination)
            {
                MountStrategyDelegate = (context => MountStrategyType.None);
                MovementProviderDelegate = (context => MovementProviderType.Navigator);
                FanoutDelegate = (context => false);

                DestinationDelegate = destination;

                Children = CreateChildren();
            }

            public MoveTo(Selection<WoWPoint> destination, Selection<MovementProviderType> movementProviderType)
            {
                MovementProviderDelegate = movementProviderType ?? (context => MovementProviderType.Flightor);
                MountStrategyDelegate = (context => MountStrategyType.MountFlying);
                FanoutDelegate = (context => false);
                DestinationDelegate = destination;

                Children = CreateChildren();
            }

            public MoveTo(Selection<WoWPoint> destination, Selection<MovementProviderType> movementProviderType, Selection<MountStrategyType> mountStrategyType, Selection<bool> fanout)
            {
                MovementProviderDelegate = movementProviderType ?? (context => MovementProviderType.Flightor);
                MountStrategyDelegate = mountStrategyType ?? (context => MountStrategyType.MountFlying);
                FanoutDelegate = fanout ?? (context => false);
                
                DestinationDelegate = destination;

                Children = CreateChildren();
            }

            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
                    new PrioritySelector(
                        new ActionFail(context =>
                        {
                            CachedDestination = DestinationDelegate(context);
                            CachedMountStrategy = MountStrategyDelegate(context);
                            CachedMovementProvider = MovementProviderDelegate(context);
                            CachedFanout = FanoutDelegate(context);
                        }
                    ),
                      

                    new Decorator(context=> CachedMountStrategy != MountStrategyType.None && !Me.Combat, new MountStrategyPS(ret=> CachedMountStrategy)),

                    // Lets get to moving shall we
                    new Decorator(context => !Navigator.AtLocation(CachedDestination),
                        new PrioritySelector(
                            new Switch<MovementProviderType>(context => CachedMovementProvider,

                                new SwitchArgument<MovementProviderType>(MovementProviderType.Flightor,
                                    new PrioritySelector(
                                        TryFlightor()
                                        //TryNavigator()
                                    )),

                                new SwitchArgument<MovementProviderType>(MovementProviderType.Navigator,
                                    new PrioritySelector(
                                        TryNavigator()
                                    )),

                                new SwitchArgument<MovementProviderType>(MovementProviderType.ClickToMove,
                                    new PrioritySelector(
                                        TryClickToMove()
                                    ))
                    )))
                    )
                };
            }



            private Composite TryFlightor()
            {
                return new Decorator(context => WoWMovement.ActiveMover.MovementInfo.CanFly || WoWMovement.ActiveMover.IsSwimming,
                    new PrioritySelector(
                        new Action(context => Flightor.MoveTo(CachedDestination, 30.0f, true))
                    ));
            }

            private Composite TryClickToMove()
            {
                return new PrioritySelector(
                    
                    // NB: Do not 'dismount' for CtM.  We may be using it for aerial navigation, also.
                    // If Navigator can generate a parital path for us, take advantage of it...
                    new Action(context =>
                    {
                        WoWPoint tempDestination =
                            Navigator.GeneratePath(WoWMovement.ActiveMover.Location, CachedDestination)
                            .Where(p => !Navigator.AtLocation(p))
                            .DefaultIfEmpty(CachedDestination)
                            .FirstOrDefault();

                        WoWMovement.ClickToMove(tempDestination);
                    })
                );
            }


            private Composite TryNavigator()
            {
                return new PrioritySelector(

                    // If we are flying, land and set up for ground travel...
                    //new Decorator(context => WoWMovement.ActiveMover.IsFlying, new LandAndDismount("Preparing for ground travel")),

                    // If we can navigate to destination, use navigator...
                    new Decorator( context => Navigator.CanNavigateFully(WoWMovement.ActiveMover.Location, CachedDestination),
                        new PrioritySelector(
                            new Action(context =>
                            {
                                MoveResult moveResult = Navigator.MoveTo(!CachedFanout ? CachedDestination : CachedDestination.FanOutRandom(10));
                                return Navigator.GetRunStatusFromMoveResult(moveResult);
                            })
                    )),

                    new ActionFail(context => Log.FailLog("Navigator unable to move to destination"))

                    );
            }

        }

        public class LandAndDismount : PrioritySelector
        {
            public LandAndDismount(string reason = "No reason provided")
            {
                Reason = reason;
                Children = CreateChildren();
            }

            // BT contruction-time properties...
            private string Reason { get; set; }

            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
					new Mount.ActionLandAndDismount(Reason),
                };
            }
        }

        public class MountStrategyPS : PrioritySelector
        {
            public Selection<MountStrategyType> MountStrategyDelegate { get; set; }
            //public Selection<WoWPoint> LandAtLocationDelegate { get; set; }

            private MountStrategyType CachedMountStrategy { get; set; }

            public MountStrategyPS(Selection<MountStrategyType> mountStrategyType)
            {
                MountStrategyDelegate = mountStrategyType;

                Children = CreateChildren();
            }
            /*
            public MountStrategyPS(Selection<MountStrategyType> mountStrategyType, Selection<WoWPoint> landAtLocation)
            {
                LandAtLocationDelegate = landAtLocation;
                Children = CreateChildren();
            }
             */

            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
                    new PrioritySelector(
                        new ActionFail(context =>
                        {
                            CachedMountStrategy = MountStrategyDelegate(context);
                            //Log.Info("=== CachedMountStrategy:" + CachedMountStrategy.ToString());
                            
                        }),
                        
                        // Cancel Shapeshift?
                        new Decorator(context => ((CachedMountStrategy == MountStrategyType.CancelShapeshift) || (CachedMountStrategy == MountStrategyType.DismountOrCancelShapeshift)) && Me.IsShapeshifted(),
                            new PrioritySelector(

                                // Need to land, if flying...
                                new Decorator(ret => WoWMovement.ActiveMover.IsFlying, new LandAndDismount()),

                                // Simply cancel shapefhit form
                                new Action(context =>
                                {
                                    TreeRoot.StatusText = "Canceling shapeshift form.";
                                    Lua.DoString("CancelShapeshiftForm()");
                                })
                            )),


                        // Dismount?
                        new Decorator(context => ((CachedMountStrategy == MountStrategyType.Dismount) || (CachedMountStrategy == MountStrategyType.DismountOrCancelShapeshift)) && Me.IsMounted(),
                            new PrioritySelector(

                                new TreeSharpExt.LogMessage("=== we should be dismounting.... ok"),
                            
                                // Need to land, if flying...
                                new Decorator(ret => WoWMovement.ActiveMover.IsFlying, new LandAndDismount()),

                                // Shapeshifted?
                                new Decorator(context => Me.IsShapeshifted(),
                                    new Action(context =>
                                    {
                                        TreeRoot.StatusText = "Canceling 'mounted' shapeshift form.";
                                        Lua.DoString("CancelShapeshiftForm()");
                                    })),

                                // LUA Dismount
                                new Decorator(context => Me.IsMounted(),
                                    new Action(context =>
                                    {
                                        TreeRoot.StatusText = "Dismounting";

                                        // Mount.Dismount() uses the Flightor landing system, which sometimes get stuck
                                        // a yard or two above the landing zone...
                                        // So, we opt to dismount via LUA since we've controlled the landing ourselves.
                                        Lua.DoString("Dismount()");
                                    }))
                                )),


                        // Mount up?
                        new Decorator(context => (CachedMountStrategy == MountStrategyType.MountFlying || CachedMountStrategy == MountStrategyType.MountGround || CachedMountStrategy == MountStrategyType.Mount),
                            new PrioritySelector(

                                //new TreeSharpExt.LogMessage("=== we're going to mount now"),

                                // Mount. Choose the best type of mount, flying preferably
                                new Decorator(ret=> CachedMountStrategy == MountStrategyType.Mount /*&& WoWMovement.ActiveMover.MovementInfo.CanFly */&& !Flightor.MountHelper.Mounted && Flightor.MountHelper.CanMount, 
                                    new Action(context =>
                                    {
                                        Log.Info("=== using flight ok");
                                        Flightor.MountHelper.MountUp();
                                    })),


                                // Flying Mount?
                                new Decorator(ret=> CachedMountStrategy == MountStrategyType.MountFlying && !Flightor.MountHelper.Mounted && Flightor.MountHelper.CanMount, 
                                    new Action(context =>
                                    {
                                        //Log.Info("=== still using flightor");
                                        Flightor.MountHelper.MountUp();
                                    })),

                                // Ground Mount? .. or generic Mount command but unable to fly
                                new Decorator(ret=> ((CachedMountStrategy == MountStrategyType.MountGround) || (CachedMountStrategy == MountStrategyType.Mount && !WoWMovement.ActiveMover.MovementInfo.CanFly)) && !Me.Mounted && Mount.CanMount(), 
                                    new Action(context =>
                                    {
                                        Log.Info("=== lets ground mount up");
                                        return Mount.MountUp(() => true,
                                            () => WoWMovement.ActiveMover.Location.Add(1000.0, 1000.0, 1000.0))
                                            ? RunStatus.Success
                                            : RunStatus.Failure;
                                    }))
                        
                        ))
                    )
                };
            }
        }


        #region Navigation Providers and overriding etc.

        private static INavigationProvider _preNavigationProvider;
        private static IPlayerMover _prePlayerMoverHandler;
        private static IStuckHandler _prevStuckHandler;
        //private static readonly Movement.NoStuckHandling NoStuckHandling = new Movement.NoStuckHandling();

        public static bool IsNavigationProviderOverridden
        {
            get { return Navigator.NavigationProvider != _preNavigationProvider; }
        }
        
        public static void RecordNavigationSystem()
        {
            _prevStuckHandler = Navigator.NavigationProvider.StuckHandler;
            _prePlayerMoverHandler = Navigator.PlayerMover;
            _preNavigationProvider = Navigator.NavigationProvider;
        }

        public static void RestoreOriginalNavigationSystem()
        {
            // Restore HB's default navigation
            Navigator.NavigationProvider.StuckHandler = _prevStuckHandler;
            Navigator.PlayerMover = _prePlayerMoverHandler;
            Navigator.NavigationProvider = _preNavigationProvider;
        }


        public static void SetNavigationSystemNoMovement()
        {
            // Override HB's navigation with no movement providers.

            Navigator.NavigationProvider.StuckHandler = new NoStuckHandling();
            Navigator.PlayerMover = new NoPlayerMovement();
            Navigator.NavigationProvider = new NoNavigation();
        }


        public class NoNavigation : INavigationProvider
        {
            public bool CanNavigateFully(WoWPoint from, WoWPoint to, int maxHops)
            {
                return true;
            }

            public bool AtLocation(WoWPoint point1, WoWPoint point2)
            {
                return true;
                //return point1.DistanceSqr(point2) <= 1 * 1;
            }

            public bool Clear()
            {
                return true;
            }

            public WoWPoint[] GeneratePath(WoWPoint from, WoWPoint to)
            {
                return new[] {new WoWPoint(from.X, from.Y, from.Z)};
            }

            public MoveResult MoveTo(WoWPoint location)
            {
                return MoveResult.Moved;
            }

            public float PathPrecision { get; set; }
            public IStuckHandler StuckHandler { get; set; }

            public NoNavigation()
            {
                StuckHandler = new NoStuckHandling();
            }
        }

        public class NoPlayerMovement : IPlayerMover
        {
            public void Move(WoWMovement.MovementDirection direction) { }

            public void MoveStop() { }

            public void MoveTowards(WoWPoint location) { }
        }

        public class NoStuckHandling : IStuckHandler
        {
            public bool IsStuck() { return false; }

            public void Reset() { }

            public void Unstick() { }
        }

        #endregion
        


        /// <summary>
        ///   Creates a behavior that does nothing more than check if we're facing the target; and if not, faces the target. (Uses a hard-coded 70degree frontal cone)
        /// </summary>
        /// <remarks>
        ///   Created 5/1/2011.
        /// </remarks>
        /// <returns>.</returns>
        public static Composite CreateFaceTargetBehavior(float viewDegrees = 70f, bool waitForFacing = true)
        {
            return CreateFaceTargetBehavior(ret => StyxWoW.Me.CurrentTarget, viewDegrees, waitForFacing);
        }

        public static Composite CreateFaceTargetBehavior(Selection<WoWUnit> toUnit, float viewDegrees = 100f, bool waitForFacing = true)
        {
            return new Decorator( ret => toUnit != null && toUnit(ret) != null && !StyxWoW.Me.IsMoving && !toUnit(ret).IsMe && !StyxWoW.Me.IsSafelyFacing(toUnit(ret), viewDegrees),
                new Action(ret =>
                {
                    WoWUnit unit = toUnit(ret);

                    //if (SingularSettings.Debug) Logger.WriteDebug("FaceTarget: facing since more than {0} degrees", (long)viewDegrees);

                    unit.Face();

                    if (!waitForFacing)
                        return RunStatus.Failure;

                    // even though we may want a tighter conical facing check, allow
                    // .. behavior to continue if 150 or better so we can cast while turning
                    if (StyxWoW.Me.IsSafelyFacing(unit, 150f))
                        return RunStatus.Failure;

                    // special handling for when consumed by Direglob and other mobs we are inside/on top of 
                    // .. as facing sometimes won't matter
                    if (StyxWoW.Me.InVehicle)
                    {
                        //Logger.WriteDebug("FaceTarget: don't wait to face {0} since in vehicle", unit.SafeName());
                        return RunStatus.Failure;
                    }

                    // otherwise, indicate behavior complete so begins again while
                    // .. waiting for facing to occur
                    return RunStatus.Success;
                })
                );
        }



        /// <summary>
        ///  Creates a behavior that does nothing more than check if we're in Line of Sight of the target; and if not, move towards the target.
        /// </summary>
        /// <remarks>
        ///  Created 23/5/2011
        /// </remarks>
        /// <returns>.</returns>
        public static Composite CreateMoveToLosBehavior()
        {
            return CreateMoveToLosBehavior(ret => StyxWoW.Me.CurrentTarget);
        }

        public static Composite CreateMoveToLosBehavior(Selection<WoWUnit> toUnit)
        {
            return new Decorator( ret => true//!MovementManager.IsMovementDisabled
                    && toUnit != null && toUnit(ret) != null && !toUnit(ret).IsMe && !InLineOfSpellSight(toUnit(ret)),
                new Sequence(
                    //new Action(ret => Logger.WriteDebug(Color.White, "MoveToLoss: moving to LoSS of {0} @ {1:F1} yds", toUnit(ret).SafeName(), toUnit(ret).Distance)),
                    new Action(ret => Navigator.MoveTo(toUnit(ret).Location)),
                    new StopMovingPS(ret=> StopMovingWhen.InLoSOfUnit, ret=> toUnit(ret), null,null)
                    //new Action(ret => StopMoving.InLosOfUnit(toUnit(ret)))
                    )
                );
        }

        /// <summary>
        /// true if Target in line of spell sight AND a spell hasn't just failed due
        /// to a line of sight error.  This test is required for the unit we are moving
        /// towards because WoWUnit.InLineOfSpellSight will return true while the
        /// WOW Game Client fails the spell cast.  See EventHandler.cs for setting
        /// LastLineOfSightError
        /// 
        /// Only use this for unit we are moving towards.  Not to be used when checking
        /// ability to cast spells on various mobs
        /// </summary>
        /// <param name="unit">target we are moving towards</param>
        /// <returns></returns>
        public static bool InLineOfSpellSight(WoWUnit unit, int timeOut = 1000)
        {
            if (unit != null && unit.InLineOfSpellSight) return true;
                /*
            {
                if ((DateTime.Now - EventHandlers.LastLineOfSightFailure).TotalMilliseconds < timeOut)
                {
                    Logger.WriteDebug(Color.White, "InLineOfSpellSight: last LoS error < {0} ms, pretending still not in LoS", timeOut);
                    return false;
                }

                return true;
            }
                 */

            return false;
        }

        public static bool InMoveToMeleeStopRange(WoWUnit unit)
        {
            if (unit == null || !unit.IsValid)
                return false;

            if (unit.IsPlayer)
                return unit.DistanceSqr < (2 * 2);

            float preferredDistance = unit.MeleeDistance() - (unit.IsMoving ? 1.5f : 1f);
            if (unit.Distance <= preferredDistance && unit.IsWithinMeleeRange)
                return true;

            return false;
        }
        

        public static Composite CreateMoveToUnitBehavior(float range, Selection<WoWUnit> onUnit)
        {
            return CreateMoveToUnitBehavior(onUnit, range);
        }
        public static Composite CreateMoveToUnitBehavior(Selection<WoWUnit> onUnit, float range, float stopAt = float.MinValue, RunStatus statusWhenMoving = RunStatus.Failure)
        {
            return new Decorator(ret => true,//!MovementManager.IsMovementDisabled,
                new Action(ret =>
                {
                    WoWUnit unit = onUnit == null ? null : onUnit(ret);
                    if (unit != null && unit.SpellDistance() > range)
                    {
                        MoveResult moveRes = Navigator.MoveTo(unit.Location);
                        Logger.WriteDebug(Color.White, "MoveToUnit[{0}]: moving within {1:F1} yds of {2} @ {3:F1} yds", moveRes, range, unit.SafeName(), unit.SpellDistance());
                        StopMoving.InRangeOfUnit(unit, stopAt == float.MinValue ? range : stopAt);
                        
                        if (moveRes != MoveResult.Failed && moveRes != MoveResult.PathGenerationFailed)
                            return statusWhenMoving;
                    }
                    return RunStatus.Failure;
                })
                );
        }
        

        /// <summary>
        ///   Creates the ensure movement stopped behavior. if no range specified, will stop immediately.  if range given, will stop if within range yds of current target
        /// </summary>
        /// <remarks>
        ///   Created 5/1/2011.
        /// </remarks>
        /// <returns>.</returns>
        public static Composite CreateEnsureMovementStoppedBehavior(float range = float.MaxValue, Selection<WoWUnit> onUnit = null, string reason = null)
        {
            if (range == float.MaxValue)
            {
                return new Decorator( ret => /*!MovementManager.IsMovementDisabled &&*/ StyxWoW.Me.IsMoving,
                    new Sequence(
                        new Action(ret => Logger.WriteDebug(Color.White, "EnsureMovementStopped: stopping! {0}", reason ?? "")),
                        new Action(ret => StopMoving.Now())
                        )
                    );
            }

            if (onUnit == null)
                onUnit = del => StyxWoW.Me.CurrentTarget;

            return new Decorator( ret => true//!MovementManager.IsMovementDisabled
                    && StyxWoW.Me.IsMoving
                    && (onUnit(ret) == null || onUnit(ret).Distance < range),
                new Sequence(
                    new Action(ret => Logger.WriteDebug(Color.White, "EnsureMovementStopped: stopping because {0}", onUnit(ret) == null ? "No CurrentTarget" : string.Format("target @ {0:F1} yds, stop range: {1:F1}", onUnit(ret).Distance, range))),
                    new Action(ret => StopMoving.Now())
                    )
                );
        }



        /// <summary>
        /// Basic stop moving class. Consider using StopMovingPS composite
        /// </summary>
        public static class StopMoving
        {
            
            public static void AtLocation(WoWPoint pt)
            {
                if (!WoWMovement.ActiveMover.IsMoving) return;
                if (Me.Location.Distance(pt) <= 1) Navigator.PlayerMover.MoveStop();

            }

            public static void InRangeOfLocation(WoWPoint pt, double range)
            {
                if (!WoWMovement.ActiveMover.IsMoving) return;
                if (Me.Location.Distance(pt) <= range) Navigator.PlayerMover.MoveStop();
            }

            public static void InRangeOfUnit(WoWUnit unit, double range)
            {
                if (!WoWMovement.ActiveMover.IsMoving) return;
                if (Me.Location.Distance(unit.Location) <= range) Navigator.PlayerMover.MoveStop();
            }

            public static void InMeleeRangeOfUnit(WoWUnit unit)
            {
                if (!WoWMovement.ActiveMover.IsMoving) return;
                if (InMoveToMeleeStopRange(unit)) Navigator.PlayerMover.MoveStop();
            }

            public static void InLosOfUnit(WoWUnit unit)
            {
                //Set(StopType.LosOfUnit, unit, WoWPoint.Empty, 0, at => Unit == null || !Unit.IsValid || Movement.InLineOfSpellSight(Unit), and);
                if (!WoWMovement.ActiveMover.IsMoving) return;
                if (unit.InLineOfSpellSight) Navigator.PlayerMover.MoveStop();
            }

            public static void Now()
            {
                Navigator.PlayerMover.MoveStop();
            }

        }
    }
}
