﻿using System.Collections.Generic;
using System.Linq;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.Pathing;
using Styx.WoWInternals.WoWObjects;
using CommonBehaviors.Actions;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Action = Styx.TreeSharp.Action;
using ActionFail = Fpsware.Library3.Extensions.TreeSharpExt.ActionFail;

namespace Fpsware.Library3.Helpers
{
    public static class RAFGroup
    {
        public delegate T Selection<out T>(object context);

        private static LocalPlayer Me { get { return (StyxWoW.Me); } }

        public static bool TryLeaderLastKnownPosition { get; set; }

        public static WoWPoint LeaderLastKnownPosition
        {
            get { return _leaderLastKnownPosition; }
            set { _leaderLastKnownPosition = value; }
        }

        //public static WoWPoint LeaderLastKnownPosition = new WoWPoint(1,1,1);

        private static readonly WoWPoint BlankPoint = new WoWPoint(1, 1, 1);

        public static string PreferedLeaderName { get; set; }

        private static WoWPlayer _leader;
        private static WoWPoint _leaderLastKnownPosition = new WoWPoint(1, 1, 1);

        public static WoWPlayer Leader
        {
            get
            {

                // Return our cached leader
                if (_leader != null && _leader.IsValid && _leader.IsAlive && (PreferedLeaderName == null || PreferedLeaderName == _leader.Name)) return _leader;

                WoWPlayer unit;

                // Try to find our prefered leader by name
                unit = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).FirstOrDefault(u => u.IsValid && u.IsAlive && u.Name == PreferedLeaderName);
                if (unit != null)
                {
                    RaFHelper.SetLeader(unit.Guid);
                    _leader = unit;

                    Log.Info("RAF Leader has been set by prefered name");
                    return _leader;
                }

                // Find a suitable leader, prefer players filling the role of Tank
                unit = Units.Friendly.Tanks.FirstOrDefault();
                if (unit != null)
                {
                    RaFHelper.SetLeader(unit.Guid);
                    _leader = unit;
                    Log.Info("RAF Leader has been set by first available tank");
                    return _leader;
                }

                // No tanks were found, try something else - Healer
                unit = Units.Friendly.Healers.FirstOrDefault();
                if (unit != null)
                {
                    RaFHelper.SetLeader(unit.Guid);
                    _leader = unit;
                    Log.Info("RAF Leader has been set by first available healer");
                    return _leader;
                }

                // Still nothing? Then use the first party member
                unit = Me.PartyMembers.FirstOrDefault();
                if (unit != null)
                {
                    RaFHelper.SetLeader(unit.Guid);
                    _leader = unit;
                    Log.Info("RAF Leader has been set by first available party member");
                    return _leader;
                }

                return null;
            }
        }


        public class FollowLeaderPS : PrioritySelector
        {
            internal Selection<WoWPlayer> FollowUnitDelegate { get; set; }
            internal Selection<int> MoveToDistanceDelegate { get; set; }
            internal Selection<bool> FanoutDelegate { get; set; }

            private bool CachedFanout { get; set; }
            private bool CachedCopyMounting { get; set; }
            private float ForcedMountDistance { get; set; }

            private int CachedMoveToDistance { get; set; }
            private int MoveToFlyingDistance { get; set; }
            private WoWUnit CachedFollowUnit { get; set; }

            private bool ApplySanityChecks { get; set; }


            public FollowLeaderPS(Selection<WoWPlayer> followUnit)
            {
                FollowUnitDelegate = followUnit;
                MoveToDistanceDelegate = (context => 15);
                FanoutDelegate = (context => false);

                CachedFanout = false;
                CachedCopyMounting = true;
                ApplySanityChecks = true;
                ForcedMountDistance = 55;
                MoveToFlyingDistance = 12;

                Children = CreateChildren();
            }

            public FollowLeaderPS(Selection<WoWPlayer> followUnit, Selection<int> moveToDistance)
            {
                FollowUnitDelegate = followUnit;
                MoveToDistanceDelegate = moveToDistance;
                FanoutDelegate = (context => false);

                CachedFanout = false;
                CachedCopyMounting = true;
                ApplySanityChecks = true;
                ForcedMountDistance = 55;
                
                MoveToFlyingDistance = 12;

                Children = CreateChildren();
            }

            public FollowLeaderPS(Selection<WoWPlayer> followUnit, Selection<int> moveToDistance, Selection<bool> useFanout)
            {
                FollowUnitDelegate = followUnit;
                MoveToDistanceDelegate = moveToDistance;
                FanoutDelegate = useFanout;

                CachedCopyMounting = true;
                ApplySanityChecks = true;
                ForcedMountDistance = 55;

                MoveToFlyingDistance = 12;

                Children = CreateChildren();
            }

            
            private List<Composite> CreateChildren()
            {
                return new List<Composite>()
                {
                    new PrioritySelector(
                        new ActionFail(context =>
                        {
                            CachedFollowUnit = FollowUnitDelegate(context);
                            CachedMoveToDistance = MoveToDistanceDelegate(context);
                            CachedFanout = FanoutDelegate(context);
                        }
                    )),

                    // Wait for casting (likely to be mounting)
                    new Decorator(ret=> Me.IsCasting, new ActionAlwaysSucceed()),

                    // Bailout if we need to
                    new Decorator(ret=> ApplySanityChecks, DoSanityChecks()),

                    // Found our leader? Clear TryLeaderLastKnownPosition. 
                    new Decorator(ret=> TryLeaderLastKnownPosition && Leader != null, new Action(context =>
                    {
                        TryLeaderLastKnownPosition = false;
                        TreeRoot.StatusText = "We have found our leader :)";
                    })),

                    // Leader not valid, try their last known location
                    new Decorator(ret=> TryLeaderLastKnownPosition, 
                        new PrioritySelector(

                            new ActionFail(context => { TreeRoot.StatusText = "Leader is out of range, using RBC to navigatee to them"; }),

                            // Are we there already?
                            // Clear our variables and go no further
                            new Decorator(ret=> Me.Location.Distance(LeaderLastKnownPosition) <= 5, 
                                new Sequence(
                                    new Action(context =>
                                    {
                                        TryLeaderLastKnownPosition = false;
                                        LeaderLastKnownPosition = new WoWPoint(1, 1, 1);
                                    })
                            )),

                            // Move there now - mount if required
                            new Decorator(ret=> Me.Location.Distance(LeaderLastKnownPosition) > 10 && LeaderLastKnownPosition != BlankPoint, 
                                new Sequence(
                                    new DecoratorContinue(ret => !Me.Mounted && Me.Location.Distance(LeaderLastKnownPosition) > ForcedMountDistance && Mount.CanMount(), new Movement.MountStrategyPS(ret=> Movement.MountStrategyType.Mount)),
                                    new DecoratorContinue(ret=> Me.IsFlying || Flightor.MountHelper.CanMount, new Movement.MoveTo(ret=> LeaderLastKnownPosition, ret=> Movement.MovementProviderType.Flightor)),
                                    new Movement.MoveTo(ret=> LeaderLastKnownPosition, ret=> Movement.MovementProviderType.Navigator)
                            ))
                            
                    )),

                    // Lets follow our leader
                    new Decorator(ret => CachedFollowUnit != null && CachedFollowUnit.IsValid && CachedFollowUnit.IsAlive,
                        new PrioritySelector(
                            
                            // Do we need to mount up?
                            new Decorator(ret => !Me.Mounted && (CachedFollowUnit.Mounted || CachedFollowUnit.Distance > ForcedMountDistance),
                                new PrioritySelector(
                                    new Decorator(ret=> CachedFollowUnit.IsFlying, new Movement.MountStrategyPS(ret=> Movement.MountStrategyType.MountFlying)),
                                    new Sequence(
                                        //new TreeSharpExt.LogMessage("=== in RAF using mount up"),
                                        new Movement.MountStrategyPS(ret=> Movement.MountStrategyType.Mount)
                                        )
                                    )
                                
                                ),

                            // Do we need to dismount?
                            new Decorator(ret => Me.Mounted && !CachedFollowUnit.Mounted && (CachedFollowUnit.Distance < (CachedMoveToDistance *1.25)),
                                new Sequence(
                                    //new TreeSharpExt.LogMessage("=== dismounting for some fucking reason"),
                                    new Movement.MountStrategyPS(ret=> Movement.MountStrategyType.DismountOrCancelShapeshift)
                                    )
                                
                                ),

                            // Stop moving if we're close enough
                            new Decorator(ret=> WoWMovement.ActiveMover.IsMoving && ((CachedFollowUnit.Distance < CachedMoveToDistance && !CachedFollowUnit.IsMoving) || (CachedFollowUnit.Distance < (CachedMoveToDistance * 0.6)  && CachedFollowUnit.IsMoving)) && CachedFollowUnit.InLineOfSight,
                                new PrioritySelector(
                                    /*new Decorator(ret=> CachedFanout, 
                                        new Action(context =>
                                        {
                                        
                                        }
                                    )),
                                     */

                                    new Sequence(
                                        //new TreeSharpExt.LogMessage("==== stop moving we are close enough"),
                                        new Movement.StopMovingPS()
                                        )
                                    
                                    )
                                
                                
                                ),
                            
                            // Store the leaders location for [possible] later use
                            new ActionFail(context => LeaderLastKnownPosition = CachedFollowUnit.Location),
                            
                            // Move to the leader
                            // ... via flying mount
                            new Decorator(ret=> CachedFollowUnit.Distance > (!Me.IsFlying ? CachedMoveToDistance : MoveToFlyingDistance) && (CachedFollowUnit.IsFlying || WoWMovement.ActiveMover.MovementInfo.CanFly),
                                new Sequence(
                                    //new TreeSharpExt.LogMessage("=== using FLIGHTOR to move to leader"),
                                    new Action(context => { Flightor.MoveTo(CachedFollowUnit.Location); })
                                    //new Movement.MoveTo(ret=> CachedFollowUnit.Location, ret=> Movement.MovementProviderType.Flightor, ret=> Movement.MountStrategyType.None)
                                    )
                                ),
                            

                            // ... via ground mount
                            new Decorator(ret=> CachedFollowUnit.Distance > (!Me.IsFlying ? CachedMoveToDistance : MoveToFlyingDistance) && !CachedFollowUnit.IsFlying,// && CachedFollowUnit.IsMoving,
                                new Sequence(
                                    //new TreeSharpExt.LogMessage("=== using navigator to move to leader"),
                                    new Movement.MoveTo(ret=> CachedFollowUnit.Location, ret=> Movement.MovementProviderType.Navigator, ret=> Movement.MountStrategyType.None, ret=> false)
                                    ))
                                    /*
                                    new Decorator(ret=> CachedFollowUnit.Distance > (!Me.IsFlying ? CachedMoveToDistance : MoveToFlyingDistance) && !CachedFollowUnit.IsFlying && !CachedFollowUnit.IsMoving,
                                new Sequence(
                                    new TreeSharpExt.LogMessage("=== using navigator to move to leader"),
                                    new Movement.MoveTo(ret=> CachedFollowUnit.Location, ret=> Movement.MovementProviderType.Navigator, ret=> Movement.MountStrategyType.None, ret=> CachedFanout)
                                    ))
                                     */
                            
                            //new Decorator(ret=> CachedFollowUnit.Distance > (!Me.IsFlying ? CachedMoveToDistance : MoveToFlyingDistance), new Movement.MoveTo(ret=> CachedFollowUnit.Location, ret=> Movement.MovementProviderType.Flightor, ret=> Movement.MountStrategyType.None))

                            ))
                };
            }


            private Composite DoSanityChecks()
            {
                return new PrioritySelector(

                    new Decorator(ret => Misc.IsUserMoving, new ActionAlwaysSucceed()),
                    new Decorator(ret => CachedFollowUnit == null && !TryLeaderLastKnownPosition, new ActionAlwaysSucceed()),
                    new Decorator(ret => Me.HasAura("Food") || Me.HasAura("Drink"), new ActionAlwaysSucceed())
                    
                    );
            }



        }

    }
}
