﻿using System;
using System.ServiceModel;
using System.Threading;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using Styx;

namespace Fpsware.Botbases.RAFBot2.Helpers
{
    public static class RAFBotComms
    {
        private static bool _isAlive;
        private static bool _isCommsOpen;
        private static bool _isClientRegistered;
        private static bool _isServerRegistered;
        private static ChannelFactory<IService> _scf;// = new ChannelFactory<IService>(new NetTcpBinding(), "net.tcp://localhost:8000");
        private static IService _s;// = scf.CreateChannel();

        public static bool IsServerRegistered
        {
            get { return _isServerRegistered; }
            private set { _isServerRegistered = value; }
        }

        public static void UnregisterServer()
        {
            if (!IsServerRegistered) return;

            try
            {
                IsServerRegistered = false;
                _svh.Close();
                _serverThread.Abort();

                Log.InitLog("RBC Server has been terminated!");
            }
            catch (Exception e) { }
        }

        private static Thread _serverThread;
        public static void RegisterServer()
        {
            if (!Setting.Instance.AdvancedUseRBC.Contains("Server")) return;
            if (IsServerRegistered && _isCommsOpen && _serverThread != null && _serverThread.IsAlive) return;

            if (_serverThread != null && _serverThread.IsAlive) { _serverThread.Abort(); Log.FailLog("---- 3.1 thread aborted"); }

            if (string.IsNullOrEmpty(Setting.Instance.AdvancedRBCServer) || string.IsNullOrEmpty(Setting.Instance.AdvancedRBCPort))
            {
                Log.FailLog("UgBCAEMAIABTAGUAcgB2AGUAcgAgAGMAYQBuACAAbgBvAHQAIABzAHQAYQByAHQAIQA=".Base64Decode());
                Log.FailLog("UgBCAEMAIABTAGUAcgB2AGUAcgAgAGEAZABkAHIAZQBzAHMAIABhAG4AZAAgAC8AIABvAHIAIABwAG8AcgB0ACAAbQB1AHMAdAAgAGIAZQAgAGYAaQBsAGwAZQBkACAAaQBuAC4A".Base64Decode());
                return;
            }

            _serverThread = new Thread(Server) { IsBackground = true };
            _serverThread.Start();

            IsServerRegistered = true;

            Log.InitLog("UgBCAEMAIABTAGUAcgB2AGUAcgAgAHMAdABhAHIAdABlAGQAIQAgAFQAaABpAHMAIABjAGgAYQByAGEAYwB0AGUAcgAgAGkAcwAgAHQAaABlACAAbABlAGEAZABlAHIAIABhAG4AZAAgAHcAaQBsAGwAIABwAHIAbwBjAGUAcwBzACAAYQBsAGwAIABSAEIAQwAgAHEAdQBlAHIAaQBlAHMALgA=".Base64Decode());
        }

        private static ServiceHost _svh;
        private static void Server()
        {
            try
            {
                //ServiceHost svh = new ServiceHost(typeof(ServiceImplementation));
                _svh = new ServiceHost(typeof(ServiceImplementation));
                _svh.AddServiceEndpoint(typeof(IService), new NetTcpBinding(), string.Format("net.tcp://{0}:{1}/RBC", Setting.Instance.AdvancedRBCServer, Setting.Instance.AdvancedRBCPort));
                _svh.Open();
                _svh.Opened += svh_Opened;
                _svh.Closed += svh_Closed;
            }
            catch (Exception e)
            {
                Log.FailLog("TwBIACAAUwBIAEkAVAAhACAAVwBlACAAZQB4AHAAaQBlAHIAZQBuAGMAZQBkACAAYQBuACAAaQBzAHMAdQBlACAAYQB0AHQAZQBtAHAAdABpAG4AZwAgAHQAbwAgAHIAZQBnAGkAcwB0AGUAcgAgAFIAQwBCACAAUwBlAHIAdgBlAHIA".Base64Decode());
                Log.FailLog(e.Message);
            }
        }

        static void svh_Closed(object sender, EventArgs e)
        {
            _isCommsOpen = false;
            Log.FailLog("---- svh_Closed");
            if (_serverThread.IsAlive) { _serverThread.Abort(); }
        }

        static void svh_Opened(object sender, EventArgs e)
        {
            Log.FailLog("---- svh_Opened");
            _isCommsOpen = true;
        }

        [ServiceContract]
        public interface IService
        {
            //[OperationContract] string DoCommand(string name); 
            
            [OperationContract]
            string Pulse(string name);
        }

        public class ServiceImplementation : IService
        {
            /*public string DoCommand(string name)
            {
                string command = name.ToUpper();

                if (command.StartsWith("DOCOMMAND"))
                {
                    return "this is your command";
                }

                if (command.StartsWith("WTF"))
                {
                    return "WTF? WTF!";
                }

                if (command.StartsWith("PULSE"))
                {
                    Log.Info("===== Pulse issues from client. Sending reply");
                    return "Pulse issued";
                }

                return "Unknown Command";
            }
             */

            public string Pulse(string query)
            {
                if (query.ToUpper() == "TABPAEMAQQBUAEkATwBOAA==".Base64Decode())
                {
                    //Log.FailLog("====== someone asked for my location");
                    return StyxWoW.Me.Location.ToString();
                }

                if (query.ToUpper() == "SABFAEEAUgBUAEIARQBBAFQA".Base64Decode())
                {
                    //Log.FailLog("====== someone performed a HEARTBEAT check");
                    //Log.FailLog("====== ...");
                    return "OK";
                }

                if (query.ToUpper() == "SwBFAFkAQgBJAE4ARABTAA==".Base64Decode())
                {
                    //Log.FailLog("====== someone performed a KEYBIND check");
                    string result = string.Format("[ADVANCEDQUESTING:{0}]", Setting.Instance.QuestUseAdvancedFeatures);

                    return result;
                }



                return null;
            }
        }

        public static bool IsClientRegistered
        {
            get { return _isClientRegistered; }
            private set { _isClientRegistered = value; }
        }

        public static void RegisterClient()
        {
            if (IsClientRegistered) return;

            try
            {
                _scf = new ChannelFactory<IService>(new NetTcpBinding(), string.Format("net.tcp://{0}:{1}/RBC", Setting.Instance.AdvancedRBCServer, Setting.Instance.AdvancedRBCPort));
                _s = _scf.CreateChannel();

                Log.InitLog("UgBCAEMAIABDAGwAaQBlAG4AdAAgAHMAdABhAHIAdABlAGQAIQA=".Base64Decode());

                IsClientRegistered = true;
            }
            catch (Exception e)
            {
                Log.FailLog("TwBIACAAUwBIAEkAVAAhACAAVwBlACAAZQB4AHAAaQBlAHIAZQBuAGMAZQBkACAAYQBuACAAaQBzAHMAdQBlACAAYQB0AHQAZQBtAHAAdABpAG4AZwAgAHQAbwAgAHIAZQBnAGkAcwB0AGUAcgAgAFIAQwBCACAAQwBsAGkAZQBuAHQA".Base64Decode());
                Log.FailLog(e.Message);
                IsClientRegistered = false;

            }
        }

        public static void UnregisterClient()
        {
            if (!IsClientRegistered) return;

            try
            {
                IsClientRegistered = false;
                _scf.Close();
                _s = null;
                //_svh.Close();
                //_serverThread.Abort();

                Log.InitLog("RBC Client has been terminated!");
            }
            catch (Exception e)
            {
                Log.Debug("error in UnregisterClient()");
                Log.Debug(e.Message);
            }
        }

        public static void DoHeartbeat()
        {
            //Log.Info("==== doing heart beat");
            string result = _s.Pulse("SABFAEEAUgBUAEIARQBBAFQA".Base64Decode());
            if (result.ToUpper() == "OK") IsAlive = true;
        }

        public static string Location
        {
            get
            {
                //Log.Info("==== doing heart beat");
                string result = _s.Pulse("TABPAEMAQQBUAEkATwBOAA==".Base64Decode());
                return result;
            }
        }

        public static string KeyBindQuery
        {
            get
            {
                string result = null;

                try { result = _s.Pulse("SwBFAFkAQgBJAE4ARABTAA==".Base64Decode()); }
                catch (Exception e) { }

                return result;
            }
        }

        public static bool IsAlive
        {
            get
            {
                //Log.Info("==== return IsAlive check");
                return _isAlive;
            }

            private set { _isAlive = value; }

        }

    }
}
