﻿using System;
using System.Globalization;
using System.IO;
using System.Net;
using Fpsware.Library3.Utility;
using Styx.Common;


namespace Fpsware.Library3.Helpers
{
    public class StatsCounter
    {
        static StatsCounter() { NodeTicker = false; }

        public static string CounterUrl { get; set; }
        public static string SettingsFolder { get; set; }

        public static bool NodeTicker { get; set; }

        /// <summary>
        /// Trigger the counter. This only works once per day.
        /// </summary>
        public static void Run()
        {
            try
            {
                if (string.IsNullOrEmpty(CounterUrl))
                {
                    Log.FailLog("The Stats Counter URL can not be blank.");
                    return;
                }

                if (NodeTicker)
                {
                    new MemoryStream(new WebClient().DownloadData(CounterUrl));
                    return;
                }

                if (string.IsNullOrEmpty(SettingsFolder))
                {
                    Log.FailLog("The SettingsFolder can not be blank.");
                    return;
                }

                string statcounterDate = DateTime.Now.DayOfYear.ToString(CultureInfo.InvariantCulture);
                string statcounterPath = string.Format("{0}\\Settings\\{1}\\Stats", Utilities.AssemblyDirectory, SettingsFolder);



                if (File.Exists(Path.Combine(statcounterPath, statcounterDate))) return;

                new MemoryStream(new WebClient().DownloadData(CounterUrl));
                if (!Directory.Exists(statcounterPath))
                {
                    Directory.CreateDirectory(statcounterPath);
                }

                File.Create(Path.Combine(statcounterPath, statcounterDate));

                string[] statcounterCleanup = Directory.GetFiles(statcounterPath);
                foreach (string counterfile in statcounterCleanup)
                {
                    string name = new FileInfo(counterfile).Name;
                    name = name.ToLower();

                    if (name == statcounterDate) continue;

                    File.Delete(counterfile);
                }

                NodeTicker = false;
            }
            catch { } // Catch all, like 404's.
        }

    }
}
