﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fpsware.Library3.Extensions;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Library3.Helpers
{
    public static class Units
    {
        public static class Hostile
        {
            private static LocalPlayer Me { get { return StyxWoW.Me; } }

            internal static IEnumerable<WoWUnit> AttackableUnits
            {
                get
                {
                    return ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(u => u.Attackable && u.CanSelect && u.IsValidTarget() && u.Distance <= 150);
                }
            }


            public static IEnumerable<WoWUnit> MobsAttackingTank()
            {
                return AttackableUnits.Where(u => Friendly.Tanks.Any(t => t.IsAlive && t.Guid == u.CurrentTargetGuid) && u.Distance < 40);
            }

            public static WoWUnit LowestHealthMobAttackingTank()
            {
                return MobsAttackingTank().OrderBy(u => u.HealthPercent).FirstOrDefault();
            }

            public static WoWUnit HighestHealthMobAttackingTank()
            {
                return MobsAttackingTank().OrderByDescending(u => u.HealthPercent).FirstOrDefault();
            }

            

            /// <summary>
            /// Units attacking you, your pet, party or raid members or any of your minions
            /// </summary>
            /// <returns></returns>
            public static IEnumerable<WoWUnit> UnitsInCombatTargetingAny
            {
                get
                {
                    return AttackableUnits.Where(u => u.Combat && (u.IsTargetingMeOrPet || u.IsTargetingMyPartyMember || u.IsTargetingMyRaidMember || u.IsTargetingAnyMinion));
                }

            }

            public static IEnumerable<WoWUnit> FindUnitsFromIds(IEnumerable<int> unitIds)
            {
                return ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(u => unitIds.Contains((int)u.Entry) && u.IsValid && u.IsAlive).ToList();
            }

            public static IEnumerable<WoWUnit> UnitsInCombatTargetingMe
            {
                get
                {
                    return AttackableUnits.Where(u => u.Combat && u.IsTargetingMe());
                }

            }
            
            public static int CountOfMobsNearTank(int maxDistaceFromTank = 15)
            {
                if (Friendly.Tanks.FirstOrDefault() == null) return 0;

                return ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                        .Count(u => u.IsValidTarget() && u.Location.Distance(Friendly.Tanks.FirstOrDefault().Location) <= maxDistaceFromTank);

            }

            public static int CountOfMobsNearUnit(WoWUnit unit = null, int maxDistaceFromUnit = 15)
            {
                if (unit == null) unit = StyxWoW.Me;

                return ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Count(u => u.IsValidTarget() && u.Location.Distance(unit.Location) <= maxDistaceFromUnit);

            }

            /// <summary>
            /// Count of units in combat with you, your pet, your party / raid members or your minions
            /// </summary>
            public static int Adds
            {
                get
                {
                    return AttackableUnits.Count(u => u.Combat && (u.IsTargetingMeOrPet || u.IsTargetingMyPartyMember || u.IsTargetingMyRaidMember || u.IsTargetingAnyMinion));
                }
            }

            public static int AddsTargetingMe
            {
                get
                {
                    return AttackableUnits.Count(u => u.Combat && u.GotTarget && u.CurrentTargetGuid == StyxWoW.Me.Guid);
                }
            }



            /// <summary>
            /// List of attackable units in [default] 40 yards. Not necessarily in combat.
            /// </summary>
            /// <param name="unit">Used for distance check from this unit. If null the player will be used</param>
            /// <param name="maxDistanceFromUnit">Maximum distance away a mob can be</param>
            /// <param name="minHealth">Minimum health % a mob can be</param>
            /// <returns>List of attackable units</returns>

            public static IEnumerable<WoWUnit> UnitsNearby(WoWUnit unit = null, double maxDistanceFromUnit = 40, int minHealth = 25)
            {
                if (unit == null) unit = StyxWoW.Me;

                return ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u => u.IsValidTarget() && u.HealthPercent >= minHealth && u.Location.Distance(unit.Location) <= maxDistanceFromUnit);
                //return AttackableUnits.Where(u => u.HealthPercent >= minHealth && u.Location.DistanceSqr(unit.Location) <= (maxDistanceFromUnit * maxDistanceFromUnit));

            }

            /// <summary>
            /// List of attackable units in [default] 40 yards. In combat.
            /// </summary>
            /// <param name="unit">Used for distance check from this unit. If null the player will be used</param>
            /// <param name="maxDistanceFromUnit">Maximum distance away a mob can be</param>
            /// <param name="minHealth">Minimum health % a mob can be</param>
            /// <returns>List of attackable units</returns>
            public static IEnumerable<WoWUnit> UnitsNearbyInCombat(WoWUnit unit = null, double maxDistanceFromUnit = 40, int minHealth = 25)
            {
                if (unit == null) unit = StyxWoW.Me;

                return ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u => u.Combat && (u.IsTargetingMeOrPet || u.IsTargetingAnyMinion || u.IsTargetingMyPartyMember || u.IsTargetingMyRaidMember) && u.IsValidTarget() && u.HealthPercent >= minHealth && u.Location.Distance(unit.Location) <= maxDistanceFromUnit);
                //return AttackableUnits.Where(u => u.HealthPercent >= minHealth && u.Location.DistanceSqr(unit.Location) <= (maxDistanceFromUnit * maxDistanceFromUnit));

            }


        }
        
        public static class Friendly
        {
            private static LocalPlayer Me { get { return StyxWoW.Me; } }

            public static IEnumerable<WoWPlayer> ResurrectablePlayers
            {
                get
                {
                    return ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(p => !p.IsMe && p.IsDead && p.IsFriendly && p.InLineOfSpellSight && p.IsInMyPartyOrRaid && p.DistanceSqr < 40 * 40 && !Blacklist.Contains(p.Guid, BlacklistFlags.All)).ToList();
                }
            }

            public static IEnumerable<WoWPlayer> GroupMembers
            {
                get
                {
                    List<WoWPlayer> members = StyxWoW.Me.RaidMembers.Union(StyxWoW.Me.PartyMembers).Distinct().Where(u => u.CanSelect).ToList();

                    if (!members.Contains(StyxWoW.Me)) { members.Add(StyxWoW.Me); }

                    return members;
                }
            }

            public static IEnumerable<WoWPlayer> GroupMembersAlive
            {
                get
                {
                    return GroupMembers.Where(u => u.IsAlive);
                }
            }



            public static List<WoWPlayer> Tanks
            {
                get
                {
                    if (!StyxWoW.Me.GroupInfo.IsInParty && !StyxWoW.Me.GroupInfo.IsInRaid) return new List<WoWPlayer>();

                    List<WoWPlayer> tanks = StyxWoW.Me.GroupInfo.RaidMembers.Union(StyxWoW.Me.GroupInfo.PartyMembers).Distinct().Where(p => p.HasRole(WoWPartyMember.GroupRole.Tank)).Select(p => p.ToPlayer()).Where(p => p != null && p.IsAlive).ToList();

                    if (!tanks.Any())
                    {
                        tanks =
                            StyxWoW.Me.GroupInfo.PartyMembers.Where(p => p.HasRole(WoWPartyMember.GroupRole.Tank))
                                .Select(p => p.ToPlayer())
                                .Where(p => p != null && p.IsAlive)
                                .ToList();
                    }

                    return tanks;
                }
            }

            /// <summary>
            /// List of all Healers in the party / raid
            /// </summary>
            public static List<WoWPlayer> Healers
            {
                get
                {
                    if (!StyxWoW.Me.GroupInfo.IsInParty && !StyxWoW.Me.GroupInfo.IsInRaid) return new List<WoWPlayer>();

                    List<WoWPlayer> healers = StyxWoW.Me.GroupInfo.RaidMembers.Union(StyxWoW.Me.GroupInfo.PartyMembers).Distinct().Where(p => p.HasRole(WoWPartyMember.GroupRole.Healer)).Select(p => p.ToPlayer()).Where(p => p != null).ToList();

                    if (!healers.Any())
                    {
                        healers = StyxWoW.Me.GroupInfo.RaidMembers.Union(StyxWoW.Me.GroupInfo.PartyMembers).Distinct().Where(p => p.HasRole(WoWPartyMember.GroupRole.Healer)).Select(p => p.ToPlayer()).Where(p => p != null).ToList();
                    }

                    return healers;
                }
            }



            public static int CountOfPlayersBelowHp(int minHealth = 99, int maxDistanceFromUnit = 40)
            {
                return CountOfPlayersBelowHp(minHealth, maxDistanceFromUnit, StyxWoW.Me);
            }

            public static int CountOfPlayersBelowHp(int minHealth = 99, int maxDistanceFromUnit = 40, WoWUnit unit = null, string withAura = null)
            {

                WoWUnit fromUnit = unit ?? StyxWoW.Me;

                IEnumerable<WoWPlayer> players = GroupMembersAlive.Where(p => p.HealthPercent <= minHealth && (String.IsNullOrEmpty(withAura) || p.HasAura(withAura)) && p.Location.Distance(fromUnit.Location) <= maxDistanceFromUnit).OrderBy(p=>p.HealthPercent);

                /*
                if (String.IsNullOrEmpty(withAura))
                {

                    players = GroupMembersAlive.Where(p => p.HealthPercent <= minHealth && (String.IsNullOrEmpty(withAura) || p.HasAura(withAura)) && p.Location.Distance(fromUnit.Location) <= maxDistanceFromUnit);
                }
                else
                {

                    players = GroupMembersAlive.Where(p => p.HealthPercent <= minHealth && p.HasMyAura(withAura) && p.Location.Distance(fromUnit.Location) <= maxDistanceFromUnit);
                }
                 */


                return players.Count();
            }















        }

        public static class Other
        {
            public static HashSet<uint> IgnoreMobs = new HashSet<uint>
            {
                52288, // Venomous Effusion (NPC near the snake boss in ZG. Its the green lines on the ground. We want to ignore them.)
                52302, // Venomous Effusion Stalker (Same as above. A dummy unit)
                52320, // Pool of Acid
                52525, // Bloodvenom

                52387, // Cave in stalker - Kilnara
            };
        }

    }
}
