﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using System.Net;
using System.Text.RegularExpressions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.Frames;
using Styx.CommonBot.Inventory;
using Styx.Helpers;
using Styx.WoWInternals.WoWObjects;
using Styx.Pathing;
using Styx.WoWInternals;
using Misc = Fpsware.Library3.Helpers.Misc;

namespace Fpsware.Botbases.RAFBot2.Questing
{
    public static class QuestingHelpers
    {
        #region Local Declarations
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        #endregion

        #region Attach and Detach Events

        public static void AttachQuestingEvents()
        {
            Lua.Events.AttachEvent("QUEST_GREETING", OnQuestGreeting);
            Lua.Events.AttachEvent("QUEST_DETAIL", OnQuestDetail);
            Lua.Events.AttachEvent("GOSSIP_SHOW", OnGossipShow);
            Lua.Events.AttachEvent("QUEST_COMPLETE", OnQuestComplete);
            Lua.Events.AttachEvent("QUEST_PROGRESS", OnQuestProgress);
        }

        public static void DetatchQuestingEvents()
        {
            Lua.Events.DetachEvent("QUEST_GREETING", OnQuestGreeting);
            Lua.Events.DetachEvent("QUEST_DETAIL", OnQuestDetail);
            Lua.Events.DetachEvent("GOSSIP_SHOW", OnGossipShow);
            Lua.Events.DetachEvent("QUEST_COMPLETE", OnQuestComplete);
            Lua.Events.DetachEvent("QUEST_PROGRESS", OnQuestProgress);

        }

        #endregion


        /// <summary>
        /// Can we use this target for something? Does it have quests to pick up / turn in or anything like that.
        /// </summary>
        public static bool IsTargetInteractable
        {
            get
            {
                WoWUnit target = null;

                if (!RAFBot2.IsLeaderValid)
                {
                    if (Me.GotTarget) target = Me.CurrentTarget;
                }
                else if (RAFGroup.Leader != null && RAFGroup.Leader.GotTarget)
                {
                    target = RAFGroup.Leader.CurrentTarget;
                }

                if (target == null) return false;

                bool result =  target.QuestGiverStatus == QuestGiverStatus.TurnIn ||
                       target.QuestGiverStatus == QuestGiverStatus.Available ||
                       target.QuestGiverStatus == QuestGiverStatus.AvailableRepeatable ||
                       //target.QuestGiverStatus == QuestGiverStatus.TurnInRepeatable ||
                       target.InteractType == WoWInteractType.TaxiPathAvailable;

                //Log.Info("===== IsTargetInteractable: " + result);
                return result;
            }
        }

        public static WoWUnit AutoQuestTurninUnit
        {
            get
            {

                if (!Setting.Instance.AutoQuestTurnIn.OK()) return null;

                WoWUnit target = null;

                target = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u =>
                    (u.QuestGiverStatus == QuestGiverStatus.TurnIn) &&
                    //(u.QuestGiverStatus == QuestGiverStatus.TurnIn || u.QuestGiverStatus == QuestGiverStatus.TurnInRepeatable) &&
                    u.Distance <= Setting.Instance.AutoQuestTurnInDistance &&
                    Navigator.CanNavigateFully(Me.Location, u.Location))
                    .OrderBy(u => u.Distance)
                    .FirstOrDefault();

                return target;
            }
        }

        public static WoWUnit AutoQuestPickupUnit
        {
            get
            {
                if (!Setting.Instance.AutoQuestPickUp.OK()) return null;

                WoWUnit target = null;

                target = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u =>
                    (u.QuestGiverStatus == QuestGiverStatus.Available) &&
                    u.Distance <= Setting.Instance.AutoQuestPickupDistance &&
                    Navigator.CanNavigateFully(Me.Location, u.Location))
                    .OrderBy(u => u.Distance)
                    .FirstOrDefault();

                return target;
            }
        }

        public static void OnQuestProgress(object obj, LuaEventArgs args)
        {
            Log.Info("===== OnQuestProgress");
            if (Setting.Instance.QuestTurnIn.ResultOK())
            {
                Log.Info("Turning in a quest ...");
                //Lua.DoString("RunMacroText(\"/click QuestFrameCompleteButton\")");
                QuestFrame.Instance.CompleteQuest();
            }
        }

        public static void OnQuestDetail(object obj, LuaEventArgs args)
        {
            Log.Info("===== OnQuestDetail");
            if (Setting.Instance.QuestPickUp.ResultOK())
            {
                Log.Info("Accepting a quest ... ");
                //if (QuestFrame.Instance != null) 
                //Lua.DoString("AcceptQuest()");
                QuestFrame.Instance.AcceptQuest();
            }
        }
        

        public static void OnQuestGreeting(object obj, LuaEventArgs args)
        {
            Log.Info("===== OnQuestGreeting");
            int activeQuests = QuestFrame.Instance.AvailableQuests.Count;

            Log.Info("==== _activeQuests:" + activeQuests);

            if (Setting.Instance.QuestPickUp.ResultOK())
            {
                Log.Info("Accepting a quest ...");

                QuestFrame.Instance.SelectAvailableQuest(0);
            }

            if (Setting.Instance.QuestTurnIn.ResultOK() && activeQuests > 0)
            {
                Log.Info("====== we need to make sure this code worked and we turned in out completed quest");
                
                // Get a list of active quests from the currently open window
                List<GossipQuestEntry> gossipQuests = QuestFrame.Instance.IsVisible ? QuestFrame.Instance.ActiveQuests : GossipFrame.Instance.ActiveQuests;

                // Check if the quest is complete against our quest logs
                GossipQuestEntry quest = (from entry in gossipQuests
                                          let pQuest = StyxWoW.Me.QuestLog.GetQuestById((uint)entry.Id)
                                          where pQuest != null && pQuest.IsCompleted
                                          select entry).FirstOrDefault();

                // We have a valid quest we can turn in to this quest giver!
                if (quest == null) return;

                // DO IT BITCH!
                QuestFrame.Instance.SelectActiveGossipQuest(quest.Index);
                Log.Info("====== it appeared to work, right?");
            }
        }

        public static void OnGossipShow(object obj, LuaEventArgs args)
        {
            Log.Info("===== OnGossipShow");

            if (Setting.Instance.QuestTurnIn.ResultOK() && GossipFrame.Instance.ActiveQuests.Count > 0)
            {
                // Get a list of active quests from the currently open window
                List<GossipQuestEntry> gossipQuests = QuestFrame.Instance.IsVisible ? QuestFrame.Instance.ActiveQuests : GossipFrame.Instance.ActiveQuests;

                // Check if the quest is complete against our quest logs
                GossipQuestEntry quest = (from entry in gossipQuests
                                          let pQuest = StyxWoW.Me.QuestLog.GetQuestById((uint)entry.Id)
                                          where pQuest != null && pQuest.IsCompleted
                                          select entry).FirstOrDefault();
                
                // We have a valid quest we can turn in to this quest giver!
                if (quest != null)
                {
                    // DO IT BITCH!
                    GossipFrame.Instance.SelectActiveGossipQuest(quest.Index);
                }
            }
            

            if (Setting.Instance.QuestPickUp.ResultOK())
            {
                Log.Info("Picking up a quest");
                
                if (QuestFrame.Instance.IsVisible)
                {
                    if (QuestFrame.Instance.AvailableQuests.Count > 0) QuestFrame.Instance.SelectAvailableQuest(0);
                }
                else if (GossipFrame.Instance.IsVisible)
                {
                    if (GossipFrame.Instance.AvailableQuests.Count > 0) GossipFrame.Instance.SelectAvailableQuest(0);
                }

            }
        }

        private static string bestQuestReward = null;
        public static void OnQuestComplete(object obj, LuaEventArgs args)
        {
            Log.Info("===== OnQuestComplete");
            if (!Setting.Instance.QuestTurnIn.ResultOK()) return;

            int countOfQuestRewards = Lua.GetReturnVal<int>("return GetNumQuestChoices()", 0);
            Log.Debug("countOfQuestRewards: " + countOfQuestRewards);

            // More than 1 reward to choose from
            if (countOfQuestRewards > 1 && Setting.Instance.QuestChooseReward.OK())
            {
                float bestOverallItemScore = Single.MinValue;
                int bestOverallItemChoice = -1;

				/*
                // Enumerate quest rewards to find the best one
                for (int i = 1; i <= countOfQuestRewards; i++)
                {
                    string itemLink = Lua.GetReturnVal<string>("return GetQuestItemLink(\"choice\", " + i.ToString(CultureInfo.InvariantCulture) + ")", 0);
                    string[] splitted = itemLink.Split(':');

                    uint itemId;
                    if (String.IsNullOrEmpty(itemLink) || (splitted.Length == 0 || splitted.Length < 2) || (!UInt32.TryParse(splitted[1], out itemId) || itemId == 0))
                    {
                        Log.Info("Parsing ItemLink for quest item failed!");
                        Log.Info("ItemLink: {0}", itemLink);
                        continue;
                    }
                    else
                    {
                        Log.Debug("itemId is good:" + itemId);
                    }

                    ItemInfo choiceItemInfo = ItemInfo.FromId(itemId);
                    if (choiceItemInfo == null)
                    {
                        Log.Info("Retrieving item info for reward item failed");
                        Log.Info("Item Id:{0} ItemLink:{1}", itemId, itemLink);
                        continue;
                    }
                    else
                    {
                        Log.Debug("choiceItemInfo is good:" + choiceItemInfo.Name);
                    }

                    // Score of the item.
                    float choiceItemScore = WeightSet.EvaluateItem(choiceItemInfo, new ItemStats(itemLink));

                    // Score the equipped item if any. otherwise 0
                    float bestEquipItemScore = Single.MinValue;

                    // The best slot
                    InventorySlot bestSlot = InventorySlot.None;
                    List<InventorySlot> inventorySlots = InventoryManager.GetInventorySlotsByEquipSlot(choiceItemInfo.EquipSlot);

                    foreach (InventorySlot slot in inventorySlots)
                    {
                        Log.Debug("foreach inventoryslots");
                        WoWItem equipped = EquippedItems[slot];
                        if (equipped == null) continue;

                        bestEquipItemScore = WeightSet.EvaluateItem(equipped, true);
                        bestSlot = slot;
                    }

                    if (bestEquipItemScore > Single.MinValue)
                    {
                        Log.Info("Equipped Item {0} scored {1}", EquippedItems[bestSlot].Name, bestEquipItemScore.ToString(CultureInfo.InvariantCulture));
                    }
                    else
                    {
                        Log.Debug("bestEquipItemScore < Single.MinValue");
                    }


                    // If this our armor type?
                    bool goodArmor = choiceItemInfo.ItemClass == WoWItemClass.Armor && (choiceItemInfo.ArmorClass == WeightSet.GetWantedArmorClass());// || miscArmorType.Contains(choiceItemInfo.InventoryType));
                    Log.Debug("goodArmor: " + goodArmor);

                    // Now we assign the overall score of this item
                    // 1. Is it a higher score than our equipped item?
                    // 2. Is this armor type good for us?
                    if (choiceItemScore > bestEquipItemScore && bestSlot != InventorySlot.None && (goodArmor && (choiceItemScore - bestEquipItemScore) > bestOverallItemScore))
                    {
                        bestOverallItemScore = (choiceItemScore - bestEquipItemScore);
                        bestOverallItemChoice = i;
                        Log.Debug("choiceItemScore and other stuff SUCCEEDED");
                        bestQuestReward = choiceItemInfo.Name;

                    }
                    else
                    {
                        Log.Debug("choiceItemScore and other stuff failed");
                    }
                    
                }
                */
                Log.Debug("finished enumerating quest rewards...");

                // We have not found a replacement item, so choose the most value item for vendoring
                if (bestOverallItemScore == Single.MinValue)
                {
                    Log.Info("No upgrades available, choosing most valuable item for vendoring");
                    ItemInfo highestItemInfo = null;

                    for (int i = 1; i <= countOfQuestRewards; i++)
                    {
                        string itemLink = Lua.GetReturnVal<string>("return GetQuestItemLink(\"choice\", " + i.ToString(CultureInfo.InvariantCulture) + ")", 0);
                        string[] splitted = itemLink.Split(':');

                        uint itemId;
                        if (String.IsNullOrEmpty(itemLink) || (splitted.Length == 0 || splitted.Length < 2) || (!UInt32.TryParse(splitted[1], out itemId) || itemId == 0))
                        {
                            Log.Info("Parsing ItemLink failed!");
                            Log.Info("ItemLink: {0}", itemLink);
                            continue;
                        }

                        ItemInfo choiceItemInfo = ItemInfo.FromId(itemId);
                        if (choiceItemInfo == null)
                        {
                            Log.Info("Retrieving item info failed!");
                            Log.Info("Item Id:{0} ItemLink:{1}", itemId, itemLink);
                            continue;
                        }

                        if (highestItemInfo == null || highestItemInfo.SellPrice > choiceItemInfo.SellPrice)
                        {
                            highestItemInfo = choiceItemInfo;
                            bestOverallItemChoice = i;
                        }
                    }
                }

                // Select the best quest reward
                //if (bestOverallItemChoice > 0)
                if (bestOverallItemChoice > 0)
                {
                    Log.Debug("bestOverallItemChoice > 0 and it is: " + bestOverallItemChoice);
                    //Log.Debug("QuestFrame.Instance.IsVisible: " + QuestFrame.Instance.IsVisible);
                    Log.Info("Best reward choice is: " + bestQuestReward);
                    QuestFrame.Instance.SelectQuestReward(bestOverallItemChoice-1);
                }
                else
                {
                    Log.Info("AHHHHH SHIIIIIIT!");
                    Log.Info("If we're here something has gone wrong and this is a fall through");
                    Log.Info("Inside OnQuestComplete. We failed to choose a quest reward, either for upgrade or vendoring.");
                    QuestFrame.Instance.SelectQuestReward(0);
                }

            }

            QuestFrame.Instance.CompleteQuest();

        }

        public static bool AdvancedQuestingInAction { get; set; }
       
        public static WoWUnit ObjectiveKillMobWoWUnit(int maxDistanceFromLeader = 50)
        {
            WoWUnit seudoRafLeader = Me;
            if (RAFGroup.Leader != null) seudoRafLeader = RAFGroup.Leader;
            
            List<WoWUnit> foundKillMobsList = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u => QuestMobsList().Contains(u.Entry) && u.IsValidTarget() && u.Location.DistanceSqr(seudoRafLeader.Location) <= maxDistanceFromLeader * maxDistanceFromLeader).OrderBy(u => u.DistanceSqr).ToList();
            if (foundKillMobsList.Any()) return foundKillMobsList.FirstOrDefault();

            // Appears some kill quests are being incorrectly reported as harvest objects. 
            // So, lets do this little test and see if there are any mobs matching our harvestable objects IDs
            foundKillMobsList = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u => QuestObjectsList().Contains(u.Entry) && u.IsValidTarget() && u.Location.DistanceSqr(seudoRafLeader.Location) <= maxDistanceFromLeader * maxDistanceFromLeader).OrderBy(u => u.DistanceSqr).ToList();
            if (foundKillMobsList.Any())
            {
                Log.FocusLog("===== found a quest mob but it was listed as object harvestable.");
                return foundKillMobsList.FirstOrDefault();
            }

            return null;
        }


        public static WoWGameObject InteractableQuestObject
        {
            get
            {
                {
                    WoWGameObject interactable = ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                        .Where(o =>
                            QuestObjectsList().Contains(o.Entry) && o.Distance < Setting.Instance.QuestObjectCollectionMaxDistance && Navigator.CanNavigateFully(Me.Location, o.Location) && !o.InUse && o.CanLoot &&
                            //(o.SubType == WoWGameObjectType.Goober || o.SubType == WoWGameObjectType.Chest) && !o.IsHerb && !o.IsMineral && o.CanLoot && 
                            // Is NOT..
                            //(o.SubType != WoWGameObjectType.Door && o.SubType != WoWGameObjectType.Chair && o.SubType != WoWGameObjectType.Button) && !Query.IsInCompetition(o, 15) && !Blacklist.Contains(o.Guid, BlacklistFlags.Loot)
                            !Query.IsInCompetition(o, 15) && !Blacklist.Contains(o.Guid, BlacklistFlags.Loot)
                        
                        )
                        .OrderBy(o => o.Distance)
                        .FirstOrDefault();
                    
                    if (interactable != null) return interactable;
                }

                return null;
            }
        }

        private static Dictionary<string, ProcessQuestData> _questMobs = new Dictionary<string, ProcessQuestData>();
        public static List<uint> QuestMobsList()
        {
            List<uint> mobsList = new List<uint>();

            foreach (KeyValuePair<string, ProcessQuestData> pair in _questMobs)
            {
                mobsList.AddRange(pair.Value.KillMobIDs);
            }

            return mobsList;
        }

        public static void AddQuestMobToKillList(uint questID, uint mobID)
        {
            ProcessQuestData pqd = new ProcessQuestData
            {
                QuestID = questID,
                HarvestObjectIDs = new List<uint>(),
                KillMobIDs = new List<uint>(new[] {mobID}),
                ObjectiveID = mobID
            };

            _questMobs.Add(questID + mobID.ToString(), pqd);
            ProcessedQuestList.Add(questID);
        }

        // quest ID, list of objects
        private static Dictionary<string, ProcessQuestData> _questObjects = new Dictionary<string, ProcessQuestData>();
        public static List<uint> QuestObjectsList()
        {
            List<uint> objectsList = new List<uint>();

            foreach (KeyValuePair<string, ProcessQuestData> pair in _questObjects)
            {
                objectsList.AddRange(pair.Value.HarvestObjectIDs);
            }

            return objectsList;
        }


        private class ProcessQuestData
        {
            public ProcessQuestData()
            {
                QuestID = 0;
                ObjectiveID = 0;
                HarvestObjectIDs = new List<uint>();
                KillMobIDs = new List<uint>();
            }
            public uint QuestID { get; set; }
            public uint ObjectiveID { get; set; }
            public List<uint> HarvestObjectIDs { get; set; }
            public List<uint> KillMobIDs { get; set; }

        }

        public static bool GotWoWHeadDataAlready(uint questID)
        {
            return ProcessedQuestList.Contains(questID);
        }


        private static List<uint> _processedQuestList = new List<uint>();
        /// <summary>
        /// Contains a list of quest IDs that have already been processed (checked on wowhead.com or other)
        /// </summary>
        public static List<uint> ProcessedQuestList
        {
            get { return _processedQuestList; }
            set { _processedQuestList = value; }
        }

        private static void GetWoWHeadIDs(uint questID, uint objectiveID)//, uint objectiveID)
        {
            string inventoryItemID = objectiveID.ToString(CultureInfo.InvariantCulture);

            Log.Info("Going to query WoWHead.com....");

            List<uint> itemList = new List<uint>();
            WebClient client = new WebClient();
            client.Headers.Add("dQBzAGUAcgAtAGEAZwBlAG4AdAA=".Base64Decode(), "TQBvAHoAaQBsAGwAYQAvADUALgAwACAAKABXAGkAbgBkAG8AdwBzACAATgBUACAANgAuADIAOwAgAFcATwBXADYANAA7ACAAcgB2ADoAMgA2AC4AMAApACAARwBlAGMAawBvAC8AMgAwADEAMAAwADEAMAAxACAARgBpAHIAZQBmAG8AeAAvADIANgAuADAA".Base64Decode());
            string downloadedString = client.DownloadString(string.Format("http://www.wowhead.com/item={0}", inventoryItemID));

            Regex patternGameObject = new Regex(string.Format("(?<=\\[{0}\\].*\"ti\":)(?<itemID>\\d+)(?=[^\\d])", inventoryItemID), RegexOptions.Multiline);
            Regex patternMobID = new Regex(string.Format("(?<=data:.*family.*\"id\":)(?<mobID>\\d+)(?=[^\\d])"), RegexOptions.Multiline);
            Regex patternMobID_alt = new Regex(string.Format("(?<=data:.*\"id\":)(?<mobID>\\d+)(?=[^\\d])"), RegexOptions.Multiline);

            MatchCollection matches = patternGameObject.Matches(downloadedString);

            // Find the game object ID that yields the quest item
            if (matches.Count > 0)
            {
                Log.Info("==== item " + objectiveID + " can be obtained by harvesting: ");
                foreach (uint item in matches.Cast<Match>().Where(match => match.Success && match.Groups["itemID"].Success).Select(match => match.Groups["itemID"].Value.ToUInt32()))
                {
                    Log.Info("==== found item: " + item);
                    itemList.Add(item);
                }

                Log.Info("WoWHead.com MATCH - Interactable object");

                ProcessQuestData pqd = new ProcessQuestData();
                pqd.QuestID = questID;
                pqd.ObjectiveID = objectiveID;
                pqd.KillMobIDs = new List<uint>();
                pqd.HarvestObjectIDs.AddRange(itemList);

                _questObjects.Add(questID + objectiveID.ToString(), pqd);
                ProcessedQuestList.Add(questID);
                return;
            }

            // Find the MobID(s) that drops our quest item
            matches = patternMobID.Matches(downloadedString);
            if (matches.Count > 0)
            {
                Log.Info("==== item " + objectiveID + " can be obtained by killing mobs: ");
                foreach (uint item in matches.Cast<Match>().Where(match => match.Success && match.Groups["mobID"].Success).Select(match => match.Groups["mobID"].Value.ToUInt32()))
                {
                    Log.Info("==== found mob: " + item);
                    itemList.Add(item);
                }

                Log.Info("WoWHead.com MATCH - Killable mob(s)");

                ProcessQuestData pqd = new ProcessQuestData();
                pqd.QuestID = questID;
                pqd.ObjectiveID = objectiveID;
                pqd.KillMobIDs.AddRange(itemList);
                pqd.HarvestObjectIDs = new List<uint>();

                _questMobs.Add(questID + objectiveID.ToString(), pqd);
                ProcessedQuestList.Add(questID);
                return;
            }


            // Second attempt at MobID(s). If the first failed we'll find it this way.
            matches = patternMobID_alt.Matches(downloadedString);
            if (matches.Count > 0)
            {
                Log.Info("==== patternMobID_alt");
                Log.Info("==== item " + objectiveID + " can be obtained by killing mobs: ");
                foreach (uint item in matches.Cast<Match>().Where(match => match.Success && match.Groups["mobID"].Success).Select(match => match.Groups["mobID"].Value.ToUInt32()))
                {
                    Log.Info("==== found mob: " + item);
                    itemList.Add(item);
                }

                Log.Info("WoWHead.com MATCH - Killable mob(s)");
                //_questMobs.Add(questID, itemList);
                ProcessQuestData pqd = new ProcessQuestData();
                pqd.QuestID = questID;
                pqd.ObjectiveID = objectiveID;
                pqd.KillMobIDs.AddRange(itemList);
                pqd.HarvestObjectIDs = new List<uint>();

                _questMobs.Add(questID + objectiveID.ToString(), pqd);
                ProcessedQuestList.Add(questID);
                return;
            }
            
            Log.FailLog("DOH! WE CAN'T FIND THE INFO WE NEED");
            ProcessedQuestList.Add(questID);
            
        }

        /// <summary>
        /// Enumerate your quest log and pulls necessary data from wowhead.com
        /// Also removes completed quests and objectives from QuestObjectsList and QuestMobsList
        /// </summary>
        public static void ProcessQuestLog()
        {
            IEnumerable<PlayerQuest> completedQuests = Me.QuestLog.GetAllQuests().Where(q => q.IsCompleted);

            List<string> removeThese = new List<string>();
            foreach (PlayerQuest quest in completedQuests)
            {
                removeThese.AddRange(from pair in _questMobs where pair.Value.QuestID == quest.Id select pair.Key);
            }
            foreach (string s in removeThese)
            {
                _questMobs.Remove(s);
            }
            removeThese.Clear();

            foreach (PlayerQuest quest in completedQuests)
            {
                removeThese.AddRange(from pair in _questObjects where pair.Value.QuestID == quest.Id select pair.Key);
            }
            foreach (string s in removeThese)
            {
                _questObjects.Remove(s);
            }
            

            IEnumerable<PlayerQuest> incompleteQuests = Me.QuestLog.GetAllQuests().Where(q => !q.IsCompleted);
            using (StyxWoW.Memory.AcquireFrame())
            {
                // Retrieve relevant information from Wowhead.com about the quests (mobs, objects etc)
                //Log.FocusLog("----[enumerating quests and pulling from wowhead where necessary]----");
                foreach (PlayerQuest quest in incompleteQuests)//.Where(q => !ProcessedQuestList.Contains(q.Id)))
                {
                    if (ProcessedQuestList.Contains(quest.Id)) continue;

                    //Log.FailLog("==== Quest: " + quest.Name + " [" + quest.Id + "]");
                    int questLogIndex = Lua.GetReturnVal<int>(string.Format("return GetQuestLogIndexByID({0})", quest.Id), 0);

                    foreach (Quest.QuestObjective questObjective in quest.GetObjectives().OrderBy(o => o.Index))
                    {
                        //Log.FailLog("===== Objective: " + questObjective.Type);
                        if (questObjective.Type != Quest.QuestObjectiveType.CollectItem && questObjective.Type != Quest.QuestObjectiveType.KillMob) continue;

                        // Is this objective completed?
                        string objectiveQuery = string.Format("return GetQuestLogLeaderBoard({0},{1})", questObjective.Index + 1, questLogIndex);
                        bool objectiveIsFinished = Lua.GetReturnVal<bool>(objectiveQuery, 2);

                        if (objectiveIsFinished) { continue; }

                        if (questObjective.Type == Quest.QuestObjectiveType.CollectItem)
                        {
                            //Log.FailLog("===== about to get CollectItem info from wowhead.com");
                            GetWoWHeadIDs(quest.Id, questObjective.ID);
                        }
                        else if (questObjective.Type == Quest.QuestObjectiveType.KillMob)
                        {
                            //Log.FailLog("===== adding KillMob info to our kill list: " + questObjective.ID);
                            AddQuestMobToKillList(quest.Id, questObjective.ID);
                        }

                    }

                    if (!ProcessedQuestList.Contains(quest.Id))
                    {
                        ProcessedQuestList.Add(quest.Id);
                    }
                }


                // Remove completed objectives
                // Not necessarily completed QUESTS, just objectives
                //Log.FocusLog("----[checking for objectives to remove from our list]----");
                foreach (PlayerQuest quest in incompleteQuests.Where(q=> ProcessedQuestList.Contains(q.Id)))
                {
                    //Log.FocusLog("==== Quest:" + quest.Name);

                    int questLogIndex = Lua.GetReturnVal<int>(string.Format("return GetQuestLogIndexByID({0})", quest.Id), 0);

                    foreach (Quest.QuestObjective questObjective in quest.GetObjectives().OrderBy(o => o.Index))
                    {
                        //Log.FocusLog("===== Objective:" + questObjective.Objective);

                        if (questObjective.Type != Quest.QuestObjectiveType.CollectItem && questObjective.Type != Quest.QuestObjectiveType.KillMob) continue;

                        // Is this objective completed?
                        string objectiveQuery = string.Format("return GetQuestLogLeaderBoard({0},{1})", questObjective.Index + 1, questLogIndex);
                        bool objectiveIsFinished = Lua.GetReturnVal<bool>(objectiveQuery, 2);

                        if (!objectiveIsFinished) continue;
                        //Log.FocusLog("===== quest objective completed:" + quest.Name);

                        if (questObjective.Type == Quest.QuestObjectiveType.CollectItem)
                        {
                            //Log.FocusLog("===== removing _questObjects ObjectiveID : " + questObjective.ID);
                            _questObjects.Remove(quest.Id + questObjective.ID.ToString());

                        }
                        else if (questObjective.Type == Quest.QuestObjectiveType.KillMob)
                        {
                            //Log.FocusLog("===== removing ObjectiveID : " + questObjective.ID);
                            _questMobs.Remove(quest.Id + questObjective.ID.ToString());
                        }

                    }
                }
            }
        }



        #region Quest Reward Equipment Selection

        // Returns a Dictionary of all equipped items
        private static Dictionary<InventorySlot, WoWItem> EquippedItems
        {
            get
            {
                Dictionary<InventorySlot, WoWItem> equipped = new Dictionary<InventorySlot, WoWItem>();
                WoWItem[] items = StyxWoW.Me.Inventory.Equipped.Items;

                equipped.Clear();
                for (int i = 0; i < 23; i++)
                {
                    equipped.Add((InventorySlot) (i + 1), items[i]);
                }

                return equipped;
            }
        }

        //public static bool Repaired = false;
       // private static readonly WeightSetEx WeightSet = WeightSetEx.CurrentWeightSet;
        

        #endregion


    }
}
