﻿using System.Drawing;
using System.Windows.Forms;

namespace Fpsware.Library3.UI.Cyotek
{
    public class DefaultTabListPageRenderer : TabListPageRenderer
    {
        #region  Public Overridden Methods

        public override void RenderHeader(Graphics g, TabListPage page, TabListPageState state)
        {
            Color fillColor;
            Color textColor;

            const int arrowSize = 9;
            Rectangle fillBounds = page.HeaderBounds;
            fillBounds.Width -= arrowSize;
            Rectangle textRectangle = Rectangle.Inflate(fillBounds, -4, -4);

            // define the most appropriate colors
            if ((state & TabListPageState.Selected) == TabListPageState.Selected)
            {
                fillColor = Color.SteelBlue;
                //fillColor = SystemColors.Highlight;
                textColor = SystemColors.HighlightText;
            }
            else if ((state & TabListPageState.Hot) == TabListPageState.Hot)
            {
                //fillColor = SystemColors.ControlLightLight;
                fillColor = Color.LightSteelBlue;
                textColor = page.Owner.ForeColor;


                int y = fillBounds.Top + ((fillBounds.Height - (arrowSize * 2)) / 2);
                int x = fillBounds.Right;

                Point point1 = new Point(x, y);
                Point point2 = new Point(x + arrowSize, y + arrowSize);
                Point point3 = new Point(x, y + (arrowSize * 2));

                using (Brush brush = new SolidBrush(fillColor))
                {
                    g.FillPolygon(brush, new Point[] { point1, point2, point3 });
                }
            }
            else
            {
                //fillColor = page.Owner.BackColorHeader;
                fillColor = page.Owner.BackColor;
                textColor = page.Owner.ForeColor;
            }

            // fill the background
            using (Brush brush = new SolidBrush(fillColor))
                g.FillRectangle(brush, fillBounds);

            // draw the selection arrow
            if ((state & TabListPageState.Selected) == TabListPageState.Selected)
            {
                int y = fillBounds.Top + ((fillBounds.Height - (arrowSize * 2)) / 2);
                int x = fillBounds.Right;

                Point point1 = new Point(x, y);
                Point point2 = new Point(x + arrowSize, y + arrowSize);
                Point point3 = new Point(x, y + (arrowSize * 2));

                using (Brush brush = new SolidBrush(fillColor))
                {
                    g.FillPolygon(brush, new Point[] { point1, point2, point3 });
                }
            }

            // draw the text
            const TextFormatFlags flags = TextFormatFlags.VerticalCenter | TextFormatFlags.Left | TextFormatFlags.NoPrefix | TextFormatFlags.SingleLine | TextFormatFlags.WordEllipsis;
            TextRenderer.DrawText(g, page.Text, page.Font, textRectangle, textColor, fillColor, flags);

            // focus
            if ((state & TabListPageState.Focused) == TabListPageState.Focused)
            {
                SizeF textSize = TextRenderer.MeasureText(g, page.Text, page.Font, textRectangle.Size, flags);
                const int offset = 2;

                //ControlPaint.DrawFocusRectangle(g, new Rectangle(textRectangle.X, textRectangle.Y, (int) textSize.Width + offset, (int) textSize.Height + offset), textColor, fillColor);
            }
        }

        #endregion  Public Overridden Methods
    }
}
