﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Windows.Forms;

namespace Fpsware.Library3.UI
{
        public class ExtendedGroupBox : GroupBox
        {
            private Color _borderColor = Color.FromArgb(201, 201, 201);
            private int _cornerRadius = 10;
            private bool _dockAllControls;
            private int _textPosition = 8;
            private bool _autoResize = false;
            private int _padding = 0;
            private int _newtop = 19;
            private int _top = 19;
            private int _left = 6;
            private int _sameRowCount = 0;
            private int _controlSize = 0;

            public ExtendedGroupBox()
            {
                DockAllControls = false;
                this.SetStyle(ControlStyles.DoubleBuffer | ControlStyles.AllPaintingInWmPaint | ControlStyles.UserPaint | ControlStyles.ContainerControl, true);
            }

            [NotifyParentProperty(true)]
            public Color BorderColor
            {
                get { return this._borderColor; }
                set
                {
                    this._borderColor = value;
                    Invalidate();
                }
            }

            /*
            public bool AutoResize
            {
                get { return _autoResize; }
                set { _autoResize = value; Invalidate(); }
            }
             */

            public int CornerRadius 
            {
                get { return _cornerRadius; }
                set
                {
                    if (value < 1) value = 1;
                    _cornerRadius = value;
                    Invalidate();
                }
            }


            public bool DockAllControls
            {
                get { return _dockAllControls; }
                set
                {
                    _dockAllControls = value;
                    Invalidate();
                }
            }

            public int TextPosition
            {
                get { return _textPosition; }
                set { _textPosition = value; Invalidate(); }
            }

            protected override void OnPaint(PaintEventArgs e)
            {
                Size tSize = TextRenderer.MeasureText(this.Text, this.Font);

                //Rectangle borderRect = e.ClipRectangle;
                Rectangle borderRect = this.ClientRectangle;
                borderRect.Y += tSize.Height / 2;
                borderRect.Height -= tSize.Height / 2;
                //ControlPaint.DrawBorder(e.Graphics, borderRect, borderColor, BorderStyle);
                DrawRoundedRectangle(e.Graphics, borderRect, _cornerRadius, new Pen(_borderColor), this.BackColor);

                //Rectangle textRect = e.ClipRectangle;
                Rectangle textRect = this.ClientRectangle;
                textRect.X += TextPosition;
                textRect.Width = tSize.Width + 5;
                textRect.Height = tSize.Height;
                e.Graphics.FillRectangle(new SolidBrush(this.BackColor), textRect);
                e.Graphics.DrawString(this.Text, this.Font, new SolidBrush(this.ForeColor), textRect);

                OrganiseAllControls();

                //ResizeControl();
            }

            /*
            private void ResizeControl()
            {
                if (!AutoResize) return;

                int height = DockTop + _controlSize + 10;

                height += DockLeft;
                this.Height = height;
            }
             */


            public int DockPadding
            {
                get { return _padding; }
                set { _padding = value; Invalidate(); }
            }

            public int DockLeft
            {
                get { return _left; }
                set { _left = value; Invalidate(); }
            }

            public int DockTop
            {
                get { return _top; }
                set { _top = value; Invalidate(); }
            }

            
            private void OrganiseAllControls()
            {
                if (!DockAllControls) return;

                _newtop = _top;
                _sameRowCount = 0;
                _controlSize = 0;
                
                List<string> sortedControls = new List<string>();

                foreach (Control c in this.Controls.Cast<Control>().OrderBy(c => c.TabIndex))
                {
                    // if the control has been sorted then ignore it
                    if (sortedControls.Contains(c.Name)) continue;

                    _controlSize += c.Size.Height + DockPadding;
                    int tabIndex = c.TabIndex;
                    
                    c.Location = new Point(_left, _newtop);
                    _sameRowCount = (Controls.Cast<Control>().Count(o => o.TabIndex == tabIndex));

                    // Only 1 item with this TabIndex then set the full width
                    if (_sameRowCount == 1)
                    {
                        c.Width = this.Width - (_left*2);
                    }
                    else
                    {
                        sortedControls.Add(c.Name);

                        // Multiple items with the same tab index, they all go on the same line and the width does not get changed
                        int newLeft = DockLeft + c.Location.X + c.Width;
                        foreach (Control o in this.Controls.Cast<Control>().Where(o => o.TabIndex == tabIndex && !sortedControls.Contains(o.Name)))
                        {
                            o.Location = new Point(newLeft, _newtop);
                            newLeft = DockLeft + o.Location.X + o.Width;
                            sortedControls.Add(o.Name);
                        }
                    }


                    _newtop += (c.Size.Height + _padding);
                }
            }

            private void DrawRoundedRectangle(Graphics gfx, Rectangle bounds, int cornerRadius, Pen drawPen, Color fillColor)
            {
                int strokeOffset = Convert.ToInt32(Math.Ceiling(drawPen.Width));
                bounds = Rectangle.Inflate(bounds, -strokeOffset, -strokeOffset);

                drawPen.EndCap = drawPen.StartCap = LineCap.Round;

                GraphicsPath gfxPath = new GraphicsPath();
                gfxPath.AddArc(bounds.X, bounds.Y, cornerRadius, cornerRadius, 180, 90);
                gfxPath.AddArc(bounds.X + bounds.Width - cornerRadius, bounds.Y, cornerRadius, cornerRadius, 270, 90);
                gfxPath.AddArc(bounds.X + bounds.Width - cornerRadius, bounds.Y + bounds.Height - cornerRadius, cornerRadius, cornerRadius, 0, 90);
                gfxPath.AddArc(bounds.X, bounds.Y + bounds.Height - cornerRadius, cornerRadius, cornerRadius, 90, 90);
                gfxPath.CloseAllFigures();

                gfx.FillPath(new SolidBrush(fillColor), gfxPath);
                gfx.DrawPath(drawPen, gfxPath);
            }
        }
}
