﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Fpsware.Library3.UI
{
    public partial class FPSDropdown : UserControl
    {
        private string _toolTipText;
        private string _settingName;
        private bool _additionalsVisible;
        private Collection<string> _dropdownItems = new Collection<string>();
        private Collection<string> _additionalsEnabledItems = new Collection<string>();
        private Collection<string> _additionalsAllButItems = new Collection<string>();

        // Event stuff when value is changed
        public delegate void DropDownSelectedItemEventHandler(object sender, DropdownValueChangedEventArgs e);
        public event DropDownSelectedItemEventHandler ValueChanged;
        public event DropDownSelectedItemEventHandler ValueChangedOther;
        protected virtual void OnValueChanged(DropdownValueChangedEventArgs e) { try { ValueChanged(this, e); } catch { } }
        protected virtual void OnValueChangedOther(DropdownValueChangedEventArgs e) { try { ValueChangedOther(this, e); } catch { } }

        public delegate void AdditionalsButtonEventHandler(object sender, AddingNewEventArgs e);
        public event AdditionalsButtonEventHandler AdditionalsButtonPressed;
        protected virtual void OnAdditionalsButtonPressed(AddingNewEventArgs e) { try { AdditionalsButtonPressed(this, e); } catch { } }

        public delegate void DropdownEventHandler(object sender, AddingNewEventArgs e);
        public event DropdownEventHandler DropdownOpened;
        protected virtual void OnDropdownOpened(AddingNewEventArgs e) { try { DropdownOpened(this, e); } catch { } }

        public FPSDropdown() { InitializeComponent(); }

        [Category("_Fpsware")]
        [Description("Show or hide the 'additionals' button")]
        [DefaultValue(false)]
        public bool AdditionalsAvailable
        {
            get { return _additionalsVisible; }
            set
            {
                _additionalsVisible = value;

                btnAdditionals.Visible = _additionalsVisible;
                ResizeDropdown();
            }
        }

        [Category("_Fpsware")]
        [Description("Set the list of items in the dropdown")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public Collection<string> DropDownItems
        {
            get { return _dropdownItems; }
        }

        [Browsable(false)]
        public string SelectedItem
        {
            get
            {
                return Dropdown.SelectedItem == null ? null : Dropdown.SelectedItem.ToString();
            }
        }


        [Browsable(false)]
        public void RemoveSelectedItem()
        {
            /*
            foreach (string item in DropDownItems)
            {
                if (item == Dropdown.SelectedItem.ToString())
                    DropDownItems.Remove(Dropdown.SelectedItem.ToString());
            }
             */

            DropDownItems.Remove(Dropdown.SelectedItem.ToString());
            Dropdown.Items.Remove(Dropdown.SelectedItem);
            
        }

        [Browsable(false)]
        public void ClearDropdownOnly()
        {
            //DropDownItems.Remove(Dropdown.SelectedItem.ToString());
            Dropdown.Items.Clear();
        }

        [Browsable(false)]
        public void ClearAll()
        {
            DropDownItems.Clear();
            Dropdown.Items.Clear();
            _dropdownItems.Clear();
            Value = "";
        }

        [Browsable(false)]
        public void AddRange(string[] items)
        {
            foreach (string o in items)
            {
                Dropdown.Items.Add(o);
            }
        }

        [Browsable(false)]
        public void SelectFirstItem()
        {
            try
            {
                if (Dropdown.Items.Count >0) Dropdown.SelectedIndex= 0;
            }
            catch (Exception e)
            {
                //Console.WriteLine(e);
            }
        }


        [Browsable(false)]
        public void ReplaceSelectedItem(string value)
        {
            int index = Dropdown.Items.IndexOf(Dropdown.SelectedItem);
            Dropdown.Items.RemoveAt(index);
            Dropdown.Items.Add(value);
            Dropdown.SelectedItem = value;
            //Dropdown.Items.Insert(index, value);
        }


        [Category("_Fpsware")]
        [Description("Set the list of items from the dropdown that will enable the 'additionals' button")]
        [DefaultValue("Label")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public Collection<string> AdditionalsEnabledItemsList
        {
            get { return _additionalsEnabledItems; }
            set { _additionalsEnabledItems = value; }
        }

        [Category("_Fpsware")]
        [Description("Any option but those listed here will enable the 'additionals' button")]
        [DefaultValue("Label")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public Collection<string> AdditionalsAllButItemsList
        {
            get { return _additionalsAllButItems; }
            set { _additionalsAllButItems = value; }
        }


        [Category("_Fpsware")]
        [Description("Gets or sets the label of this control")]
        [DefaultValue("Description")]
        public string Label
        {
            get { return lblText.Text; }
            set { lblText.Text = value; }
        }

        [Category("_Fpsware")]
        [Description("Gets or sets the spell associated with this control"), DefaultValue("")]
        public string SpellName { get; set; }

        [Category("_Fpsware")]
        [Description("Sets if the the spell exists in the spellmanager"), DefaultValue(true)]
        public bool IsSpellValid
        {
            set
            {
                lblText.ForeColor = value ? Color.Black : Color.Silver;
            }
        }

        [Category("_Fpsware")]
        [Description("Is this a pipe delimitered list")]
        [DefaultValue(false)]
        public bool IsDelimitedList { get; set; }

        [Category("_Fpsware")]
        [DefaultValue("Always")]
        [Description("Gets or sets the selected item. The passed string must exist in the collection")]
        public string Value
        {
            get
            {
                if (!IsDelimitedList)
                {
                    if (Dropdown.Items.Count <= 0) return "";
                    if (Dropdown.SelectedItem == null) return "";

                    return Dropdown.SelectedItem.ToString();
                }
                
                string result = null;
                foreach (var item in Dropdown.Items)
                {
                    // Enumerate the dropdown items and return a PIPE separated string of all values.
                    // Eg... Item1|Item2|Item3|Item4

                    if (!string.IsNullOrEmpty(result))
                    {
                        if (!string.IsNullOrEmpty(item.ToString()))
                            result += result.EndsWith("|") ? item.ToString() : "|" + item;
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(item.ToString()))
                            result = item.ToString();
                    }
                }

                return result;
            }
            set
            {
                if (!IsDelimitedList)
                {
                    if (Dropdown.Items.Count <= 0)
                    {
                        foreach (string s in _dropdownItems)
                        {
                            Dropdown.Items.Add(s);
                        }
                    }
                    Dropdown.SelectedItem = value;
                }
                else
                {
                    if (value == null) return;

                    string[] splitted = value.Split('|');
                    Dropdown.Items.Clear();
                    //DropDownItems.Clear();

                    foreach (string s in splitted)
                    {
                        Dropdown.Items.Add(s);
                    }
                }
            }
        }

        [Category("_Fpsware")]
        [Description("Gets or sets the associated setting")]
        [DefaultValue("")]
        public string SettingName
        {
            get { return _settingName ?? ""; }
            set { _settingName = value; }
        }

        [Category("_Fpsware")]
        [Description("Sorts the items in alphabetical order")]
        public bool SortItems
        {
            get { return Dropdown.Sorted; }
            set { Dropdown.Sorted = value; }
        }

        [Category("_Fpsware")]
        [Description("Gets or sets the tooltip text")]
        [DefaultValue("")]
        public string TooltipText
        {
            get { return _toolTipText; }
            set
            {
                if (value == null) return;

                _toolTipText = value;
                toolTip1.SetToolTip(Dropdown, _toolTipText);
            }
        }

        [Category("_Fpsware")]
        [Description("Gets or sets the title used in the tooltip")]
        public string TooltipTitle
        {
            get { return toolTip1.ToolTipTitle; }
            set
            {
                if (value == null) return;
                toolTip1.ToolTipTitle = value;
            }
        }

        [Category("Misc")]
        [Description("Enables or disables user changes")]
        public new bool Enabled
        {
            get { return Dropdown.Enabled; }
            set
            {
                Dropdown.Enabled = value;
                btnAdditionals.Enabled = value;
                lblText.Enabled = value;
            }
        }

        [Category("_Fpsware")]
        [Description("Sets the width of the label, thus reducing the size of the dropdown box")]
        [DefaultValue(110)]
        public int LabelSize
        {
            get { return lblText.Width; }
            set
            {
                lblText.Width = value;

                if (value > 0)
                {
                    ResizeDropdown();
                }
                else
                {
                    Dropdown.Left = lblText.Width + 0;
                    Dropdown.Width = (Width - Dropdown.Left - 0);
                }
            }
        }

        private void ResizeDropdown()
        {
            Dropdown.Left = lblText.Width + 9;

            if (!AdditionalsAvailable)
            {
                Dropdown.Width = Width - Dropdown.Location.X - 3;
            }
            else
            {
                Dropdown.Width = Width - ((Dropdown.Location.X - 3) + (btnAdditionals.Width + 10));
            }
        }

        private void cbDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (string s in AdditionalsEnabledItemsList)
            {
                if (Dropdown.SelectedItem.ToString() == s)
                {
                    AdditionalsAvailable = true;
                    break;
                }
                AdditionalsAvailable = false;
            }

            if (AdditionalsAllButItemsList.Count > 0)
                foreach (string s in AdditionalsAllButItemsList)
                {
                    if (Dropdown.SelectedItem.ToString() == s)
                    {
                        AdditionalsAvailable = false;
                        break;
                    }
                    AdditionalsAvailable = true;
                }

            OnValueChanged(new DropdownValueChangedEventArgs(Value, SettingName));
            OnValueChangedOther(new DropdownValueChangedEventArgs(Value, SettingName));
        }

        private void ucDropdown_Load(object sender, EventArgs e)
        {
            if (Dropdown.Items.Count != 0) return;
            foreach (string s in _dropdownItems)
            {
                Dropdown.Items.Add(s);
            }
        }

        private void btnAdditionals_Click(object sender, EventArgs e)
        {
            OnAdditionalsButtonPressed(new AddingNewEventArgs(SettingName));
        }

        private void Dropdown_DropDown(object sender, EventArgs e)
        {
            OnDropdownOpened(new AddingNewEventArgs(SettingName));
        }
    }

    public class DropdownValueChangedEventArgs : EventArgs
    {
        public DropdownValueChangedEventArgs(string value, string settingName)
        {
            SettingName = settingName;
            Value = value;
        }

        public string Value { get; private set; }
        public string SettingName { get; private set; }
    }

    public class AdditionalsButtonClickedEventArgs : EventArgs
    {
        public AdditionalsButtonClickedEventArgs(string settingName)
        {
            SettingName = settingName;
        }

        public string SettingName { get; private set; }
    }
}
