﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Fpsware.Library3.UI
{
    public sealed partial class FPSProgressBarExt : UserControl
    {
        LabelWithClickThrough label1 = new LabelWithClickThrough();

        public FPSProgressBarExt()
        {
            InitializeComponent();

            label1.BackColor = Color.Transparent;
            this.Controls.Add(label1);


            BackColor = Color.Gainsboro;
            label1.ForeColor = Color.Black;
            ForeColor = SystemColors.Highlight;
            label1.Location = new Point(Width / 2 - 21 / 2 - 4, Height / 2 - 15 / 2);
        }

        private Color _borderColor = Color.DarkGray;
        private float _valueForAnimation = 0;
        private float min = 0;
        private float max = 100;
        private float _value = 35;

        public float Minimum
        {
            get
            {
                return min;
            }

            set
            {
                // Prevent a negative value.
                if (value < 0)
                {
                    min = 0;
                }

                // Make sure that the minimum value is never set higher than the maximum value.
                if (value > max)
                {
                    min = value;
                    //min = value;
                }

                // Ensure value is still in range
                if (_value < min)
                {
                    _value = min;
                }

                // Invalidate the control to get a repaint.
                Invalidate();
            }
        }

        public float Maximum
        {
            get
            {
                return max;
            }

            set
            {
                // Make sure that the maximum value is never set lower than the minimum value.
                if (value < min)
                {
                    min = value;
                }

                max = value;

                // Make sure that value is still in range.
                if (_value > max)
                {
                    _value = max;
                }

                // Invalidate the control to get a repaint.
                Invalidate();
            }
        }
        
        public float Value
        {
            get
            {
                return _value;
            }

            set
            {
                float oldValue = _value;

                // Make sure that the value does not stray outside the valid range.
                if (value < min)
                {
                    _value = min;
                }
                else if (value > max)
                {
                    _value = max;
                }
                else
                {
                    _value = value;
                }

                // Invalidate only the changed area.
                Rectangle newValueRect = ClientRectangle;
                Rectangle oldValueRect = ClientRectangle;

                // Use a new value to calculate the rectangle for progress.
                float percent = (_value - min) / (max - min);
                newValueRect.Width = (int)(newValueRect.Width * percent);
                
                label1.Text = string.Format("{0}%", percent * 100);

                // Use an old value to calculate the rectangle for progress.
                percent = (oldValue - min) / (max - min);
                oldValueRect.Width = (int)(oldValueRect.Width * percent);

                

                Rectangle updateRect = new Rectangle();

                // Find only the part of the screen that must be updated.
                if (newValueRect.Width > oldValueRect.Width)
                {
                    updateRect.X = oldValueRect.Size.Width;
                    updateRect.Width = newValueRect.Width - oldValueRect.Width;
                }
                else
                {
                    updateRect.X = newValueRect.Size.Width;
                    updateRect.Width = oldValueRect.Width - newValueRect.Width;
                }

                updateRect.Height = Height;

                // Invalidate the intersection region only.
                Invalidate(updateRect);
            }





        }

        public string SettingName { get; set; }

        [Category("Data")]
        [Description("Gets or sets the border color of the control")]
        public Color BorderColor
        {
            get
            {
                return _borderColor;
            }

            set
            {
                _borderColor = value;
                Invalidate();
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            Brush b = new SolidBrush(ForeColor);

            float percent = (_value - min) / (max - min);
            Rectangle rect = ClientRectangle;
            int width = (int)(rect.Width * percent);

            e.Graphics.FillRectangle(b, 0, 0, width, Height);

            b.Dispose();

            const int borderSize = 1;

            ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
                BorderColor, borderSize, ButtonBorderStyle.Solid,
                BorderColor, borderSize, ButtonBorderStyle.Solid,
                BorderColor, borderSize, ButtonBorderStyle.Solid,
                BorderColor, borderSize, ButtonBorderStyle.Solid
                );
        }

        private void PBar_SizeChanged(object sender, EventArgs e)
        {
            label1.Location = new Point(Width/2 - 21/2 - 4, Height/2 - 15/2);
        }

        private void PBar_Resize(object sender, EventArgs e)
        {
            Invalidate();
        }

        public delegate void LineEventHandler(object sender, ProgressBarEventArgs e);
        public event LineEventHandler ValueChanged;
        private void OnValueChanged(ProgressBarEventArgs e) { try { ValueChanged(this, e); } catch (Exception exception) { } }

        private void CommonProgressBar_MouseAction(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left) return;

            Value = MousePosValue(e.Location.X, Width);

            // Trigger the 'value changed' event
            OnValueChanged(new ProgressBarEventArgs((int)Value, SettingName));


        }

        private int MousePosValue(double mousePosition, double progressBarWidth)
        {
            if (mousePosition < 0) mousePosition = 0;
            if (mousePosition > progressBarWidth) mousePosition = progressBarWidth;

            double ratio = mousePosition / progressBarWidth;
            double value = ratio * max;

            if (value >= max) value = max;
            if (value <= min) value = min;

            return (int)Math.Ceiling(value);
        }

        public class ProgressBarEventArgs : EventArgs
        {
            public ProgressBarEventArgs(int value, string settingName)
            {
                SettingName = settingName;
                Value = value;
            }

            public int Value { get; private set; }
            public string SettingName { get; private set; }
        }

        /*
        private void label1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left) return;
            
            Value = MousePosValue(e.Location.X, Width);

            // Trigger the 'value changed' event
            OnValueChanged(new ProgressBarEventArgs((int)Value, SettingName));
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }
         */

    }

    internal class LabelWithClickThrough : Label
    {
        protected override void WndProc(ref Message m)
        {
            const int WM_NCHITTEST = 0x0084;
            const int HTTRANSPARENT = (-1);

            if (m.Msg == WM_NCHITTEST)
            {
                m.Result = (IntPtr)HTTRANSPARENT;
            }
            else
            {
                base.WndProc(ref m);
            }
        }
    }
}
