﻿using System;
using CommonBehaviors.Actions;
using Fpsware.Botbases.RAFBot2.Behaviors;
using Fpsware.Library3.Composites;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.TreeSharp;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using Misc = Fpsware.Library3.Helpers.Misc;
using DecoratorThrottle = Fpsware.Library3.Extensions.TreeSharpExt.DecoratorThrottle;
using DebugMessage = Fpsware.Library3.Extensions.TreeSharpExt.LogMessage;
using ActionFail = Fpsware.Library3.Extensions.TreeSharpExt.ActionFail;

namespace Fpsware.Botbases.RAFBot2
{
    public class Alive
    {
        #region Local Declarations

        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return StyxWoW.Me.CurrentTarget; } }
        private static WoWUnit Pet { get { return StyxWoW.Me.Pet; } }
        private const int MinFreeBagSlots = 2;

        #endregion

        public static Composite CreateAliveCombatBehavior()
        {
            return
                new PrioritySelector(

                    // Restore HB navigation provier
                    new Decorator(ret => Movement.IsNavigationProviderOverridden && !EncounterLogicActions.IsTriggeredLogicStillInProgress, new Action(context => Movement.RestoreOriginalNavigationSystem())),

                    // Heal
                    new Decorator(ret => Setting.Instance.BehaviorHeal.ResultOK() && !Me.IsEatingOrDrinking() && (Setting.Instance.HealingBehaviorWhen.Contains("Always") || (Setting.Instance.HealingBehaviorWhen.Contains("Combat") && Me.Combat)),
                        RoutineManager.Current.HealBehavior),

                    // Pre Combat Behavior
                    new Decorator(ret => Setting.Instance.BehaviorPreCombat.ResultOK() && !Me.Combat && !Me.Mounted, RoutineManager.Current.PreCombatBuffBehavior),

                    // Pull Behavior if we have a valid target
                    new Decorator(ret => IsPullOption1 || IsPullOption2 || IsPullOption3 || IsPullOption4,
                        new PrioritySelector(
                            new Decorator(ret => IsPullOption1 || IsPullOption3, new Action(context => RAFGroup.Leader.CurrentTarget.Target())),
                            new Decorator(ret=> BotPoi.Current != null && BotPoi.Current.Type != PoiType.Kill,
                                new ActionFail(context =>
                                {
                                    BotPoi.Current = new BotPoi(Me.CurrentTarget, PoiType.Kill);
                                    Log.Debug("BOTPOI set to kill!");
                                })),
                            new Decorator(ret => Setting.Instance.Movement.Contains("Full Combat Movement") && !Misc.IsUserMoving, CreateCharacterMovement()),
                            //new DebugMessage("should be doing pull behavior now!"),
                            new PrioritySelector(
                                RoutineManager.Current.PullBuffBehavior, 
                                RoutineManager.Current.PullBehavior
                            )
                        )),

                    // Combat Behaviors, the important stuff
                    new Decorator(ret => Me.Combat || Setting.Instance.ForceCombatBehavior.ResultOK() || (Setting.Instance.Targeting.Contains("Assist Leader") && RAFGroup.Leader != null && RAFGroup.Leader.Combat),
                        new PrioritySelector(

                            // Take the leader's target
                            new Decorator(ret => !Me.GotValidTarget() && RAFBot2.IsLeaderValid && RAFGroup.Leader.GotValidTarget() || Setting.Instance.Targeting.Contains("Assist Leader") && RAFBot2.IsLeaderValid && RAFGroup.Leader.GotValidTarget() && (!Me.GotTarget || RAFBot2.IsLeaderValid && CT.Guid != RAFGroup.Leader.CurrentTargetGuid), new Action(context => RAFGroup.Leader.CurrentTarget.Target())),
                            
                            // Make sure our Poi is set to kill
                            new Decorator(ret=> !PoiBehaviors.IsCurrentPoiType(PoiType.Kill) && Me.GotValidTarget() && CT.IsValidTarget(), new ActionFail(context => BotPoi.Current = new BotPoi(Me.CurrentTarget, PoiType.Kill))),

                            // Encounter Logic behavior
                            new Decorator(ret => Setting.Instance.UseEncounterLogic.OK() && !Misc.IsUserMoving && (EncounterLogicActions.IsTriggeredLogicStillInProgress || EncounterLogicActions.IsEncounterConditionMet),
                                new PrioritySelector(

                                    //new TreeSharpExt.LogMessage("==== encounter logic 1"),
                                    // Ensure HB navigation is restored so we can move.
                                    new ActionFail(context => { if (Movement.IsNavigationProviderOverridden) Movement.RestoreOriginalNavigationSystem(); }),
                                    //new TreeSharpExt.LogMessage("==== encounter logic 2"),

                                    // Do the important stuff, move away from shit!
                                    EncounterLogicActions.CreateEncounterLogicActions(),
                                    //new TreeSharpExt.LogMessage("==== encounter logic 3"),

                                    // If we're not moving at least face the target
                                    new Decorator(ret=> !Me.IsMoving && Me.GotTarget && !Me.IsSafelyFacing(CT), new Action(context => CT.Face())),
                                    //new TreeSharpExt.LogMessage("==== encounter logic 4"),
                                    new ActionFail(context => { if (!Movement.IsNavigationProviderOverridden) Movement.SetNavigationSystemNoMovement(); })

                                    // While the encounter condition is in effect don't go any further
                                    //new ActionAlwaysSucceed()
                                    //new Decorator(ret => !Movement.IsNavigationProviderOverridden, new TreeSharpExt.ActionFail(context => Movement.SetNavigationSystemNoMovement()))
                                )),

                            // Movement
                            new Decorator(ret => Setting.Instance.Movement.Contains("Full Combat Movement") && !Misc.IsUserMoving && !Me.Mounted, CreateCharacterMovement()),

                            // Combat Buff
                            new Decorator(ret => Setting.Instance.BehaviorCombatBuff.ResultOK(), RoutineManager.Current.CombatBuffBehavior),

                            // Actual Combat Behavior
                            new PrioritySelector(RoutineManager.Current.CombatBehavior)

                ))
            );
        }

        /// <summary>
        /// Alive out of combat behavior
        /// </summary>
        /// <returns></returns>
        public static Composite CreateOutOfCombatBehavior()
        {
            return
                new PrioritySelector(

                    // Misc setups and stuff
                    new Decorator(ret => Setting.Instance.LootingHarvest.OK() && Harvestable.MaxDistance != Setting.Instance.HarvestRange, new Action(context => Harvestable.MaxDistance = Setting.Instance.HarvestRange)),
                    new Decorator(ret => (Setting.Instance.LootingMobs.OK() || Setting.Instance.LootingSkinMobs.OK()) && Loot.MaxDistance != Setting.Instance.HarvestRange, new Action(context => Harvestable.MaxDistance = Setting.Instance.LootRange)),

                    // Heal
                    new Decorator(ret => Setting.Instance.BehaviorHeal.ResultOK() && !Me.IsEatingOrDrinking() && (Setting.Instance.HealingBehaviorWhen.Contains("Always") || (Setting.Instance.HealingBehaviorWhen.Contains("Combat") && Me.Combat)),
                        RoutineManager.Current.HealBehavior),

                    // Rest
                    new Decorator(ret => Setting.Instance.BehaviorRest.ResultOK() && !Me.Combat, RoutineManager.Current.RestBehavior),

                    // Eating or Drinking then stop here
                    new Decorator(ret => Me.IsEatingOrDrinking(), new ActionAlwaysSucceed()),

                    // Loot mobs, or display a warning message about full bags
                    new Decorator(ret => Setting.Instance.LootingMobs.OK() && (Setting.Instance.Leader == "[None]" || RAFBot2.IsLeaderValid && !RAFGroup.Leader.Combat) && !Me.IsActuallyInCombat && Loot.LootableWoWUnit != null,
                        new PrioritySelector(

                            // Full bags? Display a warning message.
                            new DecoratorThrottle(ret => Me.FreeNormalBagSlots <= MinFreeBagSlots, TimeSpan.FromSeconds(5), new Action(c => Log.FailLog("Bags are full. No longer attempting to loot mobs"))),

                            // Loot.
                            // Because we're likely to be in a group use the trustworthy looting behavior. This will ensure all items are looted.
                            new Decorator(ret => Me.FreeNormalBagSlots > MinFreeBagSlots && !Misc.IsUserMoving && Loot.LootableWoWUnit != null,
                                new PrioritySelector(
                                    new Loot.LootingBehaviorPS(ret => Loot.LootableWoWUnit),
                                    new ActionAlwaysSucceed()
                                ))
                    )),

                    // Lets get to SKINNING
                    new Decorator(ret => Setting.Instance.LootingSkinMobs.OK() && !Me.IsFlying && Loot.SkinnableWoWUnit != null && Inventory.SkinningBagSlotsFree >= MinFreeBagSlots && Loot.SkinnableWoWUnit.CanSkin, 
                        new PrioritySelector(
                            new PrioritySelector(
                                new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Skinning " + Loot.SkinnableWoWUnit.Name; })),
                                //new Decorator(ret => Me.IsFlying, new Movement.LandAndDismount("Skinning, but we're flying. We should be on the ground")),
                                new Loot.QuickLootingBehaviorPS(ret => Loot.SkinnableWoWUnit, ret => Setting.Instance.LootingSkinMobs.OK()),
                                new ActionAlwaysSucceed()
                                ))
                            ),

                    // Lets get to harvesting
                    new Decorator(ret => Setting.Instance.LootingHarvest.OK() && !Me.IsFlying && Harvestable.LootableWoWGameObject != null && Me.FreeNormalBagSlots <= MinFreeBagSlots, 
                        new PrioritySelector(
                            
                            new Decorator(ret => ((Harvestable.LootableWoWGameObject.IsHerb && Inventory.HerbBagSlotsFree > MinFreeBagSlots) || (Harvestable.LootableWoWGameObject.IsMineral && Inventory.MiningBagSlotsFree > MinFreeBagSlots)) && Harvestable.LootableWoWGameObject.CanUse(),
                                new PrioritySelector(
                                    new ThrottlePasses(1, 3, new Action(context => { TreeRoot.StatusText = "Harvesting " + Harvestable.LootableWoWGameObject.Name; })),
                                    //new Decorator(ret => Me.IsFlying && Harvestable.LootableWoWGameObject.Distance < 10, new Movement.LandAndDismount("Harvesting, but we're flying. We should be on the ground")),
                                    new Harvestable.QuickHarvestBehaviorPS(ret => Harvestable.LootableWoWGameObject),
                                    new ActionAlwaysSucceed()
                                    ))
                            )),

                    
                
                    // Quest stuff
                    new Decorator(ret => !Me.Combat && (Setting.Instance.QuestPickUp.OK() || Setting.Instance.QuestTurnIn.OK() || (Setting.Instance.QuestUseAdvancedFeatures.OK()/* && Setting.Instance.QuestObjectiveKills.OK()*/)), QuestBehaviors.CreateQuestBehavior())

                
                    // vendoring / repair stuff
                //new Decorator(ret => !Me.Combat && !Setting.Instance.Leader.Contains("None"), Vendoring.CreateVendorRepair()),

                // This is where our dungeon stuff should go.



                );
        }

        public static Composite CreateCharacterMovement()
        {
            return new PrioritySelector(

                new Decorator(ret => Me.IsMelee() && CT != null,
                    new PrioritySelector(
                        new Movement.StopMovingPS(ret => Movement.StopMovingWhen.InMeleeRangeOfUnit, ret => CT, ret => 1, ret => CT.Location),
                        new Decorator(ret => !CT.IsWithinMeleeRange,
                            new Sequence(
                                new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Navigator, ret => Movement.MountStrategyType.None, ret=> false),
                                new ActionAlwaysFail()
                        ))
                    )),

                new Decorator(ret => Me.IsCaster() && CT != null,
                    new PrioritySelector(
                        new Decorator(ret => CT.Distance > Setting.Instance.MoveToTargetDistance || !CT.InLineOfSpellSight, new Movement.MoveTo(ret => CT.Location, ret => Movement.MovementProviderType.Navigator, ret => Movement.MountStrategyType.None, ret => false)),
                        new Decorator(ret=> CT.InLineOfSpellSight, new Movement.StopMovingPS(ret => Movement.StopMovingWhen.InRangeOfUnit, ret => CT, ret => Setting.Instance.MoveToTargetDistance))
                        
                    )),

                new Decorator(ret => !Me.IsMoving && Me.GotValidTarget() && !Me.IsSafelyFacing(CT), Movement.CreateFaceTargetBehavior())
                
                );
        }

        #region Various pull condition checks

        private static bool IsPullOption1
        {
            get
            {
                return !Me.Combat && Setting.Instance.BehaviorPull.Contains("Pull Leaders Target") && (RAFBot2.IsLeaderValid && RAFGroup.Leader.GotValidTarget()) && (!Me.GotTarget || CT.Guid != RAFGroup.Leader.CurrentTargetGuid);
            }
        }

        private static bool IsPullOption2
        {
            get
            {
                return !Me.Combat && ((Setting.Instance.QuestUseAdvancedFeatures.OK() && Setting.Instance.QuestObjectiveKills.OK() && PoiBehaviors.IsCurrentPoiType(PoiType.Kill)) || Setting.Instance.BehaviorPull.ResultOK()) && Me.GotValidTarget();
            }
        }

        private static bool IsPullOption3
        {
            get
            {
                return !Me.Combat && Setting.Instance.BehaviorPull.ResultOK() && (RAFBot2.IsLeaderValid && RAFGroup.Leader.GotValidTarget()) && (!Me.GotTarget || CT.Guid != RAFGroup.Leader.CurrentTargetGuid);
            }
        }

        private static bool IsPullOption4
        {
            get { return !Me.Combat && PoiBehaviors.IsCurrentPoiType(PoiType.Kill); }
        }

        #endregion


    }
}
