﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using JetBrains.Annotations;
using Styx;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Botbases.RAFBot2
{
    public static class GrinderHelpers
    {

        private static string _parsedGrinderString = "";
        [UsedImplicitly] private static List<GrinderMob> _grinderMobs = new List<GrinderMob>();
        public static List<GrinderMob> GrinderMobsList()
        {
            // Nothing has changed? Return the cached list
            if (string.IsNullOrEmpty(Setting.Instance.GrinderMobIDs) || _parsedGrinderString == Setting.Instance.GrinderMobIDs)
            {
                return _grinderMobs;
            }

            // Make the list
            _grinderMobs.Clear();
            string[] split = Setting.Instance.GrinderMobIDs.Split('|');
            foreach (GrinderMob grinderMob in split.Select(s => new GrinderMob(s)))
            {
                _grinderMobs.Add(grinderMob);
            }

            _parsedGrinderString = Setting.Instance.GrinderMobIDs;
            return _grinderMobs;
        }

        //private static List<WoWPoint> _workingHotspotList = new List<WoWPoint>();
        private static List<WoWPoint> _workingHotspotList = new List<WoWPoint>();
        public static List<WoWPoint> WorkingHotspotList
        {
            get
            {
                if (_workingHotspotList.Count == 0 && MasterHotspotsList().Count > 0)
                {
                    //Log.FailLog("==== active list is empty, lets populate it");
                    _workingHotspotList.AddRange(MasterHotspotsList());
                }

                //Log.FailLog("==== _workingHotspotList: " + _workingHotspotList.Count);
                return _workingHotspotList;
            }
        }

        private static WoWPoint _currentHotspot = WoWPoint.Empty;
        public static WoWPoint CurrentHotspot
        {
            get
            {
                if (_currentHotspot == WoWPoint.Empty || _currentHotspot.ToString() == "<0, 0, 0>")
                {
                    //Log.FailLog("==== lets full the hotspot thingy again");
                    //_currentHotspot = WorkingHotspotList.OrderBy(h => h.Distance(StyxWoW.Me.Location)).FirstOrDefault();
                    //_currentHotspot = WorkingHotspotList.Where(h=> h.Distance(StyxWoW.Me.Location) > 50).OrderBy(h => h.Distance(StyxWoW.Me.Location)).FirstOrDefault();
                    _currentHotspot = WorkingHotspotList.OrderBy(h => h.Distance(StyxWoW.Me.Location)).FirstOrDefault();
                    /*
                    foreach (WoWPoint point in WorkingHotspotList)
                    {
                        Log.FocusLog("=== point: " + point);
                    }
                     */
                }

                //Log.FailLog("==== _currentHotspot: " + _currentHotspot);
                return _currentHotspot;
            }

            set { _currentHotspot = value; }
        }


        private static string _parsedGrinderHotspotString = "";
        [UsedImplicitly]
        private static List<WoWPoint> _masterHotspotsList = new List<WoWPoint>();
        
        public static List<WoWPoint> MasterHotspotsList()
        {
            // Nothing has changed? Return the cached list
            if (string.IsNullOrEmpty(Setting.Instance.GrinderHotspots) || _parsedGrinderHotspotString == Setting.Instance.GrinderHotspots)
            {
                return _masterHotspotsList;
            }

            // Make the list
            _masterHotspotsList.Clear();
            string[] split = Setting.Instance.GrinderHotspots.Split('|');
            foreach (string s in split)
            {
                _masterHotspotsList.Add(MapHelpers.StringToWoWPoint(s));
            }

            _parsedGrinderHotspotString = Setting.Instance.GrinderHotspots;
            return _masterHotspotsList;
        }

        public static bool IsNearHotspot(WoWUnit u, int maxDistance)
        {
            return MasterHotspotsList().Any(point => !(u.Location.Distance(point) > maxDistance));
        }


        public class GrinderMob
        {
            public string Name { get; set; }
            public uint ID { get; set; }


            public GrinderMob()
            {
                Name = "";
                ID = 0;
            }

            public GrinderMob(string rawString)
            {
                GrinderMobRawString = rawString;
            }


            private string _rawString;
            public string GrinderMobRawString
            {
                get
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendFormat("{0} [", Name);
                    sb.AppendFormat("ID:{0}]", ID);

                    return sb.ToString();
                }

                set
                {
                    _rawString = value;

                    string nameTemp = Regex.Match(_rawString, @"^(.(?!\[))*", RegexOptions.IgnoreCase).Value;
                    string idTemp = Regex.Match(_rawString, @"(ID:\d+)", RegexOptions.IgnoreCase).Value;

                    Name = nameTemp;
                    ID = Convert.ToUInt32(Regex.Match(idTemp, @"\d+", RegexOptions.IgnoreCase).Value);

                }
            }
        }



    }

    
}
