﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Fpsware.Botbases.RAFBot2.Behaviors;
using Fpsware.Botbases.RAFBot2.Helpers;
using Fpsware.Botbases.RAFBot2.Questing;
using Fpsware.Botbases.RAFBot2.UI;
using Fpsware.Library3.Composites;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.Utility;
using Styx;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using ActionFail = Fpsware.Library3.Extensions.TreeSharpExt.ActionFail;
using Misc = Fpsware.Library3.Helpers.Misc;

namespace Fpsware.Botbases.RAFBot2
{
    public class RAFBot2 : BotBase
    {
        #region Overrides
        public override PulseFlags PulseFlags { get { return PulseFlagsLocal; } }
        public override bool RequiresProfile { get { return false; } }
        public override string Name { get { return "Fpsware RAF Bot 2"; } }
        public override Composite Root { get { return _root ?? (_root = new PrioritySelector(CreateRootBehavior())); } }

        #endregion

        #region Local Declarations

        public static PulseFlags PulseFlagsLocal;
        private Composite _root;
        public static bool IsBotPaused = false;
        public static bool IsStartupFinished = false;
        public static bool IsActionTriggersTimeReset = false;
        private readonly SvnUpdater _svnUpdater = new SvnUpdater();
        private RAFBotComms.IService _rbcService;

        public static bool IsLeaderValid { get { return Setting.Instance.Leader != "[None]" && RAFGroup.Leader != null && RAFGroup.Leader.IsValid; } }
        

        public static bool GrinderNavigatePointMoveToTest;
        public static WoWPoint GrinderNavigatePoint;
        
        private static WoWPoint _homeLocation = WoWPoint.Empty;
        public static WoWPoint HomeLocation
        {
            get { return _homeLocation; }
            set { _homeLocation = value; }
        }

        public static LocalPlayer Me { get { return StyxWoW.Me; } }
        public static WoWUnit CT { get { return StyxWoW.Me.CurrentTarget; } }
        public static WoWUnit Pet { get { return StyxWoW.Me.Pet; } }

        #endregion

        #region UI (override)
        private UIForm _frm;

        public override Form ConfigurationForm
        {
            get
            {
                try { if (_frm == null || !_frm.Visible) _frm = new UIForm(); return _frm; } catch (Exception e) { }

                return null;
            }
        }

        #endregion

        #region Pulse (override)
        public override void Pulse() { base.Pulse(); }

        #endregion

        #region Start, Stop, Initialize (override)
        public override void Initialize()
        {
            Setting.Instance.Load();
            GlobalSetting.Instance.Load();
            BotEvents.Player.OnMapChanged += Player_OnMapChanged;

            // SVN updater
            _svnUpdater.Repository = SvnUpdater.RepositoryType.Assembla;
            _svnUpdater.SVNProject = "fpsware-raf-bot";
            _svnUpdater.DownloadPath = Misc.GetBotPath();
            _svnUpdater.NameOfDownload = "Fpsware RAF Bot";

            // Run the updater if enabled - but not if this is my [development] computer(s)
			// I removed the ! in front of !Query....Mike, this way it ALWAY's bails on updating
            if (GlobalSetting.Instance.AutoUpdate.OK() && Query.IsThisMyComputer("Mike-PC|Mike-Laptop")) 
                _svnUpdater.AutoUpdate();

            base.Initialize();
        }

        private readonly List<string> _combatRoutinesPullIssue = new List<string> { "Pure"};
        public override void Start()
        {
            Log.Info(Name + " starting...");
            Lua.DoString("SetCVar('AutoLootDefault','0')");

            GrinderNavigatePointMoveToTest = false;
            IsStartupFinished = false;

            Movement.RecordNavigationSystem();
            
            PulseFlagsLocal = PulseFlags.Objects | PulseFlags.Lua | PulseFlags.BotEvents | PulseFlags.InfoPanel | PulseFlags.Targeting | PulseFlags.Plugins;

            GlobalSettings.Instance.LogoutForInactivity = false;
            Hotkeys.RegisterHotkeys();

            StatsCounter.SettingsFolder = "Fpsware RAF Bot";
            StatsCounter.CounterUrl = "http://c.statcounter.com/9200427/0/132a978f/0/";
            StatsCounter.Run();

            Lua.Events.AttachEvent("LFG_ROLE_CHECK_SHOW", Helpers.Helpers.OnLfgRoleCheck);
            Lua.Events.AttachEvent("LFG_PROPOSAL_SHOW", Helpers.Helpers.OnLfgProposalShow);
            Lua.Events.AttachEvent("RESURRECT_REQUEST", Helpers.Helpers.OnRecurrectionHandler);

            QuestingHelpers.AttachQuestingEvents();

            if (Setting.Instance.UseGrinder.OK())
            {
                GrinderActionTrigger.ActionTriggersList().ResetActionTriggerTimers();
            }

            HomeLocation = MapHelpers.StringToWoWPoint(Setting.Instance.HomeLocation);
            
            Log.Info("Start up process complete");

            base.Start();
        }

        public override void Stop()
        {
            Log.Info(Name + " stopping...");
            Lua.DoString("SetCVar('AutoLootDefault','1')");

            GlobalSettings.Instance.LogoutForInactivity = true;
            Hotkeys.UnregisterHotkeys();

            Lua.Events.DetachEvent("LFG_ROLE_CHECK_SHOW", Helpers.Helpers.OnLfgRoleCheck);
            Lua.Events.DetachEvent("LFG_PROPOSAL_SHOW", Helpers.Helpers.OnLfgProposalShow);
            Lua.Events.DetachEvent("RESURRECT_REQUEST", Helpers.Helpers.OnRecurrectionHandler);

            QuestingHelpers.DetatchQuestingEvents();

            RAFBotComms.UnregisterServer();
            Movement.RestoreOriginalNavigationSystem();
            

            base.Stop();

            Log.Info(Name + " has stopped");
        }
        #endregion

        #region Rebuild root behavior on map change
        private void Player_OnMapChanged(BotEvents.Player.MapChangedEventArgs args) { _root = null; }

        #endregion
        
        private Composite CreateRootBehavior()
        {
            return new PrioritySelector(

                #region Miscelanious start up stuff
                new Decorator(ret => !IsStartupFinished,
                    new Action(context =>
                    {
                        // Register the WCF server
                        if (Setting.Instance.AdvancedUseRBC.Contains("Server") && !string.IsNullOrEmpty(Setting.Instance.AdvancedRBCServer) && !string.IsNullOrEmpty(Setting.Instance.AdvancedRBCPort))
                        {
                            if (!RAFBotComms.IsServerRegistered) { RAFBotComms.RegisterServer(); }
                        }
                        // Register RBC client
                        if (Setting.Instance.AdvancedUseRBC.Contains("Client") && !string.IsNullOrEmpty(Setting.Instance.AdvancedRBCServer) && !string.IsNullOrEmpty(Setting.Instance.AdvancedRBCPort) && !RAFBotComms.IsClientRegistered)
                        {
                            try { RAFBotComms.RegisterClient(); _rbcService = new RAFBotComms.ServiceImplementation(); } catch (Exception e) { }
                        }

                        IsStartupFinished = true;
                    })
                    ),

                #endregion

                #region Get global key bind changes from leader using RBC

                new ThrottlePasses(1, 2,
                    new Decorator(ret => Setting.Instance.AdvancedUseRBC != "Disabled" && IsLeaderValid, new ActionFail(context =>
                    {
                        string rbcGlobalKeyBinds = RAFBotComms.KeyBindQuery;
                        if (string.IsNullOrEmpty(rbcGlobalKeyBinds)) return;

                        string[] splited = rbcGlobalKeyBinds.Split('|');

                        foreach (string s in splited)
                        {
                            string key = s.GetRegXProperty(@"(?<=\[).*?(?=[\]\:])", "");
                            string value = s.GetRegXProperty(@"(?<=:).*?(?=[\]\:])", "");
                            switch (key)
                            {
                                case "ADVANCEDQUESTING":
                                    if (Setting.Instance.QuestUseAdvancedFeatures != value)
                                    {
                                        Log.Info("Key bind passed from leader using RBC");
                                        Log.Info("Setting: UseAdvancedFeatures = " + value);
                                        Setting.Instance.QuestUseAdvancedFeatures = value;
                                    }

                                    break;
                            }
                        }
                    }))),

                #endregion


                // Dead or ghost
                new Decorator(ret => Me.IsDead || Me.IsGhost, DeathAndGhost.CreateDeathAndGhostBehavior()),

                // Maintain our bot poi
                PoiBehaviors.PoiManagement(),

                // Maintain our action trigger timers
                new Decorator(ret => !Setting.Instance.UseGrinder.OK() && IsActionTriggersTimeReset, new Action(context => IsActionTriggersTimeReset = false)),

                // Grinder navigation move to wow point. 
                new Decorator(ret => GrinderNavigatePointMoveToTest,
                    new PrioritySelector(
                        new Decorator(ret => GrinderNavigatePoint != WoWPoint.Empty && Me.Location.Distance(GrinderNavigatePoint) <= 5, new Action(
                            context =>
                            {
                                GrinderNavigatePointMoveToTest = false;
                                GrinderNavigatePoint = WoWPoint.Empty;
                            })),
                        new Decorator(ret => Setting.Instance.GrinderPreferFlyingMount.OK() && GrinderNavigatePoint != WoWPoint.Empty && Me.Location.Distance(GrinderNavigatePoint) > 5, new Action(context => Flightor.MoveTo(GrinderNavigatePoint))),
                        new Decorator(ret => GrinderNavigatePoint != WoWPoint.Empty && Me.Location.Distance(GrinderNavigatePoint) > 5, new Action(context => Navigator.MoveTo(GrinderNavigatePoint)))
                )),

                // Use Grinder
                new Decorator(ret => Setting.Instance.UseGrinder.OK() && Me.IsAlive, Grinder.CreateGrinderBehavior()),

                // Everything else (rest, heal, pull, buffs, combat etc)
                new Decorator(ret=> Me.IsAlive, 
                    new PrioritySelector(

                        // Pre Combat / Combat
                        new Decorator(ret => Me.Combat || (!Me.Combat && Me.GotValidTarget() && (Setting.Instance.BehaviorPull.OK() || Setting.Instance.QuestUseAdvancedFeatures.OK() && Setting.Instance.QuestObjectiveKills.OK())) || IsLeaderValid && RAFGroup.Leader.GotValidTarget(),  
                            Alive.CreateAliveCombatBehavior()),
                        
                        // Out of Combat (looting, resting etc)
                        Alive.CreateOutOfCombatBehavior()
                        
                        )),

                #region Follow our leader
                // Use RBC to find the leader
                new Decorator(ret => Setting.Instance.Leader != "[None]" && RAFGroup.Leader == null && !QuestingHelpers.AdvancedQuestingInAction && Setting.Instance.AdvancedUseRBC.Contains("Client") && !RAFGroup.TryLeaderLastKnownPosition && RAFBotComms.IsClientRegistered && PoiBehaviors.IsPoiFollowLeader,
                    new PrioritySelector(
                        new ThrottlePasses(1,10, new TreeSharpExt.LogMessage("Appears our leader has vanished. Lets perform a RBC query on their location")),
                        new ThrottlePasses(1,30, new Decorator(ret => Setting.Instance.AdvancedUseRBC.Contains("Client"), new Action(context => RAFBotComms.DoHeartbeat()))),
                        new Decorator(ret=> RAFBotComms.IsAlive, new Action(context =>
                        {
                            Log.Debug("query the leader for their location");
                            string result = RAFBotComms.Location;
                            if (!string.IsNullOrEmpty(result))
                            {
                                Log.Debug("we've got the leaders location via RBC. Lets move there now");
                                RAFGroup.LeaderLastKnownPosition = MapHelpers.StringToWoWPoint(result);
                                RAFGroup.TryLeaderLastKnownPosition = true;
                                Log.Debug(string.Format("our leader is {0} yards away", Me.Location.Distance(RAFGroup.LeaderLastKnownPosition)));
                            }
                        }))
                )),


                // Finally just follow the leader
                // ... but make sure we have nothing to loot/harvest/skin/kill
                new Decorator(ret => IsLeaderValid && (PoiBehaviors.IsPoiFollowLeader) && (!Setting.Instance.LootingMobs.OK() || Loot.LootableWoWUnit == null) && (!Setting.Instance.LootingSkinMobs.OK() || Loot.SkinnableWoWUnit == null) && (!Setting.Instance.LootingHarvest.OK() || Harvestable.LootableWoWGameObject == null) && (!Me.Combat || (Me.Combat && Me.Mounted)), 
                    new RAFGroup.FollowLeaderPS(ret => RAFGroup.Leader, ret=> Setting.Instance.FollowDistance, ret=> false))

                #endregion

                );
        }


        
    }
}
