﻿using System.ComponentModel;
using Fpsware.Library3.Helpers;
using Styx;
using Styx.Common;
using Styx.Helpers;

namespace Fpsware.Botbases.RAFBot2
{
    class Setting : Settings
    {
        private static Setting _instance;
        public Setting() : base(SettingsPath + ".xml") { }
        public Setting(string settingsPath) : base(settingsPath) { }
        private static string SettingsPath { get { return string.Format("{0}\\Settings\\Fpsware RAF Bot\\{1}", Utilities.AssemblyDirectory, StyxWoW.Me.Name); } }
        public static Setting Instance { get { return _instance ?? (_instance = new Setting()); } }

        // General settings

        // Automation
        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string AutomationRecurrection { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string AutomationReadyCheck { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string AutomationLFGReady { get; set; }



        // Key Bindings
        [Setting, Styx.Helpers.DefaultValue("[None]")]
        public string KeyToggleGrinder { get; set; }

        [Setting, Styx.Helpers.DefaultValue("[None]")]
        public string KeyToggleGrinderMod { get; set; }

        [Setting, Styx.Helpers.DefaultValue("[None]")]
        public string KeyToggleAdvancedQuesting { get; set; }

        [Setting, Styx.Helpers.DefaultValue("[None]")]
        public string KeyToggleAdvancedQuestingMod { get; set; }





        // Misc and advanced
        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string MiscGhostRun { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Never")]
        [Category("GhostRun")]
        public string MiscRepopBody { get; set; }


        [Setting, Styx.Helpers.DefaultValue("Common")]
        public string LastUsedTab { get; set; }

        [Setting, Styx.Helpers.DefaultValue("")]
        public string AutoDeleteItems { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string UsePlugins { get; set; }
        

        /*
[Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string AutoUpdate { get; set; }
*/

        [Setting, Styx.Helpers.DefaultValue("Never")]
        public string MiscWoWMessages { get; set; }

        [Setting, Styx.Helpers.DefaultValue("")]
        public string HomeLocation { get; set; }


        // Behaviors

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string BehaviorPreCombat { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string BehaviorRest { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string BehaviorPull { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string BehaviorHeal { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string BehaviorCombatBuff { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string ForceCombatBehavior { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string HealingBehaviorWhen { get; set; }


        // Looting

        [Setting, Styx.Helpers.DefaultValue(60)]
        public int LootRange { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string LootingMobs { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Never")]
        public string LootingSkinMobs { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Never")]
        public string LootingHarvest { get; set; }

        [Setting, Styx.Helpers.DefaultValue(60)]
        public int HarvestRange { get; set; }


        // Common

        //[Setting, DefaultValue("I just want LazyRaider / Tyrael features")]
        //public string SlaveOrMaster { get; set; }

        [Setting, Styx.Helpers.DefaultValue("I just want LazyRaider / Tyrael features")]
        public string QuickSetup { get; set; }

        [Setting, Styx.Helpers.DefaultValue("[None]")]
        public string Leader { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string AlwaysLosLeader { get; set; }

        [Setting, Styx.Helpers.DefaultValue(30), Category("Leader")]
        [Attributes.ControlType(Attributes.ControlTypeEnum.NumericUpDown), Attributes.MinMaxRange(1, 50)]
        [Description("The distance at which you will follow and move to the leader")]
        public int FollowDistance { get; set; }

        /*
        [Setting, Styx.Helpers.DefaultValue("Disabled"), Category("Leader")]
        [Attributes.DropdownItems("Enabled", "Disabled")]
        [Description("When following to the leader will move to an approximate location near then as not to stand exactly where everyone is standing. i.e random fan out")]
        public string UseCloseDestination { get; set; }
         */

        [Setting, Styx.Helpers.DefaultValue("Manual Targeting")]
        public string Targeting { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string PrioritiseSkull { get; set; }

        [Setting, Styx.Helpers.DefaultValue(35)]
        public int MoveToTargetDistance { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Non-Combat Movement Only")]
        public string Movement { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Always")]
        public string EncounterLogicTimelessIsle { get; set; }


        // QuestBehaviors

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string QuestTurnIn { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string QuestPickUp { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string QuestObjectCollection { get; set; }

        [Setting, Styx.Helpers.DefaultValue(45), Category("ObjectiveLooting"), DisplayName("Max Distance"), Attributes.ControlType(Attributes.ControlTypeEnum.NumericUpDown), Attributes.MinMaxRange(1, 150), Description("Search for valid lootable objects no further than this distance from the leader's location")]
        public int QuestObjectCollectionMaxDistance { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled"), Category("QuestTurnIn"), Attributes.DropdownItems("Enabled", "Disabled"), DisplayName("Choose Reward"), Description("Choose the most appropriate quest reward.\n\nIf no reward is an upgrade the most valuable item will be chosen for vendoring.")]
        public string QuestChooseReward { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled"), Category("QuestTurnIn"), Attributes.DropdownItems("Enabled","Disabled"), DisplayName("Auto Turn In"), Description("When in range of a quest giver automatically turn in all complete quests.\n\nDoes not require the leader to be targeting the quest giver.")]
        public string AutoQuestTurnIn { get; set; }

        [Setting, Styx.Helpers.DefaultValue(20), Category("QuestTurnIn"), DisplayName("Max Distance"), Attributes.ControlType(Attributes.ControlTypeEnum.NumericUpDown), Attributes.MinMaxRange(1,150), Description("The quest giver must be within this range of the character for auto turn in to be considered")]
        public int AutoQuestTurnInDistance { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Enabled"), Category("QuestPickUp"), Attributes.DropdownItems("Enabled", "Disabled"), DisplayName("Auto Pick Up"), Description("When in range of a quest giver automatically pick up all available quests.\n\nDoes not require the leader to be targeting the quest giver.")]
        public string AutoQuestPickUp { get; set; }

        [Setting, Styx.Helpers.DefaultValue(20), Category("QuestPickUp"), DisplayName("Max Distance"), Attributes.ControlType(Attributes.ControlTypeEnum.NumericUpDown), Attributes.MinMaxRange(1, 150), Description("The quest giver must be within this range of the character for auto pick up to be considered")]
        public int AutoQuestPickupDistance { get; set; }
        
        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string QuestObjectiveKills { get; set; }

        [Setting, Styx.Helpers.DefaultValue(45), Category("ObjectiveKills"), DisplayName("Max Distance"), Attributes.ControlType(Attributes.ControlTypeEnum.NumericUpDown), Attributes.MinMaxRange(1, 150), Description("Search for valid mobs no further than this distance from the leader's location")]
        public int QuestObjectiveKillsMaxDistance { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string QuestUseAdvancedFeatures { get; set; }


        // Grinder

        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string UseGrinder { get; set; }

        [Setting, Styx.Helpers.DefaultValue("")]
        public string GrinderMobIDs { get; set; }

        [Setting, Styx.Helpers.DefaultValue("")]
        public string GrinderHotspots { get; set; }

        [Setting, Styx.Helpers.DefaultValue(100)]
        public int GrinderMaxMobDistance { get; set; }

        [Setting, Styx.Helpers.DefaultValue("Never")]
        public string GrinderPreferFlyingMount { get; set; }

        [Setting, Styx.Helpers.DefaultValue("")]
        public string GrinderActionTriggers{ get; set; }


        // Encounter logic
     
        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string UseEncounterLogic { get; set; }


        // RAF Bot Communications
        [Setting, Styx.Helpers.DefaultValue("Disabled")]
        public string AdvancedUseRBC { get; set; }

        [Setting, Styx.Helpers.DefaultValue("localhost")]
        [Category("RBC"), DisplayName("Host Address"), Attributes.ControlType(Attributes.ControlTypeEnum.Textbox), Description("Enter the IP address of the host. If run on the same computer use the default 'localhost'")]
        public string AdvancedRBCServer { get; set; }

        [Setting, Styx.Helpers.DefaultValue("8010")]
        [Category("RBC"), DisplayName("Host Port"), Attributes.ControlType(Attributes.ControlTypeEnum.Textbox), Description("Specify the port number the host is listening on. The default is 8010.")]
        public string AdvancedRBCPort { get; set; }



        // Debugging

        // RAF Bot Communications
        [Setting, Styx.Helpers.DefaultValue("Enabled")]
        public string DebugMessages { get; set; }


    }
}
