﻿using System;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Fpsware.Library3.Utility;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace Fpsware.Botbases.RAFBot2.UI
{
    public partial class UIEncounterLogic : Form
    {
        private EncounterLogicActions.EncounterLogic _encounterLogic;

        public UIEncounterLogic()
        {
            InitializeComponent();

            // Populate the dropdowns will all the values of the appropriate enums
            foreach (object item in Enum.GetValues(typeof(EncounterLogicActions.QueryType)))
            {
                ucSource.Dropdown.Items.Add(item.ToString());
            }

            foreach (object item in Enum.GetValues(typeof(EncounterLogicActions.Action)))
            {
                ucAction.Dropdown.Items.Add(item.ToString());
            }

            foreach (object item in Enum.GetValues(typeof(EncounterLogicActions.SourceTarget)))
            {
                ucSourceTarget.Dropdown.Items.Add(item.ToString());
            }
        }

        #region Local Declarations
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return StyxWoW.Me.CurrentTarget; } }

        public EncounterLogicActions.EncounterLogic EncounterLogic
        {
            get { return _encounterLogic; }
            set
            {
                _encounterLogic = value;

                ucSource.Value = _encounterLogic.QueryType.ToString();
                ucMobID.Value = _encounterLogic.MobID.ToString();
                ucSpellName.Value = _encounterLogic.Name;
                ucSpellID.Value = _encounterLogic.SpellID.ToString();
                ucDistance.Value = _encounterLogic.EffectiveDistance;
                ucRadius.Value = _encounterLogic.Degrees;

                ucAction.Value = _encounterLogic.Action.ToString();
                ucActionItemName.Value = _encounterLogic.ActionName;
                ucActionDistance.Value = _encounterLogic.ActionDistance;
                ucSourceTarget.Value = _encounterLogic.SourceTarget.ToString();

            }
        }

        #endregion

        private void UINewEncounterLogic_Load(object sender, EventArgs e)
        {
            if (EncounterLogic != null)
            {
                btnAdd.Text = "Update";
            }
            else
            {
                ucSourceTarget.Value = "IsMyCurrentTarget";
                ucSource.Value = "SpellCast";
                ucAction.Value = "RunToDistance";
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            switch (ucSource.Value)
            {
                case "Aura":
                    sb.AppendFormat("{0} [CheckAgainst:Aura,SourceTarget:{4},SpellID:{1},MobID:{2},Distance:{3}]", ucSpellName.Value, ucSpellID.Value, ucMobID.Value, ucDistance.Value, ucSourceTarget.Value);
                    break;

                case "SpellCast":
                    sb.AppendFormat("{0} [CheckAgainst:SpellCast,SourceTarget:{5},SpellID:{1},MobID:{2},Distance:{3},Degrees:{4}]", ucSpellName.Value, ucSpellID.Value, ucMobID.Value, ucDistance.Value, ucRadius.Value, ucSourceTarget.Value);
                    break;

                case "Mob":
                    sb.AppendFormat("{0} [CheckAgainst:Mob,SourceTarget:{3},MobID:{1},Distance:{2}]", ucSpellName.Value, ucMobID.Value, ucDistance.Value, ucSourceTarget.Value);
                    break;
            }

            switch (ucAction.Value)
            {
                case "UseItem":
                case "CastSpell":
                    sb.AppendFormat(" Action[{0}:{1}]", ucAction.Value, ucActionItemName.Value);
                    break;

                case "RunToDistance":
                case "MoveToLeader":
                    sb.AppendFormat(" Action[{0}:{1}]", ucAction.Value,ucActionDistance.Value);
                    break;

                //case "MoveBehindCaster":
                //case "StopCasting":
                default:
                    sb.AppendFormat(" Action[{0}]", ucAction.Value);
                    break;

            }

            try
            {
                EncounterLogic = new EncounterLogicActions.EncounterLogic(sb.ToString());
            }
            catch (Exception exception)
            {
                Log.FailLog("There was an exception during a new Encounter Logic creation event.");
                Log.FailLog("Have you completed all the relevant fields?");
                MessageBox.Show("There was an exception during a new Encounter Logic creation event. New logic has not been created.\n\nHave you completed all relevant fields?", "Encounter Logic Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            DialogResult = DialogResult.OK;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void ucSpellName_ValueChanged(object sender, Library3.UI.TextBoxValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucSpellID_ValueChanged(object sender, Library3.UI.TextBoxValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucDistance_ValueChanged(object sender, Library3.UI.NumbericUpDownValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucRadius_ValueChanged(object sender, Library3.UI.NumbericUpDownValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucActionTaken_ValueChanged(object sender, Library3.UI.DropdownValueChangedEventArgs e)
        {
            switch (ucSource.Value)
            {
                case "Aura":
                    ucSource.TooltipTitle = "QueryType: Aura";
                    ucSource.TooltipText = "Perform the assigned action when your source target has the assigned aura";

                    ucSpellID.Enabled = true;
                    ucDistance.Enabled = true;
                    ucMobID.Enabled = true;
                    ucRadius.Enabled = false;
                    ucSpellName.Enabled = true;

                    ucSpellName.Label = "Aura Name";
                    ucSpellID.Label = "Aura ID";

                    btnCurrentObject.Text = "Get Target's Auras";
                    break;

                case "SpellCast":
                    ucSource.TooltipTitle = "QueryType: SpellCast";
                    ucSource.TooltipText = "Perform the assigned action when your source target casts the assigned spell";

                    ucSpellID.Enabled = true;
                    ucDistance.Enabled = true;
                    ucMobID.Enabled = true;
                    ucRadius.Enabled = true;
                    ucSpellName.Enabled = true;

                    ucSpellName.Label = "Spell Name";
                    ucSpellID.Label = "Spell ID";

                    btnCurrentObject.Text = "Target's Current Spell";
                    break;

                case "Mob":
                    ucSource.TooltipTitle = "QueryType: Mob";
                    ucSource.TooltipText = "Perform the assigned action when your source target is within [Distance] of you";

                    ucSpellID.Enabled = false;
                    ucDistance.Enabled = true;
                    ucMobID.Enabled = true;
                    ucRadius.Enabled = false;
                    ucSpellName.Enabled = true;

                    ucSpellName.Label = "Mob Name";
                    btnCurrentObject.Text = "Current Target";
                    break;
            }

            UpdateWhatsItDo();
        }

        private void btnCurrentSpell_Click(object sender, EventArgs e)
        {
            ObjectManager.Update();

            switch (ucSource.Value)
            {
                case "Aura":
                    if (Me.GotTarget)
                    {
                        ucMobID.Value = Me.CurrentTarget.Entry.ToString();

                        using (UITargetsAuras logicForm = new UITargetsAuras())
                        {
                            if (logicForm.ShowDialog() == DialogResult.OK)
                            {
                                string result = logicForm.SelectedItem;

                                ucSpellID.Value = Regex.Match(result, @"(?<=\[).*?(?=[\],])", RegexOptions.IgnoreCase).Value;
                                ucSpellName.Value = Regex.Match(result, @"(?<=).*?(?=[\[])", RegexOptions.IgnoreCase).Value.Trim();

                            }
                        }
                    }

                    break;

                case "SpellCast":
                    if (Me.GotTarget && CT.IsCasting)
                    {
                        ucMobID.Value = Me.CurrentTarget.Entry.ToString();
                        ucSpellName.Value = CT.CastingSpell.Name;
                        ucSpellID.Value = CT.CastingSpellId.ToString();

                        if (CT.CastingSpell.HasRange)
                        {
                            ucDistance.Value = (int)CT.CastingSpell.MaxRange;
                        }
                    }
                    break;

                

                case "Mob":
                    if (Me.GotTarget)
                    {
                        ucMobID.Value = Me.CurrentTarget.Entry.ToString();
                        ucSpellName.Value = Me.CurrentTarget.Name;
                        ucDistance.Value = 20;
                    }
                    break;
            }

            UpdateWhatsItDo();
        }

        private void ucAction_ValueChangedOther(object sender, Library3.UI.DropdownValueChangedEventArgs e)
        {
            switch (ucAction.Value)
            {
                case "RunToDistance":
                    ucActionDistance.Enabled = true;
                    ucActionItemName.Enabled = false;
                    break;

                case "StopCasting":
                    ucActionDistance.Enabled = false;
                    ucActionItemName.Enabled = false;
                    break;

                case "MoveBehindCaster":
                    ucActionDistance.Enabled = false;
                    ucActionItemName.Enabled = false;
                    break;

                case "MoveToLeader":
                    ucActionItemName.Enabled = false;
                    ucActionDistance.Enabled = true;
                    break;
                
                case "UseItem":
                    ucActionDistance.Enabled = false;
                    ucActionItemName.Enabled = true;
                    ucActionItemName.Label = "Item Name";
                    break;

                case "CastSpell":
                    ucActionDistance.Enabled = false;
                    ucActionItemName.Enabled = true;
                    ucActionItemName.Label = "Spell Name";
                    break;
            }

            UpdateWhatsItDo();
        }

        private void UpdateWhatsItDo()
        {
            StringBuilder sb = new StringBuilder();

            switch (ucSource.Value)
            {
                case "Aura":
                    sb.AppendFormat("When [SourceTarget:{3}] is [MobID:{0}] and they have an aura [Aura:{1}({2}] perform the Action ", ucMobID.Value, ucSpellName.Value, ucSpellID.Value,ucSourceTarget.Value);
                    break;

                case "SpellCast":
                    sb.AppendFormat("When [SourceTarget:{5}] is [MobID:{0}] and casting [SpellName:{1}({2})], if you're within [Degrees:{3}] field of view AND within [Distance:{4}] yards perform the Action ", ucMobID.Value, ucSpellName.Value, ucSpellID.Value, ucRadius.Value,ucDistance.Value,ucSourceTarget.Value);
                    break;

                case "Mob":
                    sb.AppendFormat("When [SourceTarget:{3}] matching [MobName: {1}({0})] is within [Distance:{2} yards] perform the Action ", ucMobID.Value, ucSpellName.Value, ucDistance.Value, ucSourceTarget.Value);
                    break;
            }

            switch (ucAction.Value)
            {
                case "UseItem":
                case "CastSpell":
                    sb.AppendFormat("[{0}:{1}]", ucAction.Value, ucActionItemName.Value);
                    break;

                case "RunToDistance":
                    sb.AppendFormat("[{0}] and move away from the mob to a distance of [Action Distance:{1}]", ucAction.Value, ucActionDistance.Value);
                    break;

                case "StopCasting":
                    sb.AppendFormat("[{0}], this will stop all offensive spell casting", ucAction.Value);
                    break;

                case "MoveBehindCaster":
                    sb.AppendFormat("[{0}]", ucAction.Value);
                    break;

                case "MoveToLeader":
                    sb.AppendFormat("[{0}] and move no less than [Action Distance:{1}] yards from them", ucAction.Value, ucActionDistance.Value);
                    break;

            }

            lblWhatsItDo.Text = sb.ToString();
        }

        private void ucMobID_ValueChanged(object sender, Library3.UI.TextBoxValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucActionDistance_ValueChanged(object sender, Library3.UI.NumbericUpDownValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucActionItemName_ValueChanged(object sender, Library3.UI.TextBoxValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }

        private void ucSourceTarget_ValueChangedOther(object sender, Library3.UI.DropdownValueChangedEventArgs e)
        {
            UpdateWhatsItDo();
        }
    }
}
