﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Fpsware.Library3.Extensions;
using Fpsware.Library3.Helpers;
using Fpsware.Library3.UI;
using Fpsware.Library3.UI.Cyotek;
using Fpsware.Library3.UI.Popup;
using Fpsware.Library3.Utility;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Misc = Fpsware.Library3.Helpers.Misc;

namespace Fpsware.Botbases.RAFBot2.UI
{
    public partial class UIForm : Form
    {
        public UIForm() { InitializeComponent(); }

        private bool _isLoading = true;
        private readonly UISettingsPopulater _uiSettings= new UISettingsPopulater();
        private readonly UISettingsPopulater _uiGlobalSettings = new UISettingsPopulater();
        private readonly SvnUpdater _svnUpdater = new SvnUpdater();

        private void btnClose_Click(object sender, EventArgs e)
        {
            Setting.Instance.Save();
            GlobalSetting.Instance.Save();
            Close();
        }

        private void UIForm_Load(object sender, EventArgs e)
        {

            Setting.Instance.Load();
            GlobalSetting.Instance.Load();

            // Populate the UI with appropriate INDIVIDUAL USER settings
            _uiSettings.SettingType = typeof (Setting);
            _uiSettings.SettingInstance = Setting.Instance;
            _uiSettings.HostControl = tabList1;
            _uiSettings.Go();

            // Populate the UI with appropriate GLOBAL settings
            _uiGlobalSettings.SettingType = typeof(GlobalSetting);
            _uiGlobalSettings.SettingInstance = GlobalSetting.Instance;
            _uiGlobalSettings.HostControl = tabList1;
            _uiGlobalSettings.Go();

            // SVN updater
            _svnUpdater.Repository = SvnUpdater.RepositoryType.Assembla;
            _svnUpdater.SVNProject = "fpsware-raf-bot";
            _svnUpdater.DownloadPath = Misc.GetBotPath();
            _svnUpdater.NameOfDownload = "Fpsware RAF Bot";

            // Run the updater if enabled
            try
            {
                if (GlobalSetting.Instance.AutoUpdate.OK() && !Query.IsThisMyComputer("Mike-PC|Mike-Laptop"))
                    _svnUpdater.AutoUpdate();
            }
            catch (Exception exception) { }

            // Toggle the Grinder warning label
            lblGrinderWarning.Visible = Setting.Instance.UseGrinder.OK();
            lblOnlineUpdate.Visible = GlobalSetting.Instance.AutoUpdate.OK() && Query.IsThisMyComputer("Mike-PC|Mike-Laptop");

            // Populate the leader dropdown
            try
            {
                PopulateDynamicLeader();
            }
            catch (Exception exception) { }

            //Make things loot pretty and select the first item in the list.
            try
            {
                ucGrinderHotspots.SelectFirstItem();
                ucGrinderMobs.SelectFirstItem();
                ucEncounterLogic.SelectFirstItem();
                ucAutoDelete.SelectFirstItem();
                ucActionTriggers.SelectFirstItem();

            }
            catch (Exception exception) { }

            // Select the last used tab
            try
            {
                if (!string.IsNullOrEmpty(Setting.Instance.LastUsedTab))
                {
                    foreach (TabListPage page in tabList1.TabListPages.Cast<TabListPage>().Where(page => page.Text == Setting.Instance.LastUsedTab))
                    {
                        tabList1.SelectedPage = page;
                        break;
                    }
                }

                ucEncounterLogic.TooltipText = ucEncounterLogic.SelectedItem.ToString();
            }
            catch (Exception exception) {  }



            string downloadedString;
            WebClient client;
            client = new WebClient();
            downloadedString = client.DownloadString("http://www.wowhead.com/item=57787");
            //Log.Info(downloadedString);

            


            /*
            string URL = "http://www.wowhead.com/item=57787";
            HttpWebRequest myRequest = (HttpWebRequest)WebRequest.Create(URL);
            myRequest.Method = "GET";
            WebResponse myResponse = myRequest.GetResponse();
            StreamReader sr = new StreamReader(myResponse.GetResponseStream(), System.Text.Encoding.UTF8);
            string result = sr.ReadToEnd();
            sr.Close();
            myResponse.Close();

            Log.Info(result);
             */







            _isLoading = false;
        }

        private void PopulateDynamicLeader()
        {
            // populate our dynamic leader dropdown list
            ucLeader.Dropdown.Items.Clear();
            ucLeader.Dropdown.Items.Add("[None]");
            ucLeader.Dropdown.Items.Add("[Automatic]");

            try
            {
                IOrderedEnumerable<WoWPlayer> units = ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().Where(u => u.IsAlive && u.IsFriendly && u.Distance < 100 && u.IsPlayer).OrderBy(u => u.Name);
                foreach (WoWPlayer unit in units.Where(unit => unit.Guid != StyxWoW.Me.Guid && unit.Name.ToUpper() != "UNKNOWN"))
                {
                    ucLeader.Dropdown.Items.Add(unit.Name);
                }

                ucLeader.Dropdown.SelectedItem = Setting.Instance.Leader;
                if (ucLeader.Dropdown.SelectedItem == null || (string)ucLeader.Dropdown.SelectedItem == "")
                {
                    ucLeader.Value = "[Automatic]";
                }
            }
            catch (Exception e) { }

        }

        private void fpsDropdown9_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            lblPullBehavior.Visible = (Setting.Instance.BehaviorPull.OK());
            lblPullLeaderWarning.Visible = (Setting.Instance.BehaviorPull.Contains("Pull Leaders Target"));
        }

        private void fpsDropdown2_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {

        }

        private void fpsDropdown15_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            lblFullCombatMovementWarning.Visible = (Setting.Instance.Movement.Contains("Full Combat Movement"));
            label5.Visible = (!Setting.Instance.Movement.Contains("Full Combat Movement"));
        }

        private void ucGrinder_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            lblGrinderWarning.Visible = Setting.Instance.UseGrinder.OK();
        }

        private void fpsDropdown14_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            
        }

        private void splitGetBehindSpells_Click(object sender, EventArgs e)
        {
            using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                logicForm.StartPosition = FormStartPosition.CenterParent;
                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogic.Dropdown.Items.Add(logicForm.EncounterLogic.EncounterLogicRawString);
                    GlobalSetting.Instance.EncounterLogic = ucEncounterLogic.Value;
                    ucEncounterLogic.Dropdown.SelectedItem = logicForm.EncounterLogic.EncounterLogicRawString;
                }
            }
        }

        private void editSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
           using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                logicForm.StartPosition = FormStartPosition.CenterParent;
                EncounterLogicActions.EncounterLogic encounter = new EncounterLogicActions.EncounterLogic(ucEncounterLogic.Dropdown.SelectedItem.ToString());
                logicForm.EncounterLogic = encounter;

                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogic.ReplaceSelectedItem(logicForm.EncounterLogic.EncounterLogicRawString);
                }
            }
        }

        private void splitHotspots_Click(object sender, EventArgs e)
        {
            if (ucGrinderHotspots.Dropdown.Items.Contains(StyxWoW.Me.Location.ToString()))
            {
                Log.Info("This hotspot is already present");
                return;
            }
            ucGrinderHotspots.Dropdown.Items.Add(StyxWoW.Me.Location.ToString());
            Setting.Instance.GrinderHotspots = ucGrinderHotspots.Value;

            ucGrinderHotspots.Dropdown.SelectedItem = StyxWoW.Me.Location.ToString();
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(ucGrinderHotspots.SelectedItem))
            {
                ucGrinderHotspots.RemoveSelectedItem();
            }

            Setting.Instance.GrinderHotspots = ucGrinderHotspots.Value;
            ucGrinderHotspots.Dropdown.SelectedIndex = ucGrinderHotspots.Dropdown.Items.Count - 1;
        }

        private void deleteAllToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ucGrinderHotspots.ClearAll();
            ucGrinderHotspots.Dropdown.Items.Clear();
            Setting.Instance.GrinderHotspots = "";
        }

        private void navigateToSelectedHotspotToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Log.Info("=== this is our string: " + ucGrinderHotspots.Value);
            //string wowpoint = ucGrinderHotspots.Value;
            WoWPoint navigatePoint = MapHelpers.StringToWoWPoint(ucGrinderHotspots.Value);
            
            RAFBot2.GrinderNavigatePoint = navigatePoint;
            RAFBot2.GrinderNavigatePointMoveToTest = true;
        }

        private void toolStripMenuItem3_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(ucEncounterLogic.SelectedItem))
            {
                ucEncounterLogic.RemoveSelectedItem();
            }

            GlobalSetting.Instance.EncounterLogic = ucEncounterLogic.Value;
            ucEncounterLogic.Dropdown.SelectedIndex = ucEncounterLogic.Dropdown.Items.Count - 1;
        }

        private void splitRunFromSpells_Click(object sender, EventArgs e)
        {
            using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                /*
                logicForm.DefaultEncounterAction = EncounterActionTaken.RunFromSpell;
                logicForm.StartPosition = FormStartPosition.CenterParent;
                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogicRunFromSpells.Dropdown.Items.Add(logicForm.EncounterLogic.EncounterLogicRawString);
                    GlobalSetting.Instance.EncounterLogicRunFromSpells = ucEncounterLogicRunFromSpells.Value;

                    ucEncounterLogicRunFromSpells.Dropdown.SelectedItem = logicForm.EncounterLogic.EncounterLogicRawString;
                }
                 */
            }
        }

        private void splitRunFromMob_Click(object sender, EventArgs e)
        {
            using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                /*
logicForm.RunFromMobLogic = true;
                logicForm.DefaultEncounterAction = EncounterActionTaken.AvoidMob;
                logicForm.StartPosition = FormStartPosition.CenterParent;
                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogicRunFromMobs.Dropdown.Items.Add(logicForm.EncounterLogic.EncounterLogicRawString);
                    GlobalSetting.Instance.EncounterLogicRunFromMobs = ucEncounterLogicRunFromMobs.Value;

                    ucEncounterLogicRunFromMobs.Dropdown.SelectedItem = logicForm.EncounterLogic.EncounterLogicRawString;
                }
*/
            }
        }

        private void toolStripMenuItem11_Click(object sender, EventArgs e)
        {
            /*
using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                logicForm.DefaultEncounterAction = EncounterActionTaken.AvoidMob;
                logicForm.StartPosition = FormStartPosition.CenterParent;
                EncounterLogic encounter = new EncounterLogic(ucEncounterLogicRunFromMobs.Dropdown.SelectedItem + " Action:" + (int)EncounterActionTaken.AvoidMob);
                logicForm.EncounterLogic = encounter;

                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogicRunFromMobs.ReplaceSelectedItem(logicForm.EncounterLogic.EncounterLogicRawString);
                }
            }
*/
        }

        private void toolStripMenuItem8_Click(object sender, EventArgs e)
        {
            using (UIEncounterLogic logicForm = new UIEncounterLogic())
            {
                /*
 logicForm.DefaultEncounterAction = EncounterActionTaken.RunFromSpell;
                logicForm.StartPosition = FormStartPosition.CenterParent;
                EncounterLogic encounter = new EncounterLogic(ucEncounterLogicRunFromSpells.Dropdown.SelectedItem + " Action:" + (int)EncounterActionTaken.RunFromSpell);
                logicForm.EncounterLogic = encounter;

                if (logicForm.ShowDialog() == DialogResult.OK)
                {
                    ucEncounterLogicRunFromSpells.ReplaceSelectedItem(logicForm.EncounterLogic.EncounterLogicRawString);
                }
*/
            }
        }



        private void toolStripMenuItem4_Click(object sender, EventArgs e)
        {
            ucEncounterLogic.ClearAll();
            ucEncounterLogic.Dropdown.Items.Clear(); ;
            GlobalSetting.Instance.EncounterLogic = "";
        }


        private void ctxMnuHotspots_Opened(object sender, EventArgs e)
        {
            /*
            navigateToSelectedHotspotToolStripMenuItem.Enabled = !(String.IsNullOrEmpty(ucGrinderHotspots.SelectedItem));
            toolStripMenuItem1.Enabled = !(String.IsNullOrEmpty(ucGrinderHotspots.SelectedItem));
             */
        }

        private void deleteAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //ucEncounterLogicRunFromMobs.ClearAll();
            ucGrinderMobs.ClearAll();
            ucGrinderMobs.Dropdown.Items.Clear(); 
            Setting.Instance.GrinderMobIDs = "";
        }

        private void option1ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(ucGrinderMobs.SelectedItem))
            {
                ucGrinderMobs.RemoveSelectedItem();
            }

            Setting.Instance.GrinderMobIDs= ucGrinderMobs.Value;
            ucGrinderMobs.Dropdown.SelectedIndex = ucGrinderMobs.Dropdown.Items.Count - 1;
        }

        private void splitMobs_Click(object sender, EventArgs e)
        {
            if (StyxWoW.Me.GotValidTarget())
            {
                if (GrinderHelpers.GrinderMobsList().Any(mob => mob.ID == StyxWoW.Me.CurrentTarget.Entry))
                {
                    Log.Info("Mob entry ID already exists");
                    return;
                }

                GrinderHelpers.GrinderMob grinderMob = new GrinderHelpers.GrinderMob
                {
                    Name = StyxWoW.Me.CurrentTarget.Name,
                    ID = StyxWoW.Me.CurrentTarget.Entry
                };

                ucGrinderMobs.Dropdown.Items.Add(grinderMob.GrinderMobRawString);
                Setting.Instance.GrinderMobIDs = ucGrinderMobs.Value;
                ucGrinderMobs.Dropdown.SelectedItem = grinderMob.GrinderMobRawString;
            }
            else
            {
                Log.Info("You must select a valid combat target");
            }
        }

        private void toolStripMenuItem12_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(ucAutoDelete.SelectedItem))
            {
                ucAutoDelete.RemoveSelectedItem();
            }

            Setting.Instance.AutoDeleteItems = ucAutoDelete.Value;
            ucAutoDelete.Dropdown.SelectedIndex = ucAutoDelete.Dropdown.Items.Count - 1;
        }

        private void toolStripMenuItem13_Click(object sender, EventArgs e)
        {
            ucAutoDelete.Dropdown.Items.Clear();
            Setting.Instance.AutoDeleteItems = "";
        }

        private void splitAutoDelete_Click_1(object sender, EventArgs e)
        {
            using (UIInventoryItems form = new UIInventoryItems())
            {
                form.StartPosition = FormStartPosition.CenterParent;

                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (!string.IsNullOrEmpty(form.SelectedInventoryItem))
                    {
                        if (!form.SelectedInventoryItem.Contains("|"))
                        {
                            // Single item selected
                            ucAutoDelete.Dropdown.Items.Add(form.SelectedInventoryItem);
                            ucAutoDelete.Dropdown.SelectedItem = form.SelectedInventoryItem;
                        }
                        else
                        {
                            // Multiple items selected
                            string[] items = form.SelectedInventoryItem.Split('|');
                            foreach (string item in items)
                            {
                                ucAutoDelete.Dropdown.Items.Add(item);
                            }
                        }
                    }

                    Setting.Instance.AutoDeleteItems = ucAutoDelete.Value;

                }
            }
        }

        private void timerCorrectToggles_Tick(object sender, EventArgs e)
        {
            // Grinder was toggled via key press. Update the UI to reflect the matching setting
            if (Setting.Instance.UseGrinder.OK() && ucGrinder.Value == "Disabled" || !Setting.Instance.UseGrinder.OK() && ucGrinder.Value == "Enabled")
            {
                ucGrinder.Value = (Setting.Instance.UseGrinder.OK() ? "Enabled" : "Disabled");
            }
        }

        private void splitButton1_Click(object sender, EventArgs e)
        {

            using (UITrigger form = new UITrigger())
            {
                //logicForm.ActionTrigger2 .DefaultEncounterAction = EncounterActionTaken.RunFromSpell;
                form.StartPosition = FormStartPosition.CenterParent;
                if (form.ShowDialog() == DialogResult.OK)
                {
                    ucActionTriggers.Dropdown.Items.Add(form.ActionTrigger);
                    Setting.Instance.GrinderActionTriggers = ucActionTriggers.Value;

                    ucActionTriggers.Dropdown.SelectedItem = form.ActionTrigger;
                }
            }

        }

        private void toolStripMenuItem14_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(ucActionTriggers.SelectedItem))
            {
                ucActionTriggers.RemoveSelectedItem();
            }

            Setting.Instance.GrinderActionTriggers = ucActionTriggers.Value;
            ucActionTriggers.Dropdown.SelectedIndex = ucActionTriggers.Dropdown.Items.Count - 1;
        }

        private void toolStripMenuItem15_Click(object sender, EventArgs e)
        {
            ucActionTriggers.ClearAll();
            ucActionTriggers.Dropdown.Items.Clear();
            Setting.Instance.GrinderActionTriggers = "";
        }

        private void editSelectedToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            using (UITrigger form = new UITrigger())
            {
                form.StartPosition = FormStartPosition.CenterParent;
                form.ActionTrigger = ucActionTriggers.Dropdown.SelectedItem.ToString();

                if (form.ShowDialog() == DialogResult.OK)
                {
                    ucActionTriggers.ReplaceSelectedItem(form.ActionTrigger);

                }
            }
        }

        private void btnSetHomeLocation_Click_1(object sender, EventArgs e)
        {
            Setting.Instance.HomeLocation = StyxWoW.Me.Location.ToString();
            ucHomeLocation.Value = Setting.Instance.HomeLocation;
            RAFBot2.HomeLocation = MapHelpers.StringToWoWPoint(Setting.Instance.HomeLocation);
        }

        private void tabList1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Setting.Instance.LastUsedTab = tabList1.SelectedPage.Text;
            //Log.Info("=== tab page is: " + Setting.Instance.LastUsedTab);
        }

        private void exportLogicToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void importLogicToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void ucEncounterLogic_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            ucEncounterLogic.TooltipText = ucEncounterLogic.SelectedItem.ToString();
        }

        private void ucQuestTurnInQuests_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("QuestTurnIn");

            // Add some controls
            //uiAdditionals.AddDropDown("Mantain LoS", "AlwaysLOSLeader", new List<string> { "Always", "Never" }, "Leader", "Maintain line of sight on the leader at all times");
            //uiAdditionals.AddNumericUpdown("Max Follow Distance", "FollowDistance", 1, 50, "Maximum Follow Distance", "If the leader is further away than this the character will move closer. Only applicable when out of combat");

            // show it
            popup.Show(sender as FPSDropdown);
        }

        private void ucLeader_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("Leader");
            popup.Show(sender as FPSDropdown);
        }

        private void ucMiscGhostRun_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("GhostRun");
            popup.Show(sender as FPSDropdown);
        }

        private void ucQuestPickup_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("QuestPickUp");
            popup.Show(sender as FPSDropdown);
        }

        private void ucQuestAdvancedFeatures_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            ucQuestCollectObjects.Enabled = Setting.Instance.QuestUseAdvancedFeatures.OK();
            ucQuestKillObjectiveTarget.Enabled = Setting.Instance.QuestUseAdvancedFeatures.OK();

            lblObjectCollection.Visible = Setting.Instance.QuestUseAdvancedFeatures.OK();
        }

        private void fpsDropdown5_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("RBC");
            popup.Show(sender as FPSDropdown);
        }

        private void ucQuestKillObjectiveTarget_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("ObjectiveKills");
            popup.Show(sender as FPSDropdown);
        }

        private void ucLeader_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {
            if (Setting.Instance.Leader != "[None]" && Setting.Instance.Leader != "[Automatic]")
            {
                RAFGroup.PreferedLeaderName = Setting.Instance.Leader;
            }
        }

        private void ucQuickSetup_ValueChangedOther(object sender, DropdownValueChangedEventArgs e)
        {

        }

        private void ucQuestCollectObjects_AdditionalsButtonPressed(object sender, AddingNewEventArgs e)
        {
            UIAdditionals uiAdditionals = new UIAdditionals { SettingInstance = Setting.Instance, SettingType = typeof(Setting) };
            Popup popup = new Popup(uiAdditionals) { Resizable = false, DropShadowEnabled = true, ShowingAnimation = PopupAnimations.Center };

            uiAdditionals.AddCategory("ObjectiveLooting");
            popup.Show(sender as FPSDropdown);
        }

    }
}
